
(function ($, window, ko, crossroads, hasher) {
	
	'use strict';
	
	var
		/**
		 * @type {Object}
		 */
		Consts = {},
	
		/**
		 * @type {Object}
		 */
		Enums = {},
	
		/**
		 * @type {Object.<Function>}
		 */
		Utils = {},
	
		/**
		 * @type {Object.<Function>}
		 */
		Validator = {},
	
		/**
		 * @type {Object}
		 */
		I18n = window.pSevenI18N || {},
	
		/**
		 * @type {CApp|Object}
		 */
		App = {},
		
		/**
		 * @type {Object.<Function>}
		 */
		AfterLogicApi = {},
	
		/**
		 * @type {AjaxAppDataResponse|Object}
		 */
		AppData = window.pSevenAppData || {},
	
		/**
		 * @type {boolean}
		 */
		bExtApp = false,
	
		$html = $('html'),
	
		/**
		 * @type {boolean}
		 */
		bIsIosDevice = -1 < navigator.userAgent.indexOf('iPhone') ||
			-1 < navigator.userAgent.indexOf('iPod') ||
			-1 < navigator.userAgent.indexOf('iPad'),
	
		/**
		 * @type {boolean}
		 */
		bIsAndroidDevice = -1 < navigator.userAgent.toLowerCase().indexOf('android'),
	
		/**
		 * @type {boolean}
		 */
		bMobileDevice = bIsIosDevice || bIsAndroidDevice,
	
		aViewMimeTypes = [
			'image/jpeg', 'image/png', 'image/gif',
			'text/plain', 'text/css',
			'text/rfc822-headers', 'message/delivery-status',
			'application/x-httpd-php', 'application/javascript'
		]
	;
	
	if (window.Modernizr && navigator)
	{
		// v = 15;
		window.Modernizr.addTest('pdf', function(){
			var mimes = navigator.mimeTypes;
			for (var i = 0, i_len = mimes.length; i < i_len; i++) {
				if (mimes[i].type === 'application/pdf') {
					return true;
				}
			}
			return false;
		});
	}
	
	bExtApp = true;
	
	
	/**
	 * @constructor
	 */
	function CBrowser()
	{
		this.ie = /msie/.test(navigator.userAgent.toLowerCase()) && !window.opera;
		this.ie8AndBelow = this.ie && this.getIeVersion() <= 8;
		this.ie9AndBelow = this.ie && this.getIeVersion() <= 9;
		this.opera = !!window.opera;
		this.firefox = /firefox/.test(navigator.userAgent.toLowerCase());
		this.chrome = /chrome/.test(navigator.userAgent.toLowerCase());
	}
	
	CBrowser.prototype.getIeVersion = function ()
	{
		var
			sUa = navigator.userAgent.toLowerCase(),
			iVersion = Utils.pInt(sUa.slice(sUa.indexOf('msie') + 4, sUa.indexOf(';', sUa.indexOf('msie') + 4)))
		;
		
		return iVersion;
	};
	
	
	/**
	 * @constructor
	 */
	function CAjax()
	{
		this.sUrl = '?/Ajax/';
		this.aRequests = [];
		
		this.hasOpenedRequests = false;
		
		$(document).ajaxStart(_.bind(function () {
			this.hasOpenedRequests = true;
		}, this));
		
		$(document).ajaxStop(_.bind(function () {
			this.hasOpenedRequests = false;
		}, this));
	}
	
	/**
	 * @param {string} sAction
	 */
	CAjax.prototype.isAllowedActionWithoutAuth = function (sAction)
	{
		var aActionsWithoutAuth = ['Login', 'LoginLanguageUpdate', 'Logout', 'AccountCreate'];
		
		return _.indexOf(aActionsWithoutAuth, sAction) !== -1;
	};
	
	CAjax.prototype.isAllowedExtAction = function (sAction)
	{
		return sAction === 'HelpdeskRegister' || sAction === 'HelpdeskForgot' || sAction === 'HelpdeskLogin' || sAction === 'Logout';
	};
	
	/**
	 * @param {Object} oParameters
	 * @param {Function=} fResponseHandler
	 * @param {Object=} oContext
	 * @param {Function=} fDone
	 */
	CAjax.prototype.doSend = function (oParameters, fResponseHandler, oContext, fDone)
	{
		var
			doneFunc = _.bind((fDone || null), this, oParameters, fResponseHandler, oContext),
			failFunc = _.bind(this.fail, this, oParameters, fResponseHandler, oContext),
			alwaysFunc = _.bind(this.always, this, oParameters),
			oXhr = null
		;
		
		if (AfterLogicApi.runPluginHook)
		{
			AfterLogicApi.runPluginHook('ajax-default-request', [oParameters.Action, oParameters]);
		}
		
		if (AppData.Token)
		{
			oParameters.Token = AppData.Token;
		}
	
		this.abortRequests(oParameters);
	
		oXhr = $.ajax({
			url: this.sUrl,
			type: 'POST',
			async: true,
			dataType: 'json',
			data: oParameters,
			success: doneFunc,
			error: failFunc,
			complete: alwaysFunc
		});
	
		this.aRequests.push({Parameters: oParameters, Xhr: oXhr});
	};
	
	/**
	 * @param {Object} oParameters
	 * @param {Function=} fResponseHandler
	 * @param {Object=} oContext
	 */
	CAjax.prototype.send = function (oParameters, fResponseHandler, oContext)
	{
		var
			bCurrentAccountId = oParameters.AccountID === undefined,
			bAccountExists = bCurrentAccountId || AppData.Accounts.hasAccountWithId(oParameters.AccountID)
		;
		
		if (oParameters && (AppData.Auth && bAccountExists || this.isAllowedActionWithoutAuth(oParameters.Action)))
		{
			if (bCurrentAccountId && oParameters.Action !== 'Login')
			{
				oParameters.AccountID = AppData.Accounts.currentId();
			}
			
			this.doSend(oParameters, fResponseHandler, oContext, this.done);
		}
	};
	
	/**
	 * @param {Object} oParameters
	 * @param {Function=} fResponseHandler
	 * @param {Object=} oContext
	 */
	CAjax.prototype.sendExt = function (oParameters, fResponseHandler, oContext)
	{	
		var
			aActionsWithoutAuth = [
				'HelpdeskRegister', 
				'HelpdeskForgot', 
				'HelpdeskLogin',
				'HelpdeskForgotChangePassword',
				'Logout',
				'CalendarList',
				'EventList'
			],
			bAllowWithoutAuth = _.indexOf(aActionsWithoutAuth, oParameters.Action) !== -1
		;
		
		if (oParameters && (AppData.Auth || bAllowWithoutAuth))
		{
			if (AppData.TenantHash)
			{
				oParameters.TenantHash = AppData.TenantHash;
			}
			
			this.doSend(oParameters, fResponseHandler, oContext, this.doneExt);
		}
	};
	
	/**
	 * @param {Object} oParameters
	 */
	CAjax.prototype.abortRequests = function (oParameters)
	{
		switch (oParameters.Action)
		{
			case 'MessageMove':
			case 'MessageDelete':
				this.abortRequestByActionName('MessageList', oParameters.Folder);
				this.abortRequestByActionName('Message');
				break;
			case 'MessageList':
			case 'MessageSetSeen':
				this.abortRequestByActionName('MessageList', oParameters.Folder);
				break;
			case 'FolderClear':
				this.abortRequestByActionName('MessageList', oParameters.Folder);
				
				// FolderCounts-request aborted during folder cleaning, not to get the wrong information.
				this.abortRequestByActionName('FolderCounts');
				break;
			case 'ContactList':
			case 'GlobalContactList':
				this.abortRequestByActionName('ContactList');
				this.abortRequestByActionName('GlobalContactList');
				break;
			case 'Contact':
			case 'GlobalContact':
				this.abortRequestByActionName('Contact');
				this.abortRequestByActionName('GlobalContact');
				break;
		}
	};
	
	/**
	 * @param {string} sAction
	 * @param {string=} sFolder
	 */
	CAjax.prototype.abortRequestByActionName = function (sAction, sFolder)
	{
		_.each(this.aRequests, function (oReq, iIndex) {
			if (oReq && oReq.Parameters.Action === sAction && 
				(sAction !== 'MessageList' || sFolder === oReq.Parameters.Folder))
			{
				oReq.Xhr.abort();
				this.aRequests[iIndex] = undefined;
			}
		}, this);
		
		this.aRequests = _.compact(this.aRequests);
	};
	
	CAjax.prototype.abortAllRequests = function ()
	{
		_.each(this.aRequests, function (oReq) {
			if (oReq)
			{
				oReq.Xhr.abort();
			}
		}, this);
		
		this.aRequests = [];
	};
	
	/**
	 * @param {Object} oParameters
	 * @param {Function} fResponseHandler
	 * @param {Object} oContext
	 * @param {{Result:boolean}} oData
	 * @param {string} sType
	 * @param {Object} oXhr
	 */
	CAjax.prototype.done = function (oParameters, fResponseHandler, oContext, oData, sType, oXhr)
	{
		var
			bAllowedActionWithoutAuth = this.isAllowedActionWithoutAuth(oParameters.Action),
			bAccountExists = AppData.Accounts.hasAccountWithId(oParameters.AccountID),
			bDefaultAccount = (oParameters.AccountID === AppData.Accounts.defaultId())
		;
		
		if (bAllowedActionWithoutAuth || bAccountExists)
		{
			if (oData && !oData.Result)
			{
				switch (oData.ErrorCode)
				{
					case Enums.Errors.InvalidToken:
						if (!bAllowedActionWithoutAuth)
						{
							App.tokenProblem();
						}
						break;
					case Enums.Errors.AuthError:
						if (bDefaultAccount && !bAllowedActionWithoutAuth)
						{
							this.abortAllRequests();
							App.authProblem();
						}
						break;
				}
			}
	
			this.executeResponseHandler(fResponseHandler, oContext, oData, oParameters);
		}
	};
	
	/**
	 * @param {Object} oParameters
	 * @param {Function} fResponseHandler
	 * @param {Object} oContext
	 * @param {{Result:boolean}} oData
	 * @param {string} sType
	 * @param {Object} oXhr
	 */
	CAjax.prototype.doneExt = function (oParameters, fResponseHandler, oContext, oData, sType, oXhr)
	{
		this.executeResponseHandler(fResponseHandler, oContext, oData, oParameters);
	};
	
	/**
	 * @param {Object} oParameters
	 * @param {Function} fResponseHandler
	 * @param {Object} oContext
	 * @param {Object} oXhr
	 * @param {string} sType
	 * @param {string} sErrorText
	 */
	CAjax.prototype.fail = function (oParameters, fResponseHandler, oContext, oXhr, sType, sErrorText)
	{
		var oData = {'Result': false, 'ErrorCode': 0};
		
		switch (sType)
		{
			case 'abort':
				oData = {'Result': false, 'ErrorCode': Enums.Errors.NotDisplayedError};
				break;
			default:
			case 'error':
			case 'parseerror':
				if (sErrorText === '')
				{
					oData = {'Result': false, 'ErrorCode': Enums.Errors.NotDisplayedError};
				}
				else
				{
					oData = {'Result': false, 'ErrorCode': Enums.Errors.DataTransferFailed};
				}
				break;
		}
		
		this.executeResponseHandler(fResponseHandler, oContext, oData, oParameters);
	};
	
	/**
	 * @param {Function} fResponseHandler
	 * @param {Object} oContext
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CAjax.prototype.executeResponseHandler = function (fResponseHandler, oContext, oData, oParameters)
	{
		if (!oData)
		{
			oData = {'Result': false, 'ErrorCode': 0};
		}
		
		if (AfterLogicApi.runPluginHook)
		{
			AfterLogicApi.runPluginHook('ajax-default-response', [oParameters.Action, oData]);
		}
		
		if (typeof fResponseHandler === 'function')
		{
			fResponseHandler.apply(oContext, [oData, oParameters]);
		}
	};
	
	/**
	 * @param {Object} oXhr
	 * @param {string} sType
	 * @param {{Action:string}} oParameters
	 */
	CAjax.prototype.always = function (oParameters, oXhr, sType)
	{
		_.each(this.aRequests, function (oReq, iIndex) {
			if (oReq && _.isEqual(oReq.Parameters, oParameters))
			{
				this.aRequests[iIndex] = undefined;
			}
		}, this);
		
		this.aRequests = _.compact(this.aRequests);
	};
	
	
	
	/**
	 * @enum {string}
	 */
	Enums.Screens = {
		'Login': 'login',
		'Information': 'information',
		'Header': 'header',
		'Mailbox': 'mailbox',
		'BottomMailbox': 'bottommailbox',
		'SingleMessageView': 'single-message-view',
		'Compose': 'compose',
		'SingleCompose': 'single-compose',
		'Settings': 'settings',
		'Contacts': 'contacts',
		'Calendar': 'calendar',
		'FileStorage': 'files',
		'Helpdesk': 'helpdesk',
		'SingleHelpdesk': 'single-helpdesk'
	};
	
	/**
	 * @enum {number}
	 */
	Enums.MailboxLayout = {
		'Side': 0,
		'Bottom': 1
	};
	
	/**
	 * @enum {number}
	 */
	Enums.CalendarDefaultTab = {
		'Day': 1,
		'Week': 2,
		'Month': 3
	};
	
	/**
	 * @enum {number}
	 */
	Enums.TimeFormat = {
		'F24': 0,
		'F12': 1
	};
	
	/**
	 * @enum {number}
	 */
	Enums.Errors = {
		'InvalidToken': 101,
		'AuthError': 102,
		'DataBaseError': 104,
		'LicenseProblem': 105,
		'DemoLimitations': 106,
		'Recaptcha': 107,
		'AccessDenied': 108,
		'CanNotGetMessage': 202,
		'ImapQuota': 205,
		'NotSavedInSentItems': 304,
		'CanNotChangePassword': 502,
		'AccountOldPasswordNotCorrect': 503,
		'FetcherIncServerNotAvailable': 702,
		'FetcherLoginNotCorrect': 703,
		'HelpdeskThrowInWebmail': 805,
		'HelpdeskUserNotExists': 807,
		'HelpdeskUserNotActivated': 808,
		'MailServerError': 901,
		'DataTransferFailed': 1100,
		'NotDisplayedError': 1155
	};
	
	/**
	 * @enum {number}
	 */
	Enums.FolderTypes = {
		'Inbox': 1,
		'Sent': 2,
		'Drafts': 3,
		'Spam': 4,
		'Trash': 5,
		'Virus': 6,
		'Starred': 7,
		'System': 9,
		'User': 10
	};
	
	/**
	 * @enum {string}
	 */
	Enums.FolderFilter = {
		'Flagged': 'flagged'
	};
	
	/**
	 * @enum {number}
	 */
	Enums.LoginFormType = {
		'Email': 0,
		'Login': 3,
		'Both': 4
	};
	
	/**
	 * @enum {number}
	 */
	Enums.LoginSignMeType = {
		'DefaultOff': 0,
		'DefaultOn': 1,
		'Unuse': 2
	};
	
	/**
	 * @enum {string}
	 */
	Enums.ReplyType = {
		'Reply': 'reply',
		'ReplyAll': 'reply-all',
		'Forward': 'forward'
	};
	
	/**
	 * @enum {number}
	 */
	Enums.Importance = {
		'Low': 5,
		'Normal': 3,
		'High': 1
	};
	
	/**
	 * @enum {number}
	 */
	Enums.Sensivity = {
		'Nothing': 0,
		'Confidential': 1,
		'Private': 2,
		'Personal': 3
	};
	
	/**
	 * @enum {string}
	 */
	Enums.ContactEmailType = {
		'Personal': 'Personal',
		'Business': 'Business',
		'Other': 'Other'
	};
	
	/**
	 * @enum {string}
	 */
	Enums.ContactPhoneType = {
		'Mobile': 'Mobile',
		'Personal': 'Personal',
		'Business': 'Business'
	};
	
	/**
	 * @enum {string}
	 */
	Enums.ContactAddressType = {
		'Personal': 'Personal',
		'Business': 'Business'
	};
	
	/**
	 * @enum {string}
	 */
	Enums.ContactSortType = {
		'Email': 'Email',
		'Name': 'Name',
		'Frequency': 'Frequency'
	};
	
	/**
	 * @enum {number}
	 */
	Enums.SaveMail = {
		'Hidden': 0,
		'Checked': 1,
		'Unchecked': 2
	};
	
	/**
	 * @enum {string}
	 */
	Enums.SettingsTab = {
		'Common': 'common',
		'EmailAccounts': 'accounts',
		'Calendar': 'calendar',
		'MobileSync': 'mobile_sync',
		'OutLookSync': 'outlook_sync',
		'Helpdesk': 'helpdesk'
	};
	
	/**
	 * @enum {string}
	 */
	Enums.AccountSettingsTab = {
		'Properties': 'properties',
		'Signature': 'signature',
		'Filters': 'filters',
		'Autoresponder': 'autoresponder',
		'Forward': 'forward',
		'Folders': 'folders',
		'FetcherInc': 'fetcher-inc',
		'FetcherOut': 'fetcher-out',
		'FetcherSig': 'fetcher-sig'
	};
	/**
	 * @enum {number}
	 */
	Enums.ContactsGroupListType = {
		'Personal': 0,
		'SubGroup': 1,
		'Global': 2
	};
	
	/**
	 * @enum {string}
	 */
	Enums.IcalType = {
		Request: 'REQUEST',
		Reply: 'REPLY',
		Cancel: 'CANCEL',
		Save: 'SAVE'
	};
	
	/**
	 * @enum {string}
	 */
	Enums.IcalConfig = {
		Accepted: 'ACCEPTED',
		Declined: 'DECLINED',
		Tentative: 'TENTATIVE',
		NeedsAction: 'NEEDS-ACTION'
	};
	
	/**
	 * @enum {number}
	 */
	Enums.IcalConfigInt = {
		Accepted: 1,
		Declined: 2,
		Tentative: 3,
		NeedsAction: 0
	};
	
	/**
	 * @enum {number}
	 */
	Enums.Key = {
		'Tab': 9,
		'Enter': 13,
		'Shift': 16,
		'Ctrl': 17,
		'Space': 32,
		'PageUp': 33,
		'PageDown': 34,
		'End': 35,
		'Home': 36,
		'Up': 38,
		'Down': 40,
		'Left': 37,
		'Right': 39,
		'Del': 46,
		'a': 65,
		'c': 67,
		'f': 70,
		'n': 78,
		'p': 80,
		'q': 81,
		'r': 82,
		's': 83,
		'v': 86,
		'F5': 116,
		'Comma': 188,
		'Dot': 190
	};
	
	/**
	 * @enum {number}
	 */
	Enums.FileStorageType = {
		'Private': 0,
		'Corporate': 1
	};
	
	/**
	 * @enum {number}
	 */
	Enums.HelpdeskThreadStates = {
		'None': 0,
		'Pending': 1,
		'Waiting': 2,
		'Answered': 3,
		'Resolved': 4,
		'Deferred': 5
	};
	
	/**
	 * @enum {number}
	 */
	Enums.HelpdeskPostType = {
		'Normal': 0,
		'Internal': 1,
		'System': 2
	};
	
	/**
	 * @enum {number}
	 */
	Enums.HelpdeskFilters = {
		'All': 0,
		'Pending': 1,
		'Resolved': 2,
		'InWork': 3,
		'Open': 4,
		'Archived': 9
	};
	
	/**
	 * @enum {number}
	 */
	Enums.CalendarAccess = {
		'Full': 0,
		'Write': 1,
		'Read': 2
	};
	
	/**
	 * @enum {number}
	 */
	Enums.CalendarEditRecurrenceEvent = {
		'None': 0,
		'OnlyThisInstance': 1,
		'AllEvents': 2
	};
	
	/**
	 * @enum {number}
	 */
	Enums.CalendarRepeatPeriod = {
		'None': 0,
		'Daily': 1,
		'Weekly': 2,
		'Monthly': 3,
		'Yearly': 4
	};
	
	Enums.DesktopNotifications = {
		'Allowed': 0,
		'NotAllowed': 1,
		'Denied': 2,
		'NotSupported': 9
	};
	
	Enums.PhoneAction = {
		'Settings': 'settings',
		'Incoming': 'incoming',
		'Outgoing': 'outgoing'
	};
	
	Enums.HtmlEditorImageSizes = {
		'Small': 'small',
		'Medium': 'medium',
		'Large': 'large',
		'Original': 'original'
	};
	
	ko.bindingHandlers.command = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			var
				jqElement = $(oElement),
				oCommand = fValueAccessor()
			;
	
			if (!oCommand || !oCommand.enabled || !oCommand.canExecute)
			{
				throw new Error('You are not using command function');
			}
	
			jqElement.addClass('command');
			ko.bindingHandlers[jqElement.is('form') ? 'submit' : 'click'].init.apply(oViewModel, arguments);
		},
	
		'update': function (oElement, fValueAccessor) {
	
			var
				bResult = true,
				jqElement = $(oElement),
				oCommand = fValueAccessor()
			;
	
			bResult = oCommand.enabled();
			jqElement.toggleClass('command-not-enabled', !bResult);
	
			if (bResult)
			{
				bResult = oCommand.canExecute();
				jqElement.toggleClass('unavailable', !bResult);
			}
	
			jqElement.toggleClass('command-disabled disable disabled', !bResult);
	
			jqElement.toggleClass('command-disabled', !bResult);
	//		if (jqElement.is('input') || jqElement.is('button'))
	//		{
	//			jqElement.prop('disabled', !bResult);
	//		}
		}
	};
	
	ko.bindingHandlers.onEnter = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			ko.bindingHandlers.event.init(oElement, function () {
				return {
					'keyup': function (oData, oEvent) {
						if (oEvent && 13 === window.parseInt(oEvent.keyCode, 10))
						{
							$(oElement).trigger('change');
							fValueAccessor().call(this, oData);
						}
					}
				};
			}, fAllBindingsAccessor, oViewModel);
		}
	};
	ko.bindingHandlers.onCtrlEnter = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			ko.bindingHandlers.event.init(oElement, function () {
				return {
					'keydown': function (oData, oEvent) {
						if (oEvent && 13 === window.parseInt(oEvent.keyCode, 10) && oEvent.ctrlKey)
						{
							$(oElement).trigger('change');
							fValueAccessor().call(this, oData);
	
							return false;
						}
	
						return true;
					}
				};
			}, fAllBindingsAccessor, oViewModel);
		}
	};
	
	ko.bindingHandlers.onEsc = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			ko.bindingHandlers.event.init(oElement, function () {
				return {
					'keyup': function (oData, oEvent) {
						if (oEvent && 27 === window.parseInt(oEvent.keyCode, 10))
						{
							$(oElement).trigger('change');
							fValueAccessor().call(this, oData);
						}
					}
				};
			}, fAllBindingsAccessor, oViewModel);
		}
	};
	
	ko.bindingHandlers.onFocusSelect = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			ko.bindingHandlers.event.init(oElement, function () {
				return {
					'focus': function () {
						oElement.select();
					},
					'mouseup': function () {
						return false;
					}
				};
			}, fAllBindingsAccessor, oViewModel);
		}
	};
	
	ko.bindingHandlers.onFocusMoveCaretToEnd = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			ko.bindingHandlers.event.init(oElement, function () {
				return {
					'focus': function () {
						Utils.moveCaretToEnd(oElement);
					}
				};
			}, fAllBindingsAccessor, oViewModel);
		}
	};
	
	ko.bindingHandlers.onEnterChange = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			ko.bindingHandlers.event.init(oElement, function () {
				return {
					'keyup': function (oData, oEvent) {
						if (oEvent && 13 === window.parseInt(oEvent.keyCode, 10))
						{
							$(oElement).trigger('change');
						}
					}
				};
			}, fAllBindingsAccessor, oViewModel);
		}
	};
	
	ko.bindingHandlers.jhasfocus = {
		'init': function (oElement, fValueAccessor) {
			$(oElement).on('focus', function () {
				fValueAccessor()(true);
			}).on('blur', function () {
				fValueAccessor()(false);
			});
		},
		'update': function (oElement, fValueAccessor) {
			if (ko.utils.unwrapObservable(fValueAccessor()))
			{
				if (!$(oElement).is(':focus'))
				{
					$(oElement).focus();
				}
			}
			else
			{
				if ($(oElement).is(':focus'))
				{
					$(oElement).blur();
				}
			}
		}
	};
	
	ko.bindingHandlers.fadeIn = {
		'update': function (oElement, fValueAccessor) {
			if (ko.utils.unwrapObservable(fValueAccessor()))
			{
				$(oElement).hide().fadeIn('fast');
			}
		}
	};
	
	ko.bindingHandlers.fadeOut = {
		'update': function (oElement, fValueAccessor) {
			if (ko.utils.unwrapObservable(fValueAccessor()))
			{
				$(oElement).fadeOut();
			}
		}
	};
	
	ko.bindingHandlers.i18n = {
		'init': function (oElement, fValueAccessor) {
	
			var
				sKey = $(oElement).data('i18n'),
				sValue = sKey ? Utils.i18n(sKey) : sKey
			;
	
			if ('' !== sValue)
			{
				switch (fValueAccessor()) {
				case 'value':
					$(oElement).val(sValue);
					break;
				case 'text':
					$(oElement).text(sValue);
					break;
				case 'html':
					$(oElement).html(sValue);
					break;
				case 'title':
					$(oElement).attr('title', sValue);
					break;
				case 'placeholder':
					$(oElement).attr({'placeholder': sValue});
					break;
				}
			}
		}
	};
	
	ko.bindingHandlers.link = {
		'init': function (oElement, fValueAccessor) {
			$(oElement).attr('href', ko.utils.unwrapObservable(fValueAccessor()));
		}
	};
	
	ko.bindingHandlers.title = {
		'init': function (oElement, fValueAccessor) {
			$(oElement).attr('title', ko.utils.unwrapObservable(fValueAccessor()));
		}
	};
	
	ko.bindingHandlers.initDom = {
		'init': function (oElement, fValueAccessor) {
			if (fValueAccessor()) {
				if (_.isArray(fValueAccessor()))
				{
					var
						aList = fValueAccessor(),
						iIndex = aList.length - 1
					;
	
					for (; 0 <= iIndex; iIndex--)
					{
						aList[iIndex]($(oElement));
					}
				}
				else
				{
					fValueAccessor()($(oElement));
				}
			}
		}
	};
	
	ko.bindingHandlers.customScrollbar = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			var
				jqElement = $(oElement),
				oCommand = fValueAccessor()
			;
			
			oCommand = /** @type {{scrollToTopTrigger:{subscribe:Function},scrollToBottomTrigger:{subscribe:Function},scrollTo:{subscribe:Function},reset:Function}}*/ oCommand;
			
			jqElement.addClass('scroll-wrap').customscroll(oCommand);
			
			if (!Utils.isUnd(oCommand.reset)) {
				oElement._customscroll_reset = _.throttle(function () {
					jqElement.data('customscroll').reset();
				}, 100);
			}
			
			if (!Utils.isUnd(oCommand.scrollToTopTrigger) && Utils.isFunc(oCommand.scrollToTopTrigger.subscribe)) {
				oCommand.scrollToTopTrigger.subscribe(function () {
					if (jqElement.data('customscroll')) {
						jqElement.data('customscroll')['scrollToTop']();
					}
				});
			}
			
			if (!Utils.isUnd(oCommand.scrollToBottomTrigger) && Utils.isFunc(oCommand.scrollToBottomTrigger.subscribe)) {
				oCommand.scrollToBottomTrigger.subscribe(function () {
					if (jqElement.data('customscroll')) {
						jqElement.data('customscroll')['scrollToBottom']();
					}
				});
			}
			
			if (!Utils.isUnd(oCommand.scrollTo) && Utils.isFunc(oCommand.scrollTo.subscribe)) {
				oCommand.scrollTo.subscribe(function () {
					if (jqElement.data('customscroll')) {
						jqElement.data('customscroll')['scrollTo'](oCommand.scrollTo());
					}
				});
			}
		},
		
		'update': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
			if (oElement._customscroll_reset) {
				oElement._customscroll_reset();
			}
			if (!Utils.isUnd(fValueAccessor().top)) {
				$(oElement).data('customscroll')['vertical'].set(fValueAccessor().top);
			}
		}
	};
	
	ko.bindingHandlers.customOptions = {
		'init': function () {
			return {
				'controlsDescendantBindings': true
			};
		},
	
		'update': function (element, valueAccessor, allBindingsAccessor, viewModel, bindingContext) {
			var i = 0, j = 0;
			var previousSelectedValues = ko.utils.arrayMap(ko.utils.arrayFilter(element.childNodes, function (node) {
				return node.tagName && node.tagName === 'OPTION' && node.selected;
			}), function (node) {
				return ko.selectExtensions.readValue(node) || node.innerText || node.textContent;
			});
			var previousScrollTop = element.scrollTop;
			var value = ko.utils.unwrapObservable(valueAccessor());
	
			// Remove all existing <option>s.
			while (element.length > 0)
			{
				ko.cleanNode(element.options[0]);
				element.remove(0);
			}
	
			if (value)
			{
				if (typeof value.length !== 'number')
				{
					value = [value];
				}
	
				var optionsBind = allBindingsAccessor()['optionsBind'];
				for (i = 0, j = value.length; i < j; i++)
				{
					var option = document.createElement('OPTION');
					var optionValue = ko.utils.unwrapObservable(value[i]);
					ko.selectExtensions.writeValue(option, optionValue);
					option.appendChild(document.createTextNode(optionValue));
					element.appendChild(option);
					if (optionsBind)
					{
						option.setAttribute('data-bind', optionsBind);
						ko.applyBindings(bindingContext['createChildContext'](optionValue), option);
					}
				}
	
				var newOptions = element.getElementsByTagName('OPTION');
				var countSelectionsRetained = 0;
				var isIe = navigator.userAgent.indexOf("MSIE 6") >= 0;
				for (i = 0, j = newOptions.length; i < j; i++)
				{
					if (ko.utils.arrayIndexOf(previousSelectedValues, ko.selectExtensions.readValue(newOptions[i])) >= 0)
					{
						 if (isIe) {
							 newOptions[i].setAttribute("selected", true);
						 } else {
							newOptions[i].selected = true;
						 }
	
						countSelectionsRetained++;
					}
				}
	
				element.scrollTop = previousScrollTop;
	
				if (countSelectionsRetained < previousSelectedValues.length)
				{
					ko.utils.triggerEvent(element, 'change');
				}
			}
		}
	};
	
	ko.bindingHandlers.splitter = {
		'init': function (oElement, fValueAccessor) {
			var
				jqElement = $(oElement),
				oCommand = fValueAccessor()
			;
	
			setTimeout(function(){
				jqElement.splitter(_.defaults(
					oCommand,
					{
						'name': '',
						'sizeLeft': 200,
						'minLeft': 20,
						'minRight': 40
					}
				));
			}, 1);
		}
	};
	
	ko.bindingHandlers.dropdown = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			var
				jqElement = $(oElement),
				oCommand = _.defaults(
					fValueAccessor(), {
						'disabled': 'disabled',
						'expand': 'expand',
						'control': true,
						'container': '.dropdown_content',
						'passClick': true,
						'trueValue': true
					}
				),
				element = oCommand['control'] ? jqElement.find('.control') : jqElement,
				oDocument = $(document),
				callback = function () {
					if (!Utils.isUnd(oCommand['callback'])) {
						oCommand['callback'].call(
							oViewModel,
							jqElement.hasClass(oCommand['expand']) ? oCommand['trueValue'] : false,
							jqElement
						);
					}
				},
				stop = function (event) {
					event.stopPropagation();
				}
			;
			
			if (!oCommand['passClick']) {
				jqElement.find(oCommand['container']).click(stop);
				element.click(stop);
			}
			
			jqElement.removeClass(oCommand['expand']);
			
			if (oCommand['close'] && oCommand['close']['subscribe']) {
				oCommand['close'].subscribe(function (bValue) {
					if (!bValue) {
						oDocument.unbind('click.dropdown');
						jqElement.removeClass(oCommand['expand']);
					}
					
					callback();
				});
			}
	
			//TODO fix data-bind click
			element.click(function(){
				if (!jqElement.hasClass(oCommand['disabled'])) {
					
					jqElement.toggleClass(oCommand['expand']);
					
					_.defer(function(){
						callback();
					});
					
					if (jqElement.hasClass(oCommand['expand'])) {
						
						if (oCommand['close'] && oCommand['close']['subscribe']) {
							oCommand['close'](true);
						}
						
						_.defer(function(){
							oDocument.one('click.dropdown', function () {
								if (oCommand['close'] && oCommand['close']['subscribe']) {
									oCommand['close'](false);
								}
								jqElement.removeClass(oCommand['expand']);
								
								callback();
							});
						});
					}
				}
			});
		}
	};
	
	ko.bindingHandlers.customSelect = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			var
				jqElement = $(oElement),
				oCommand = _.defaults(
					fValueAccessor(), {
						'disabled': 'disabled',
						'selected': 'selected',
						'expand': 'expand',
						'control': true,
						'input': false
					}
				),
				aOptions = [],
				oControl = oCommand['control'] ? jqElement.find('.control') : jqElement,
				oContainer = jqElement.find('.dropdown_content'),
				oText = jqElement.find('.link'),
				
				updateField = function (value) {
					_.each(aOptions, function (item) {
						item.removeClass(oCommand['selected']);
					});
					var item = _.find(oCommand['options'], function (item) {
						return item[oCommand['optionsValue']] === value;
					});
					if (Utils.isUnd(item)) {
						item = oCommand['options'][0];
					}
					aOptions[_.indexOf(oCommand['options'], item)].addClass(oCommand['selected']);
					oText.text($.trim(item[oCommand['optionsText']]));
					
					return item[oCommand['optionsValue']];
				}
			;
	
			var updateList = function () {
				oContainer.empty();
				aOptions = [];
	
				_.each(oCommand['options'], function (item) {
					var
						oOption = $('<span class="item"></span>')
							.text(item[oCommand['optionsText']])
							.data('value', item[oCommand['optionsValue']]),
						isDisabled = item['isDisabled']
						;
	
					if (isDisabled)
					{
						oOption.data('isDisabled', isDisabled).addClass('disabled');
					}
					else
					{
						oOption.data('isDisabled', isDisabled).removeClass('disabled');
					}
	
					aOptions.push(oOption);
					oContainer.append(oOption);
				}, this);
			};
			updateList();
	
			oContainer.on('click', '.item', function () {
				var jqItem = $(this);
	
				if(!jqItem.data('isDisabled'))
				{
					oCommand.value(jqItem.data('value'));
				}
			});
	
			if (!oCommand.input && oCommand['value'] && oCommand['value'].subscribe)
			{
				oCommand['value'].subscribe(function () {
					var mValue = updateField(oCommand['value']());
					if (oCommand['value']() !== mValue)
					{
						oCommand['value'](mValue);
					}
				}, oViewModel);
	
				oCommand['value'].valueHasMutated();
			}
	
			if(oCommand.alarmOptions)
			{
				oCommand.alarmOptions.subscribe(function () {
					updateList();
				}, oViewModel);
			}
	
			//TODO fix data-bind click
			jqElement.removeClass(oCommand['expand']);
			oControl.click(function(ev){
				if (!jqElement.hasClass(oCommand['disabled'])) {
					jqElement.toggleClass(oCommand['expand']);
	
					if (jqElement.hasClass(oCommand['expand'])) {
	//				if (jqElement.hasClass(oCommand['expand']) && !$(ev.target).hasClass('disabled')) {
						_.defer(function(){
							$(document).one('click', function () {
								jqElement.removeClass(oCommand['expand']);
							});
						});
					}
	//				else
	//				{
	//					jqElement.addClass(oCommand['expand']);
	//				}
				}
			});
		}
	};
	
	ko.bindingHandlers.moveToFolderFilter = {
		
		'init': function (oElement, fValueAccessor, allBindingsAccessor, viewModel, bindingContext) {
			var
				jqElement = $(oElement),
				oCommand = _.defaults(
					fValueAccessor(), {
						'disabled': 'disabled',
						'selected': 'selected',
						'expand': 'expand',
						'control': true,
						'container': '.content'
					}
				),
				oControl = oCommand['control'] ? jqElement.find('.control') : jqElement,
				oContainer = jqElement.find(oCommand['container']),
				text = jqElement.find('.link')
			;
	
			jqElement.removeClass(oCommand['expand']);
	
			oCommand['options'].subscribe(function (aValue) {
				var
					sValue = oCommand['value'](),
					oFolderItem = _.find(aValue, function (oItem) {
						return oItem.id === sValue;
					})
				;
	
				if (!oFolderItem)
				{
					oCommand['value']('');
				}
			});
	
			if (oCommand['value'] && oCommand['value'].subscribe)
			{
				oCommand['value'].subscribe(function (sValue) {
					var
						aOptions = _.isArray(oCommand['options']) ? oCommand['options'] : oCommand['options'](),
						oFindItem = _.find(aOptions, function (oItem) {
							return oItem[oCommand['optionsValue']] === sValue;
						})
					;
	
					if (!oFindItem)
					{
						oFindItem = _.find(aOptions, function (oItem) {
							return oItem[oCommand['optionsValue']] === '';
						});
	
						if (oFindItem && '' !== sValue)
						{
							oCommand['value']('');
						}
					}
	
					if (oFindItem)
					{
						text.text($.trim(oFindItem[oCommand['optionsText']]));
					}
				});
	
				oCommand['value'].valueHasMutated();
			}
	
			oContainer.on('click', '.item', function () {
				var sFolderName = $(this).data('value');
				oCommand['value'](sFolderName);
			});
	
			oControl.click(function(){
				if (!jqElement.hasClass(oCommand['disabled'])) {
					jqElement.toggleClass(oCommand['expand']);
					if (jqElement.hasClass(oCommand['expand'])) {
						_.defer(function(){
							$(document).one('click', function () {
								jqElement.removeClass(oCommand['expand']);
							});
						});
					}
				}
			});
		},
		'update': function (oElement, fValueAccessor) {
			var
				oCommand = _.defaults(
					fValueAccessor(), {
						'disabled': 'disabled',
						'selected': 'selected',
						'expand': 'expand',
						'control': true,
						'container': '.content'
					}
				),
				oContainer = $(oElement).find(oCommand['container']),
				aOptions = _.isArray(oCommand['options']) ? oCommand['options'] : oCommand['options'](),
				sFolderName = oCommand['value'] ? oCommand['value']() : ''
			;
	
			oContainer.empty();
	
			_.each(aOptions, function (item) {
				var oOption = $('<span class="item"></span>')
					.text(item[oCommand['optionsText']])
					.data('value', item[oCommand['optionsValue']]);
	
				if (sFolderName === item[oCommand['optionsValue']])
				{
					oOption.addClass('selected');
				}
	
				oContainer.append(oOption);
			});
		}
	};
	
	ko.bindingHandlers.contactcard = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel) {
			var
				jqElement = $(oElement),
				bShown = false,
				oCommand = _.defaults(
					fValueAccessor(), {
						'disabled': 'disabled',
						'expand': 'expand',
						'control': true
					}
				),
				element = oCommand['control'] ? jqElement.find('.control') : jqElement
			;
	
			if (oCommand['trigger'] !== undefined && oCommand['trigger'].subscribe !== undefined) {
				
				jqElement.removeClass(oCommand['expand']);
				
				element.bind({
					'mouseover': function() {
						if (!jqElement.hasClass(oCommand['disabled']) && oCommand['trigger']()) {
							bShown = true;
							_.delay(function () {
								if (bShown) {
									jqElement.addClass(oCommand['expand']);
								}
							}, 200);
						}
					},
					'mouseout': function() {
						if (oCommand['trigger']()) {
							bShown = false;
							_.delay(function () {
								if (!bShown) {
									jqElement.removeClass(oCommand['expand']);
								}
							}, 200);
						}
					}
				});
	
			}
		}
	};
	
	ko.bindingHandlers.checkmail = {
		'update': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
				
			var
				oOptions = oElement.oOptions || null,
				jqElement = oElement.jqElement || null,
				oIconIE = oElement.oIconIE || null,
				values = fValueAccessor(),
				state = values.state
			;
	
			if (values.state !== undefined) {
				if (!jqElement)
				{
					oElement.jqElement = jqElement = $(oElement);
				}
	
				if (!oOptions)
				{
					oElement.oOptions = oOptions = _.defaults(
						values, {
							'activeClass': 'process',
							'duration': 800
						}
					);
				}
	
				Utils.deferredUpdate(jqElement, state, oOptions['duration'], function(element, state){
					if (App.browser.ie9AndBelow)
					{
						if (!oIconIE)
						{
							oElement.oIconIE = oIconIE = jqElement.find('.icon');
						}
	
						if (!oIconIE.__intervalIE && !!state)
						{
							var
								i = 0,
								style = ''
							;
	
							oIconIE.__intervalIE = setInterval(function() {
								style = '0px -' + (20 * i) + 'px';
								i = i < 7 ? i + 1 : 0;
								oIconIE.css({'background-position': style});
							} , 1000/12);
						}
						else
						{
							oIconIE.css({'background-position': '0px 0px'});
							clearInterval(oIconIE.__intervalIE);
							oIconIE.__intervalIE = null;
						}
					}
					else
					{
						element.toggleClass(oOptions['activeClass'], state);
					}
				});
			}
		}
	};
	
	ko.bindingHandlers.heightAdjust = {
		'update': function (oElement, fValueAccessor, fAllBindingsAccessor) {
				
			var 
				jqElement = oElement.jqElement || null,
				height = 0,
				sLocation = fValueAccessor().location
			;
	
			if (!jqElement) {
				oElement.jqElement = jqElement = $(oElement);
			}
	
			_.delay(function () {
				_.each(fValueAccessor().elements, function (mItem) {
					var element = mItem();
					if (element) {
						height += element.is(':visible') ? element.outerHeight() : 0;
					}
				});
				
				if (sLocation === 'top' || sLocation === undefined) {
					jqElement.css({
						'padding-top': height,
						'margin-top': -height
					});
				} else if (sLocation === 'bottom') {
					jqElement.css({
						'padding-bottom': height,
						'margin-bottom': -height
					});
				}
			}, 400);
		}
	};
	
	ko.bindingHandlers.triggerInview = {
		'update': function (oElement, fValueAccessor) {
			if (fValueAccessor().trigger().length <= 0 )
			{
				return;
			}
			
			_.defer(function () {
				var 
					$element = $(oElement),
					frameHeight = $element.height(),
					oCommand = fValueAccessor(),
					elements = null
				;
				
				oCommand = /** @type {{selector:string}}*/ oCommand;
				elements = $element.find(oCommand.selector);
				
				elements.each(function () {
					this.$el = $(this);
					this.inviewHeight = this.$el.height();
					this.inview = false;
				});
				
				var delayedScroll = _.debounce(function () {
					
					elements.each(function () {
						var inview = this.inview || false;
						
						var elOffset = this.$el.position().top + parseInt(this.$el.css('margin-top'), 10);
						if (elOffset > 0 && elOffset < frameHeight)
						{
							if (!inview)
							{
								this.inview = true;
								this.$el.trigger('inview');                        
							}
						}
						else
						{
							this.inview = false;
						}
					});
				}, 2000);
				
				$element.scroll(delayedScroll);
				
				delayedScroll();
			});
		}
	};
	
	ko.bindingHandlers.watchWidth = {
		'init': function (oElement, fValueAccessor) {
			$(window).bind('resize', function () {
				fValueAccessor()($(oElement).outerWidth());
			});
		}
	};
	
	ko.bindingHandlers.columnCalc = {
		'init': function (oElement, fValueAccessor) {
	
			var
				$oElement = $(oElement),
				oProp = fValueAccessor()['prop'],
				$oItem = null,
				iWidth = 0
			;
				
			$oItem = $oElement.find(fValueAccessor()['itemSelector']);
	
			if ($oItem[0] === undefined) {
				return;
			}
			
			iWidth = $oItem.outerWidth(true);
			iWidth = 1 >= iWidth ? 1 : iWidth;
			
			if (oProp)
			{
				$(window).bind('resize', function () {
					var iW = $oElement.width();
					oProp(0 < iW ? Math.floor(iW / iWidth) : 1);
				});
			}
		}
	};
	
	ko.bindingHandlers.quickReplyAnim = {
		'update': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
	
			var
				jqTextarea = oElement.jqTextarea || null,
				jqStatus = oElement.jqStatus || null,
				jqButtons = oElement.jqButtons || null,
				jqElement = oElement.jqElement || null,
				oPrevActions = oElement.oPrevActions || null,
				values = fValueAccessor(),
				oActions = null
			;
	
			oActions = _.defaults(
				values, {
					'saveAction': false,
					'sendAction': false,
					'activeAction': false
				}
			);
	
			if (!jqElement)
			{
				oElement.jqElement = jqElement = $(oElement);
				oElement.jqTextarea = jqTextarea = jqElement.find('textarea');
				oElement.jqStatus = jqStatus = jqElement.find('.status');
				oElement.jqButtons = jqButtons = jqElement.find('.buttons');
				
				oElement.oPrevActions = oPrevActions = {
					'saveAction': null,
					'sendAction': null,
					'activeAction': null
				};
			}
	
			if (jqElement.is(':visible'))
			{
				if (App.browser.ie9AndBelow)
				{
					if (jqTextarea && !jqElement.defualtHeight && !jqTextarea.defualtHeight)
					{
						jqElement.defualtHeight = jqElement.outerHeight();
						jqTextarea.defualtHeight = jqTextarea.outerHeight();
						jqStatus.defualtHeight = jqButtons.outerHeight();
						jqButtons.defualtHeight = jqButtons.outerHeight();
					}
	
					_.defer(function () {
						var 
							activeChanged = oPrevActions.activeAction !== oActions['activeAction'],
							sendChanged = oPrevActions.sendAction !== oActions['sendAction'],
							saveChanged = oPrevActions.saveAction !== oActions['saveAction']
						;
	
						if (activeChanged)
						{
							if (oActions['activeAction'])
							{
								jqTextarea.animate({
									'height': jqTextarea.defualtHeight + 50
								}, 300);
								jqElement.animate({
									'max-height': jqElement.defualtHeight + jqButtons.defualtHeight + 50
								}, 300);
							}
							else
							{
								jqTextarea.animate({
									'height': jqTextarea.defualtHeight
								}, 300);
								jqElement.animate({
									'max-height': jqElement.defualtHeight
								}, 300);
							}
						}
	
						if (sendChanged || saveChanged)
						{
							if (oActions['sendAction'])
							{
								jqElement.animate({
									'max-height': '30px'
								}, 300);
								jqStatus.animate({
									'max-height': '30px',
									'opacity': 1
								}, 300);
							}
							else if (oActions['saveAction'])
							{
								jqElement.animate({
									'max-height': 0
								}, 300);
							}
							else
							{
								jqElement.animate({
									'max-height': jqElement.defualtHeight + jqButtons.defualtHeight + 50
								}, 300);
								jqStatus.animate({
									'max-height': 0,
									'opacity': 0
								}, 300);
							}
						}
					});
				}
				else
				{
					jqElement.toggleClass('saving', oActions['saveAction']);
					jqElement.toggleClass('sending', oActions['sendAction']);
					jqElement.toggleClass('active', oActions['activeAction']);
				}
			}
	
			_.defer(function () {
				oPrevActions = oActions;
			});
		}
	};
	
	ko.extenders.reversible = function (oTarget)
	{
		var mValue = oTarget();
	
		oTarget.commit = function ()
		{
			mValue = oTarget();
		};
	
		oTarget.revert = function ()
		{
			oTarget(mValue);
		};
	
		oTarget.commitedValue = function ()
		{
			return mValue;
		};
	
		oTarget.changed = function ()
		{
			return mValue !== oTarget();
		};
		
		return oTarget;
	};
	
	ko.extenders.autoResetToFalse = function (oTarget, iOption)
	{
		oTarget.iTimeout = 0;
		oTarget.subscribe(function (bValue) {
			if (bValue)
			{
				window.clearTimeout(oTarget.iTimeout);
				oTarget.iTimeout = window.setTimeout(function () {
					oTarget.iTimeout = 0;
					oTarget(false);
				}, Utils.pInt(iOption));
			}
		});
	
		return oTarget;
	};
	
	/**
	 * @param {(Object|null|undefined)} oContext
	 * @param {Function} fExecute
	 * @param {(Function|boolean|null)=} fCanExecute
	 * @return {Function}
	 */
	Utils.createCommand = function (oContext, fExecute, fCanExecute)
	{
		var
			fResult = fExecute ? function () {
				if (fResult.canExecute && fResult.canExecute())
				{
					return fExecute.apply(oContext, Array.prototype.slice.call(arguments));
				}
				return false;
			} : function () {}
		;
	
		fResult.enabled = ko.observable(true);
	
		fCanExecute = Utils.isUnd(fCanExecute) ? true : fCanExecute;
		if (Utils.isFunc(fCanExecute))
		{
			fResult.canExecute = ko.computed(function () {
				return fResult.enabled() && fCanExecute.call(oContext);
			});
		}
		else
		{
			fResult.canExecute = ko.computed(function () {
				return fResult.enabled() && !!fCanExecute;
			});
		}
	
		return fResult;
	};
	
	ko.bindingHandlers.autocomplete = {
		'init': function (oElement, fValueAccessor) {
			
			function split(val)
			{
				return val.split(/,\s*/);
			}
	
			function extractLast(term)
			{
				return split(term).pop();
			}
	
			var 
				fCallback = fValueAccessor(),
				oJqElement = $(oElement)
			;
			
			if (fCallback && oJqElement && oJqElement[0])
			{
				oJqElement.autocomplete({
					'minLength': 1,
					
					'source': function (request, response) {
						fCallback(extractLast(request['term']), response);
					},
					'search': function () {
						var term = extractLast(this.value);
						if (term.length < 1) {
							return false;
						}
	
						return true;
					},
					'focus': function () {
						return false;
					},
					'select': function (event, ui) {
						var terms = split(this.value);
	
						terms.pop();
						terms.push(ui['item']['value']);
						terms.push('');
	
						this.value = terms.join(', ').slice(0, -2);
	
						oJqElement.trigger('change');
	
						// Move to the end of the input string
						var moveCursorToEnd = function(el) {
							var endIndex = el.value.length;
	
							//Chrome
							el.blur();
							el.focus();
							//IE, firefox and Opera
							if (el.setSelectionRange) {
								el.setSelectionRange(endIndex, endIndex);
							}
						};
						moveCursorToEnd(oJqElement[0]);
	
						return false;
					}
				});
			}
		}
	};
	
	ko.bindingHandlers.autocompleteSimple = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
	
			var
				jqEl = $(oElement),
				oOptions = fValueAccessor(),
				fCallback = oOptions['callback'],
				oDataAccessor = oOptions.dataAccessor ? oOptions.dataAccessor : null
			;
	
			if (fCallback && jqEl && jqEl[0])
			{
				jqEl.autocomplete({
					'minLength': 1,
					'source': function (request, response) {
						fCallback(request['term'], response);
					},
					'focus': function () {
						return false;
					},
					'select': function (oEvent, oItem) {
						_.delay(function () {
							jqEl.trigger('change');
						}, 5);
	
						if (oDataAccessor)
						{
							oDataAccessor(oItem.item);
						}
	
						return true;
					}
				});
			}
		}
	};
	
	ko.bindingHandlers.draggablePlace = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
			var oAllBindingsAccessor = fAllBindingsAccessor ? fAllBindingsAccessor() : null;
			$(oElement).draggable({
				'distance': 20,
				'handle': '.dragHandle',
				'cursorAt': {'top': 0, 'left': 0},
	//			'cursorAt': {'top': 22, 'left': 3},
				'helper': function (oEvent) {
					return fValueAccessor().call(oViewModel, oEvent && oEvent.target ? ko.dataFor(oEvent.target) : null);
				},
				'start': (oAllBindingsAccessor && oAllBindingsAccessor['draggableDragStartCallback']) ? oAllBindingsAccessor['draggableDragStartCallback'] : Utils.emptyFunction,
				'stop': (oAllBindingsAccessor && oAllBindingsAccessor['draggableDragStopCallback']) ? oAllBindingsAccessor['draggableDragStopCallback'] : Utils.emptyFunction
			}).on('mousedown', function () {
				Utils.removeActiveFocus();
			});
		}
	};
	
	ko.bindingHandlers.droppable = {
		'init': function (oElement, fValueAccessor) {
			var fValueFunc = fValueAccessor();
			if (false !== fValueFunc)
			{
				$(oElement).droppable({
					'hoverClass': 'droppableHover',
					'drop': function (oEvent, oUi) {
						fValueFunc(oEvent, oUi);
					}
				});
			}
		}
	};
	
	ko.bindingHandlers.draggable = {
		'init': function (oElement, fValueAccessor) {
			$(oElement).attr('draggable', ko.utils.unwrapObservable(fValueAccessor()));
		}
	};
	
	ko.bindingHandlers.autosize = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
	
			var
				jqEl = $(oElement),
				oOptions = fValueAccessor(),
				iHeight = jqEl.height(),
				iOuterHeight = jqEl.outerHeight(),
				iInnerHeight = jqEl.innerHeight(),
				iBorder = iOuterHeight - iInnerHeight,
				iPaddingTB = iInnerHeight - iHeight,
				iMinHeight = oOptions.minHeight ? oOptions.minHeight : 0,
				iMaxHeight = oOptions.maxHeight ? oOptions.maxHeight : 0,
				iScrollableHeight = oOptions.scrollableHeight ? oOptions.scrollableHeight : 1000,// max-height of .scrollable_field
				oAutosizeTrigger = oOptions.autosizeTrigger ? oOptions.autosizeTrigger : null,
					
				/**
				 * @param {boolean=} bIgnoreScrollableHeight
				 */
				fResize = function (bIgnoreScrollableHeight) {
					var iPadding = 0;
	
					if (App.browser.firefox)
					{
						iPadding = parseInt(jqEl.css('padding-top'), 10) * 2;
					}
	
					if (iMaxHeight)
					{
						/* 0-timeout to get the already changed text */
						setTimeout(function () {
							if (jqEl.prop('scrollHeight') < iMaxHeight)
							{
								jqEl.height(iMinHeight - iPaddingTB - iBorder);
								jqEl.height(jqEl.prop('scrollHeight') + iPadding - iPaddingTB);
							}
							else
							{
								jqEl.height(iMaxHeight - iPaddingTB - iBorder);
							}
						}, 100);
					}
					else if (bIgnoreScrollableHeight || jqEl.prop('scrollHeight') < iScrollableHeight)
					{
						setTimeout(function () {
							jqEl.height(iMinHeight - iPaddingTB - iBorder);
							jqEl.height(jqEl.prop('scrollHeight') + iPadding - iPaddingTB);
	//						$('.calendar_event .scrollable_field').scrollTop(jqEl.height('scrollHeight'))
						}, 100);
					}
				}
			;
	
			jqEl.on('keydown', function(oEvent, oData) {
				fResize();
			});
			jqEl.on('paste', function(oEvent, oData) {
				fResize();
			});
	//		jqEl.on('input', function(oEvent, oData) {
	//			fResize();
	//		});
	//		ko.bindingHandlers.event.init(oElement, function () {
	//			return {
	//				'keydown': function (oData, oEvent) {
	//					fResize();
	//					return true;
	//				}
	//			};
	//		}, fAllBindingsAccessor, oViewModel);
	
			if (oAutosizeTrigger)
			{
				oAutosizeTrigger.subscribe(function (arg) {
					fResize(arg);
				}, this);
			}
	
			fResize();
		}
	};
	
	ko.bindingHandlers.customBind = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
	
			var
				jqEl = $(oElement),
				oOptions = fValueAccessor(),
				oKeydown = oOptions.onKeydown ? oOptions.onKeydown : null,
				oKeyup = oOptions.onKeyup ? oOptions.onKeyup : null,
				oPaste = oOptions.onPaste ? oOptions.onPaste : null,
				oInput = oOptions.onInput ? oOptions.onInput : null,
				oValueObserver = oOptions.valueObserver ? oOptions.valueObserver : null
			;
	
			ko.bindingHandlers.event.init(oElement, function () {
				return {
					'keydown': function (oData, oEvent) {
						if(oKeydown)
						{
							oKeydown.call(this, oElement, oEvent, oValueObserver);
						}
						return true;
					},
					'keyup': function (oData, oEvent) {
						if(oKeyup)
						{
							oKeyup.call(this, oElement, oEvent, oValueObserver);
						}
						return true;
					},
					'paste': function (oData, oEvent) {
						if(oPaste)
						{
							oPaste.call(this, oElement, oEvent, oValueObserver);
						}
						return true;
					},
					'input': function (oData, oEvent) {
						if(oInput)
						{
							oInput.call(this, oElement, oEvent, oValueObserver);
						}
						return true;
					}
				};
			}, fAllBindingsAccessor, oViewModel);
		}
	};
	
	ko.bindingHandlers.fade = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
	
			var jqEl = $(oElement),
				jqElFaded = $('<span class="faded"></span>'),
				oOptions = fValueAccessor(),
				oColor = oOptions.color ? oOptions.color : null
			;
			
			jqEl.parent().addClass('fade');
			jqEl.after(jqElFaded);
	
			updateColor(oColor());
	
			oColor.subscribe(function (sColor) {
				updateColor(sColor);
			}, this);
	
			function updateColor(sColor)
			{
				var
					oHex2Rgb = hex2Rgb(sColor),
					sRGBColor = "rgba("+oHex2Rgb.r+","+oHex2Rgb.g+","+oHex2Rgb.b
				;
	
				colorIt(sColor, sRGBColor);
			}
	
			function hex2Rgb(sHex) {
				// Expand shorthand form (e.g. "03F") to full form (e.g. "0033FF")
				var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
				sHex = sHex.replace(shorthandRegex, function(m, r, g, b) {
					return r + r + g + g + b + b;
				});
	
				var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(sHex);
				return result ? {
					r: parseInt(result[1], 16),
					g: parseInt(result[2], 16),
					b: parseInt(result[3], 16)
				} : null;
			}
	
			function colorIt (hex, rgb)
			{
				if(Utils.isRTL)
				{
					jqElFaded
						.css("filter", "progid:DXImageTransform.Microsoft.gradient(startColorstr='" + hex + "', endColorstr='" + hex + "',GradientType=1 )")
						.css("background-image", "-webkit-gradient(linear, left top, right top, color-stop(0%," + rgb + ",1)" + "), color-stop(100%," + rgb + ",0)" + "))")
						.css("background-image", "-moz-linear-gradient(left, " + rgb + ",1)" + "0%, " + rgb + ",0)" + "100%)")
						.css("background-image", "-webkit-linear-gradient(left, " + rgb + "1)" + "0%," + rgb + ",0)" + "100%)")
						.css("background-image", "-o-linear-gradient(left, " + rgb + ",1)" + "0%," + rgb + ",0)" + "100%)")
						.css("background-image", "-ms-linear-gradient(left, " + rgb + ",1)" + "0%," + rgb + ",0)" + "100%)")
						.css("background-image", "linear-gradient(left, " + rgb + ",1)" + "0%," + rgb + ",0)" + "100%)");
				}
				else
				{
					jqElFaded
						.css("filter", "progid:DXImageTransform.Microsoft.gradient(startColorstr='" + hex + "', endColorstr='" + hex + "',GradientType=1 )")
						.css("background-image", "-webkit-gradient(linear, left top, right top, color-stop(0%," + rgb + ",0)" + "), color-stop(100%," + rgb + ",1)" + "))")
						.css("background-image", "-moz-linear-gradient(left, " + rgb + ",0)" + "0%, " + rgb + ",1)" + "100%)")
						.css("background-image", "-webkit-linear-gradient(left, " + rgb + ",0)" + "0%," + rgb + ",1)" + "100%)")
						.css("background-image", "-o-linear-gradient(left, " + rgb + ",0)" + "0%," + rgb + ",1)" + "100%)")
						.css("background-image", "-ms-linear-gradient(left, " + rgb + ",0)" + "0%," + rgb + ",1)" + "100%)")
						.css("background-image", "linear-gradient(left, " + rgb + ",0)" + "0%," + rgb + ",1)" + "100%)");
				}
			}
		}
	};
	
	ko.bindingHandlers.highlighter = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
	
			var
				jqEl = $(oElement),
				oOptions = fValueAccessor(),
				oValueObserver = oOptions.valueObserver ? oOptions.valueObserver : null,
				oHighlighterValueObserver = oOptions.highlighterValueObserver ? oOptions.highlighterValueObserver : null,
				oHighlightTrigger = oOptions.highlightTrigger ? oOptions.highlightTrigger : null,
				aWords = ['from:', 'to:', 'subject:', 'text:'],
				rPattern = getRegExp(),
				iPrevKeyCode = -1,
				qqq = 0
			;
	
			oHighlightTrigger.notifySubscribers();
	
			oHighlightTrigger.subscribe(function () {
				setTimeout(function () {
					highlight(false);
				}, 0);
			}, this);
	
			oHighlighterValueObserver.subscribe(function () {
				jqEl.text(oValueObserver());
			}, this);
	
	
	//		// http://jsfiddle.net/tG9Qa/
	//		jqEl.on('input', function(oEvent) {
	//			return filter_newlines(jqEl);
	//		});
	//		function filter_newlines(div) {
	//			var node, prev, _i, _len, _ref, _results;
	//			prev = null;
	//			_ref = div.contents();
	//			_results = [];
	//			for (_i = 0, _len = _ref.length; _i < _len; _i++) {
	//				node = _ref[_i];
	//				if (node.nodeType === 3) {
	//					node.nodeValue = node.nodeValue.replace('\n', '');
	//					if (prev) {
	//						node.nodeValue = prev.nodeValue + node.nodeValue;
	//						$(prev).remove();
	//					}
	//					_results.push(prev = node);
	//				} else if (node.tagName.toLowerCase() === 'br') {
	//					_results.push($(node).remove());
	//				} else {
	//					$(node).css('display', 'inline');
	//					filter_newlines($(node));
	//					_results.push(prev = null);
	//				}
	//			}
	//			return _results;
	//		}
	
			ko.bindingHandlers.event.init(oElement, function () {
				return {
					'keydown': function (oData, oEvent) {
	//					if (oEvent.keyCode === Enums.Key.Enter)
	//					{
	//						oEvent.preventDefault();
	//
	//					}
	//					return true;
						return oEvent.keyCode !== Enums.Key.Enter;
					},
					'keyup': function (oData, oEvent) {
						var
							aMoveKeys = [Enums.Key.Left, Enums.Key.Right, Enums.Key.Home, Enums.Key.End],
							bMoveKeys = -1 !== Utils.inArray(oEvent.keyCode, aMoveKeys)
						;
	
						if (!(
								oEvent.keyCode === Enums.Key.Enter ||
								oEvent.keyCode === Enums.Key.Ctrl ||
								((oEvent.ctrlKey || iPrevKeyCode === Enums.Key.Ctrl) && oEvent.keyCode === Enums.Key.a) ||
								oEvent.keyCode === Enums.Key.Shift ||
								((oEvent.shiftKey || iPrevKeyCode === Enums.Key.Shift) && bMoveKeys)
							))
						{
							oValueObserver(jqEl.text());
							highlight(false);
						}
						iPrevKeyCode = oEvent.keyCode;
						return true;
					},
					'paste': function (oData, oEvent) {
						setTimeout(function () {
							oValueObserver(oEvent.target.innerText);
							highlight(false);
						}, 0);
						return true;
					}
				};
			}, fAllBindingsAccessor, oViewModel);
	
			function highlight(bNotRestoreSel) {
				var
					iCaretPos = 0,
					sContent = jqEl.text(),
					aContent = sContent.split(rPattern),
					aDividedContent = [],
					sReplaceWith = '<span class="search_highlight"' + '>$&</span>'
				;
	
				//sContent = sContent.replace(/\u00a0/g, " ");
				//sContent = sContent.replace(/\s/g, "\u00a0");
	
				$.each(aContent, function (i, sEl) {
					if (_.any(aWords, function (oAnyEl) {return oAnyEl === sEl;}))
					{
						$.each(sEl, function (i, sElem) {
							aDividedContent.push($(sElem.replace(/(.)/, sReplaceWith)));
						});
					}
					else
					{
						$.each(sEl, function(i, sElem) {
							aDividedContent.push(document.createTextNode(sElem));
						});
					}
				});
	
				if (bNotRestoreSel)
				{
					jqEl.empty().append(aDividedContent);
				}
				else
				{
					iCaretPos = getCaretOffset();
					jqEl.empty().append(aDividedContent);
					setCursor(iCaretPos);
				}
			}
	
			function getRegExp()
			{
				var sPatt = '';
	
				$.each(aWords, function(i, oEl)
				{
					sPatt = (!i) ? (sPatt + '\\b' + oEl) : (sPatt + '|\\b' + oEl);
				});
	
				return new RegExp('(' + sPatt + ')', 'g');
			}
	
			function getCaretOffset()
			{
				var caretOffset = 0,
					range,
					preCaretRange,
					textRange,
					preCaretTextRange
				;
	
				if (typeof window.getSelection !== "undefined")
				{
					range = window.getSelection().getRangeAt(0);
					preCaretRange = range.cloneRange();
					preCaretRange.selectNodeContents(oElement);
					preCaretRange.setEnd(range.endContainer, range.endOffset);
					caretOffset = preCaretRange.toString().length;
				}
				else if (typeof document.selection !== "undefined" && document.selection.type !== "Control")
				{
					textRange = document.selection.createRange();
					preCaretTextRange = document.body.createTextRange();
					preCaretTextRange.moveToElementText(oElement);
					preCaretTextRange.setEndPoint("EndToEnd", textRange);
					caretOffset = preCaretTextRange.text.length;
				}
				qqq = caretOffset;
				return caretOffset;
			}
	
			function setCursor(iCaretPos)
			{
				var range,
					selection,
					textRange
				;
	
				if (!oElement)
				{
					return false;
				}
				else if(document.createRange)
				{
					range = document.createRange();
					range.selectNodeContents(oElement);
					range.setStart(oElement, iCaretPos);
					range.setEnd(oElement, iCaretPos);
					selection = window.getSelection();
					selection.removeAllRanges();
					selection.addRange(range);
				}
				else if(oElement.createTextRange)
				{
					textRange = oElement.createTextRange();
					textRange.collapse(true);
					textRange.moveEnd(iCaretPos);
					textRange.moveStart(iCaretPos);
					textRange.select();
					return true;
				}
				else if(oElement.setSelectionRange)
				{
					oElement.setSelectionRange(iCaretPos, iCaretPos);
					return true;
				}
	
				return false;
			}
	
	
	
	
	//		function setCursorPosition (iStartOffset)
	//		{
	//			if (document.createRange && window.getSelection)
	//			{
	//				var
	//					oRange = document.createRange(),
	//					oSel = window.getSelection()
	//					;
	//
	//				oSel.removeAllRanges();
	//				oRange.setStart(oElement, iStartOffset);
	//				oRange.setEnd(oElement, iStartOffset);
	//				oRange.collapse(true);
	//				oSel.addRange(oRange);
	//			}
	//		}
	//		function getSelectionRanges ()
	//		{
	//			var
	//				aRanges = []
	//				;
	//
	//
	//			if (window.getSelection)
	//			{
	//				var
	//					oSel = window.getSelection(),
	//					oRange = null,
	//					iIndex = 0,
	//					iLen = oSel.rangeCount
	//					;
	//
	//				for (; iIndex < iLen; ++iIndex)
	//				{
	//					oRange = oSel.getRangeAt(iIndex);
	//					aRanges.push(oRange);
	//				}
	//			}
	//
	//			return aRanges;
	//		}
	//		function setSelectionRanges (aRanges)
	//		{
	//			var
	//				oSel = null,
	//				iIndex = 0,
	//				iLen = 0,
	//				sRangeText = ''
	//				;
	//
	//			if (window.getSelection && $.isArray(aRanges))
	//			{
	//				oSel = window.getSelection();
	//				iLen = aRanges.length;
	//
	//				oSel.removeAllRanges();
	//
	//				for (; iIndex < iLen; ++iIndex)
	//				{
	//					sRangeText += aRanges[iIndex];
	//					oSel.addRange(aRanges[iIndex]);
	//				}
	//			}
	//
	//			return sRangeText;
	//		}
		}
	};
	
	/*ko.bindingHandlers.buttonText = {
		'init': function (oElement, fValueAccessor, fAllBindingsAccessor, oViewModel, bindingContext) {
	
			var
				jqEl = $(oElement),
				oOptions = fAllBindingsAccessor().buttonText,
				oValueObserver = oOptions.valueObserver ? oOptions.valueObserver : null,
				oTextTrigger = oOptions.textTrigger ? oOptions.textTrigger : null,
				sAction = oOptions.action ? oOptions.action : null,
				aText = []
			;
	
			switch (sAction)
			{
				case 'save':
					aText = ['Save', 'Saving...'];
					break;
				case 'send':
					aText = ['Send', 'Sending...'];
					break;
			}
	
			jqEl.text(aText[0]);
	
			oTextTrigger.subscribe(function(bState)
			{
				if(bState)
				{
					sAction ? jqEl.text(aText[1]) : jqEl.addClass('');
				}
				else
				{
					sAction ? jqEl.text(aText[0]) : jqEl.addClass('');
				}
			}, this);
		}
	};*/
	
	
	
	/**
	 * @type {Function}
	 */
	Utils.inArray = $.inArray;
	
	/**
	 * @type {Function}
	 */
	Utils.isFunc = $.isFunction;
	
	/**
	 * @type {Function}
	 */
	Utils.trim = $.trim;
	
	/**
	 * @type {Function}
	 */
	Utils.emptyFunction = function () {};
	
	/**
	 * @param {*} mValue
	 * 
	 * @return {boolean}
	 */
	Utils.isUnd = function (mValue)
	{
		return undefined === mValue;
	};
	
	/**
	 * @param {*} oValue
	 * 
	 * @return {boolean}
	 */
	Utils.isNull = function (oValue)
	{
		return null === oValue;
	};
	
	/**
	 * @param {*} oValue
	 * 
	 * @return {boolean}
	 */
	Utils.isNormal = function (oValue)
	{
		return !Utils.isUnd(oValue) && !Utils.isNull(oValue);
	};
	
	/**
	 * @param {(string|number)} mValue
	 * 
	 * @return {boolean}
	 */
	Utils.isNumeric = function (mValue)
	{
		return Utils.isNormal(mValue) ? (/^[1-9]+[0-9]*$/).test(mValue.toString()) : false;
	};
	
	/**
	 * @param {*} mValue
	 * 
	 * @return {number}
	 */
	Utils.pInt = function (mValue)
	{
		return Utils.isNormal(mValue) && '' !== mValue ? window.parseInt(mValue, 10) : 0;
	};
	
	/**
	 * @param {*} mValue
	 * 
	 * @return {string}
	 */
	Utils.pString = function (mValue)
	{
		return Utils.isNormal(mValue) ? mValue.toString() : '';
	};
	
	/**
	 * @param {*} aValue
	 * 
	 * @return {boolean}
	 */
	Utils.isNonEmptyArray = function (aValue)
	{
		return _.isArray(aValue) && 0 < aValue.length;
	};
	
	/**
	 * @param {Object} oObject
	 * @param {string} sName
	 * @param {*} mValue
	 */
	Utils.pImport = function (oObject, sName, mValue)
	{
		oObject[sName] = mValue;
	};
	
	/**
	 * @param {Object} oObject
	 * @param {string} sName
	 * @param {*} mDefault
	 * @return {*}
	 */
	Utils.pExport = function (oObject, sName, mDefault)
	{
		return Utils.isUnd(oObject[sName]) ? mDefault : oObject[sName];
	};
	
	/**
	 * @param {string} sText
	 * 
	 * @return {string}
	 */
	Utils.encodeHtml = function (sText)
	{
		return (sText) ? sText.toString()
			.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;')
			.replace(/"/g, '&quot;').replace(/'/g, '&#039;') : '';
	};
	
	/**
	 * @param {string} sKey
	 * @param {?Object=} oValueList
	 * @param {?string=} sDefaulValue
	 * @param {number=} nPluralCount
	 * 
	 * @return {string}
	 */
	Utils.i18n = function (sKey, oValueList, sDefaulValue, nPluralCount) {
	
		var
			sValueName = '',
			sResult = Utils.isUnd(I18n[sKey]) ? (Utils.isNormal(sDefaulValue) ? sDefaulValue : sKey) : I18n[sKey]
		;
	
		if (!Utils.isUnd(nPluralCount))
		{
			sResult = (function (nPluralCount, sResult) {
				var
					nPlural = Utils.getPlural(AppData.User.DefaultLanguage, nPluralCount),
					aPluralParts = sResult.split('|')
				;
	
				return (aPluralParts && aPluralParts[nPlural]) ? aPluralParts[nPlural] : (
					aPluralParts && aPluralParts[0] ? aPluralParts[0] : sResult);
	
			}(nPluralCount, sResult));
		}
	
		if (Utils.isNormal(oValueList))
		{
			for (sValueName in oValueList)
			{
				if (oValueList.hasOwnProperty(sValueName))
				{
					sResult = sResult.replace('%' + sValueName + '%', oValueList[sValueName]);
				}
			}
		}
	
		return sResult;
	};
	
	/**
	 * @param {number} iNum
	 * @param {number} iDec
	 * 
	 * @return {number}
	 */
	Utils.roundNumber = function (iNum, iDec)
	{
		return Math.round(iNum * Math.pow(10, iDec)) / Math.pow(10, iDec);
	};
	
	/**
	 * @param {(number|string)} iSizeInBytes
	 * 
	 * @return {string}
	 */
	Utils.friendlySize = function (iSizeInBytes)
	{
		var
			iBytesInKb = 1024,
			iBytesInMb = iBytesInKb * iBytesInKb,
			iBytesInGb = iBytesInKb * iBytesInKb * iBytesInKb
		;
	
		iSizeInBytes = Utils.pInt(iSizeInBytes);
	
		if (iSizeInBytes >= iBytesInGb)
		{
			return Utils.roundNumber(iSizeInBytes / iBytesInGb, 1) + Utils.i18n('MAIN/GIGABYTES');
		}
		else if (iSizeInBytes >= iBytesInMb)
		{
			return Utils.roundNumber(iSizeInBytes / iBytesInMb, 1) + Utils.i18n('MAIN/MEGABYTES');
		}
		else if (iSizeInBytes >= iBytesInKb)
		{
			return Utils.roundNumber(iSizeInBytes / iBytesInKb, 0) + Utils.i18n('MAIN/KILOBYTES');
		}
	
		return iSizeInBytes + Utils.i18n('MAIN/BYTES');
	};
	
	Utils.timeOutAction = (function () {
	
		var oTimeOuts = {};
	
		return function (sAction, fFunction, iTimeOut) {
			if (Utils.isUnd(oTimeOuts[sAction]))
			{
				oTimeOuts[sAction] = 0;
			}
	
			window.clearTimeout(oTimeOuts[sAction]);
			oTimeOuts[sAction] = window.setTimeout(fFunction, iTimeOut);
		};
	}());
	
	/**
	 * @param {...*} mArgs javascript annotation for variable numbers of arguments
	 */
	Utils.log = function (mArgs)
	{
	//	if (window.console && window.console.log)
	//	{
	//		window.console.log(mArgs);
	//	}
	};
	
	/**
	 * @param {string} sFullEmail
	 * 
	 * @return {Object}
	 */
	Utils.getEmailParts = function (sFullEmail)
	{
		var
			iQuote1Pos = sFullEmail.indexOf('"'),
			iQuote2Pos = sFullEmail.indexOf('"', iQuote1Pos + 1),
			iLeftBrocketPos = sFullEmail.indexOf('<', iQuote2Pos),
			iPrevLeftBroketPos = -1,
			iRightBrocketPos = -1,
			sName = '',
			sEmail = ''
		;
	
		while (iLeftBrocketPos !== -1)
		{
			iPrevLeftBroketPos = iLeftBrocketPos;
			iLeftBrocketPos = sFullEmail.indexOf('<', iLeftBrocketPos + 1);
		}
	
		iLeftBrocketPos = iPrevLeftBroketPos;
		iRightBrocketPos = sFullEmail.indexOf('>', iLeftBrocketPos + 1);
	
		if (iLeftBrocketPos === -1)
		{
			sEmail = $.trim(sFullEmail);
		}
		else
		{
			sName = (iQuote1Pos === -1) ?
				$.trim(sFullEmail.substring(0, iLeftBrocketPos)) :
				$.trim(sFullEmail.substring(iQuote1Pos + 1, iQuote2Pos));
	
			sEmail = $.trim(sFullEmail.substring(iLeftBrocketPos + 1, iRightBrocketPos));
		}
	
		return {
			'name': sName,
			'email': sEmail,
			'FullEmail': sFullEmail
		};
	};
	
	/**
	 * @param {string} sValue
	 * 
	 * @return {boolean}
	 */
	Utils.isCorrectEmail = function (sValue)
	{
		return !!(sValue.match(/^[A-Z0-9\"!#\$%\^\{\}`~&'\+\-=_\.]+@[A-Z0-9\.\-]+$/i));
	};
	
	/**
	 * @param {string} sAddresses
	 * 
	 * @return {Array}
	 */
	Utils.getIncorrectEmailsFromAddressString = function (sAddresses)
	{
		var
			aEmails = sAddresses.replace(/"[^"]*"/g, '').replace(/;/g, ',').split(','),
			aIncorrectEmails = [],
			iIndex = 0,
			iLen = aEmails.length,
			sFullEmail = '',
			oEmailParts = null
		;
	
		for (; iIndex < iLen; iIndex++)
		{
			sFullEmail = $.trim(aEmails[iIndex]);
			if (sFullEmail.length > 0)
			{
				oEmailParts = Utils.getEmailParts($.trim(aEmails[iIndex]));
				if (!Utils.isCorrectEmail(oEmailParts.email))
				{
					aIncorrectEmails.push(oEmailParts.email);
				}
			}
		}
	
		return aIncorrectEmails;
	};
	
	/**
	 * Gets link for contacts inport.
	 *
	 * @return {string}
	 */
	Utils.getImportContactsLink = function ()
	{
		return '?/ImportContacts/';
	};
	
	/**
	 * Gets link for contacts export.
	 *
	 * @return {string}
	 */
	Utils.getExportContactsLink = function ()
	{
		return '?/Raw/Contacts/';
	};
	
	/**
	 * Gets link for calendar export by hash.
	 *
	 * @param {number} iAccountId
	 * @param {string} sHash
	 * 
	 * @return {string}
	 */
	Utils.getExportCalendarLinkByHash = function (iAccountId, sHash)
	{
		return '?/Raw/Calendar/' + iAccountId + '/' + sHash;
	};
	
	/**
	 * Gets link for download by hash.
	 *
	 * @param {number} iAccountId
	 * @param {string} sHash
	 * @param {boolean=} bIsExt = false
	 * @param {string=} sTenatHash = ''
	 * 
	 * @return {string}
	 */
	Utils.getDownloadLinkByHash = function (iAccountId, sHash, bIsExt, sTenatHash)
	{
		bIsExt = Utils.isUnd(bIsExt) ? false : !!bIsExt;
		sTenatHash = Utils.isUnd(sTenatHash) ? '' : sTenatHash;
	
		return '?/Raw/Download/' + iAccountId + '/' + sHash + '/' + (bIsExt ? '1' : '0') + ('' === sTenatHash ? '' : '/' + sTenatHash);
	};
	
	/**
	 * Gets link for view by hash.
	 *
	 * @param {number} iAccountId
	 * @param {string} sHash
	 * @param {boolean=} bIsExt = false
	 * @param {string=} sTenatHash = ''
	 * 
	 * @return {string}
	 */
	Utils.getViewLinkByHash = function (iAccountId, sHash, bIsExt, sTenatHash)
	{
		bIsExt = Utils.isUnd(bIsExt) ? false : !!bIsExt;
		sTenatHash = Utils.isUnd(sTenatHash) ? '' : sTenatHash;
		
		return '?/Raw/View/' + iAccountId + '/' + sHash + '/' + (bIsExt ? '1' : '0') + ('' === sTenatHash ? '' : '/' + sTenatHash);
	};
	
	/**
	 * Gets link for thumbnail by hash.
	 *
	 * @param {number} iAccountId
	 * @param {string} sHash
	 *
	 * @return {string}
	 */
	Utils.getViewThumbnailLinkByHash = function (iAccountId, sHash)
	{
		return '?/Raw/Thumbnail/' + iAccountId + '/' + sHash;
	};
	
	/**
	 * Gets link for download by hash.
	 *
	 * @param {number} iAccountId
	 * @param {string} sHash
	 * 
	 * @return {string}
	 */
	Utils.getFilestorageDownloadLinkByHash = function (iAccountId, sHash)
	{
		return '?/Raw/FilesDownload/' + iAccountId + '/' + sHash;
	};
	
	/**
	 * Gets link for download by hash.
	 *
	 * @param {number} iAccountId
	 * @param {string} sHash
	 * 
	 * @return {string}
	 */
	Utils.getFilestorageViewLinkByHash = function (iAccountId, sHash)
	{
		return '?/Raw/FilesView/' + iAccountId + '/' + sHash;
	};
	
	/**
	 * Gets link for public by hash.
	 *
	 * @param {string} sHash
	 * 
	 * @return {string}
	 */
	Utils.getFilestoragePublicViewLinkByHash = function (sHash)
	{
		return '?/Window/Files/0/' + sHash;
	};
	
	/**
	 * Gets link for public by hash.
	 *
	 * @param {string} sHash
	 * 
	 * @return {string}
	 */
	Utils.getFilestoragePublicDownloadLinkByHash = function (sHash)
	{
		return '?/Raw/FilesPub/0/' + sHash;
	};
	
	/**
	 * @param {number} iMonth
	 * @param {number} iYear
	 * 
	 * @return {number}
	 */
	Utils.daysInMonth = function (iMonth, iYear)
	{
		if (0 < iMonth && 13 > iMonth && 0 < iYear)
		{
			return new Date(iYear, iMonth, 0).getDate();
		}
	
		return 31;
	};
	
	/** 
	 * @return {string}
	 */
	Utils.getAppPath = function ()
	{
		return window.location.protocol + '//' + window.location.host + window.location.pathname;
	};
	
	Utils.WindowOpener = {
	
		_iDefaultRatio: 0.8,
		_aOpenedWins: [],
		
		/**
		 * @param {{folder:Function, uid:Function}} oMessage
		 * @param {boolean=} bDrafts
		 */
		openMessage: function (oMessage, bDrafts)
		{
			if (oMessage)
			{
				var
					sFolder = oMessage.folder(),
					sUid = oMessage.uid(),
					sHash = '',
					oWin = null
				;
				
				if (bDrafts)
				{
					sHash = App.Routing.buildHashFromArray([Enums.Screens.SingleCompose, 'drafts', sFolder, sUid]);
				}
				else
				{
					sHash = App.Routing.buildHashFromArray([Enums.Screens.SingleMessageView, sFolder, 'msg' + sUid]);
				}
	
				oWin = this.openTab(sHash);
			}
		},
	
		/**
		 * @param {string} sUrl
		 * 
		 * @return Object
		 */
		openTab: function (sUrl)
		{
			var oWin = null;
	
			oWin = window.open(sUrl, '_blank');
			oWin.focus();
			oWin.name = AppData.Accounts.currentId();
	
			this._aOpenedWins.push(oWin);
			
			return oWin;
		},
		
		/**
		 * @param {string} sUrl
		 * @param {string} sPopupName
		 * @param {boolean=} bMenubar = false
		 * 
		 * @return Object
		 */
		open: function (sUrl, sPopupName, bMenubar)
		{
			var
				sMenubar = (bMenubar) ? ',menubar=yes' : ',menubar=no',
				sParams = 'location=no,toolbar=no,status=no,scrollbars=yes,resizable=yes' + sMenubar,
				oWin = null
			;
	
			sPopupName = sPopupName.replace(/\W/g, ''); // forbidden characters in the name of the window for ie
			sParams += this._getSizeParameters();
	
			oWin = window.open(sUrl, sPopupName, sParams);
			oWin.focus();
			oWin.name = AppData.Accounts.currentId();
	
			this._aOpenedWins.push(oWin);
			
			return oWin;
		},
	
		closeAll: function ()
		{
			var
				iLen = this._aOpenedWins.length,
				iIndex = 0,
				oWin = null
			;
	
			for (; iIndex < iLen; iIndex++)
			{
				oWin = this._aOpenedWins[iIndex];
				if (!oWin.closed)
				{
					oWin.close();
				}
			}
	
			this._aOpenedWins = [];
		},
	
		/**
		 * @return string
		 */
		_getSizeParameters: function ()
		{
			var
				iScreenWidth = window.screen.width,
				iWidth = Math.ceil(iScreenWidth * this._iDefaultRatio),
				iLeft = Math.ceil((iScreenWidth - iWidth) / 2),
	
				iScreenHeight = window.screen.height,
				iHeight = Math.ceil(iScreenHeight * this._iDefaultRatio),
				iTop = Math.ceil((iScreenHeight - iHeight) / 2)
			;
	
			return ',width=' + iWidth + ',height=' + iHeight + ',top=' + iTop + ',left=' + iLeft;
		}
	};
	
	/**
	 * @param {Object} oInput
	 */
	Utils.moveCaretToEnd = function (oInput)
	{
		var oTextRange, iLen;
	
		if (oInput.createTextRange)
		{
			//ie6-8
			oTextRange = oInput.createTextRange();
			oTextRange.collapse(false);
			oTextRange.select();
		}
		else if (oInput.setSelectionRange)
		{
			// ff, opera, ie9
			// Double the length because Opera is inconsistent about whether a carriage return is one character or two. Sigh.
			iLen = oInput.value.length * 2;
			oInput.setSelectionRange(iLen, iLen);
		}
	};
	
	/**
	 * @param {?} oObject
	 * @param {string} sDelegateName
	 * @param {Array=} aParameters
	 */
	Utils.delegateRun = function (oObject, sDelegateName, aParameters)
	{
		if (oObject && oObject[sDelegateName])
		{
			oObject[sDelegateName].apply(oObject, _.isArray(aParameters) ? aParameters : []);
		}
	};
	
	/**
	 * @param {string} input
	 * @param {number} multiplier
	 * @return {string}
	 */
	Utils.strRepeat = function (input, multiplier)
	{
		return (new Array(multiplier + 1)).join(input);
	};
	
	
	Utils.deferredUpdate = function (element, state, duration, callback) {
		
		if (!element.__interval && !!state)
		{
			element.__state = true;
			callback(element, true);
	
			element.__interval = window.setInterval(function () {
				if (!element.__state)
				{
					callback(element, false);
					window.clearInterval(element.__interval);
					element.__interval = null;
				}
			}, duration);
		}
		else if (!state)
		{
			element.__state = false;
		}
	};
	
	Utils.draggableMessages = function ()
	{
		return $('<div class="draggable draggableMessages"><div class="content"><span class="count-text"></span></div></div>').appendTo('#pSevenHidden');
	};
	
	Utils.draggableContacts = function ()
	{
		return $('<div class="draggable draggableContacts"><div class="content"><span class="count-text"></span></div></div>').appendTo('#pSevenHidden');
	};
	
	Utils.removeActiveFocus = function ()
	{
		if (document && document.activeElement && document.activeElement.blur)
		{
			var oA = $(document.activeElement);
			if (oA.is('input') || oA.is('textarea'))
			{
				document.activeElement.blur();
			}
		}
	};
	
	Utils.uiDropHelperAnim = function (oEvent, oUi)
	{
		var 
			helper = oUi.helper.clone().appendTo('#pSevenHidden'),
			target = $(oEvent.target).find('.animGoal'),
			position = target[0] ? target.offset() : $(oEvent.target).offset()
		;
	
		helper.animate({
			'left': position.left + 'px',
			'top': position.top + 'px',
			'font-size': '0px',
			'opacity': 0
		}, 800, 'easeOutQuint', function() {
			 $(this).remove();
		});
	};
	
	Utils.inFocus = function ()
	{
		var
			mTagName = document && document.activeElement ? document.activeElement.tagName : null,
			mContentEditable = document && document.activeElement ? document.activeElement.contentEditable : null
		;
		return 'INPUT' === mTagName || 'TEXTAREA' === mTagName || 'IFRAME' === mTagName || mContentEditable === 'true';
	};
	
	Utils.removeSelection = function ()
	{
		if (window.getSelection)
		{
			window.getSelection().removeAllRanges();
		}
		else if (document.selection)
		{
			document.selection.empty();
		}
	};
	
	Utils.getMonthNamesArray = function ()
	{
		var
			aMonthes = Utils.i18n('DATETIME/MONTH_NAMES').split(' '),
			iLen = 12,
			iIndex = aMonthes.length
		;
		
		for (; iIndex < iLen; iIndex++)
		{
			aMonthes[iIndex] = '';
		}
		
		return aMonthes;
	};
	
	/**
	 * http://docs.translatehouse.org/projects/localization-guide/en/latest/l10n/pluralforms.html?id=l10n/pluralforms
	 * 
	 * @param {string} sLang
	 * @param {number} iNumber
	 * 
	 * @return {number}
	 */
	Utils.getPlural = function (sLang, iNumber)
	{
		var iResult = 0;
		iNumber = Utils.pInt(iNumber);
	
		switch (sLang)
		{
			case 'Arabic':
				iResult = (iNumber === 0 ? 0 : iNumber === 1 ? 1 : iNumber === 2 ? 2 : iNumber % 100 >= 3 && iNumber % 100 <= 10 ? 3 : iNumber % 100 >= 11 ? 4 : 5);
				break;
			case 'Bulgarian':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Chinese-Simplified':
				iResult = 0;
				break;
			case 'Chinese-Traditional':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Czech':
				iResult = (iNumber === 1) ? 0 : (iNumber >= 2 && iNumber <= 4) ? 1 : 2;
				break;
			case 'Danish':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Dutch':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'English':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Estonian':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Finish':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'French':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'German':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Greek':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Hebrew':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Hungarian':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Italian':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Japanese':
				iResult = 0;
				break;
			case 'Korean':
				iResult = 0;
				break;
			case 'Latvian':
				iResult = (iNumber % 10 === 1 && iNumber % 100 !== 11 ? 0 : iNumber !== 0 ? 1 : 2);
				break;
			case 'Lithuanian':
				iResult = (iNumber % 10 === 1 && iNumber % 100 !== 11 ? 0 : iNumber % 10 >= 2 && (iNumber % 100 < 10 || iNumber % 100 >= 20) ? 1 : 2);
				break;
			case 'Norwegian':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Persian':
				iResult = 0;
				break;
			case 'Polish':
				iResult = (iNumber === 1 ? 0 : iNumber % 10 >= 2 && iNumber % 10 <= 4 && (iNumber % 100 < 10 || iNumber % 100 >= 20) ? 1 : 2);
				break;
			case 'Portuguese-Portuguese':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Portuguese-Brazil':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Romanian':
				iResult = (iNumber === 1 ? 0 : (iNumber === 0 || (iNumber % 100 > 0 && iNumber % 100 < 20)) ? 1 : 2);
				break;
			case 'Russian':
				iResult = (iNumber % 10 === 1 && iNumber % 100 !== 11 ? 0 : iNumber % 10 >= 2 && iNumber % 10 <= 4 && (iNumber % 100 < 10 || iNumber % 100 >= 20) ? 1 : 2);
				break;
			case 'Serbian':
				iResult = (iNumber % 10 === 1 && iNumber % 100 !== 11 ? 0 : iNumber % 10 >= 2 && iNumber % 10 <= 4 && (iNumber % 100 < 10 || iNumber % 100 >= 20) ? 1 : 2);
				break;
			case 'Spanish':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Swedish':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Thai':
				iResult = 0;
				break;
			case 'Turkish':
				iResult = (iNumber === 1 ? 0 : 1);
				break;
			case 'Ukrainian':
				iResult = (iNumber % 10 === 1 && iNumber % 100 !== 11 ? 0 : iNumber % 10 >= 2 && iNumber % 10 <= 4 && (iNumber % 100 < 10 || iNumber % 100 >= 20) ? 1 : 2);
				break;
			default:
				iResult = 0;
				break;
		}
	
		return iResult;
	};
	
	/**
	 * @param {string} sFile
	 * 
	 * @return {string}
	 */
	Utils.getFileExtension = function (sFile)
	{
		var 
			sResult = '',
			iIndex = sFile.lastIndexOf('.')
		;
		
		if (iIndex > -1)
		{
			sResult = sFile.substr(iIndex + 1);
		}
	
		return sResult;
	};
	
	/**
	 * @param {string} sFile
	 * 
	 * @return {string}
	 */
	Utils.getFileNameWithoutExtension = function (sFile)
	{
		var 
			sResult = sFile,
			iIndex = sFile.lastIndexOf('.')
		;
		if (iIndex > -1)
		{
			sResult = sFile.substr(0, iIndex);	
		}
		return sResult;
	};
	
	/**
	 * @param {Object} oElement
	 * @param {Object} oItem
	 */
	Utils.defaultOptionsAfterRender = function (oElement, oItem)
	{
		if (oItem)
		{
			if (!Utils.isUnd(oItem.disable))
			{
				ko.applyBindingsToNode(oElement, {
					'disable': oItem.disable
				}, oItem);
			}
		}
	};
	
	/**
	 * @param {string} sDateFormat
	 * 
	 * @return string
	 */
	Utils.getDateFormatForMoment = function (sDateFormat)
	{
		var sMomentDateFormat = 'MM/DD/YYYY';
		
		switch (sDateFormat)
		{
			case 'MM/DD/YYYY':
				sMomentDateFormat = 'MM/DD/YYYY';
				break;
			case 'DD/MM/YYYY':
				sMomentDateFormat = 'DD/MM/YYYY';
				break;
			case 'DD Month YYYY':
				sMomentDateFormat = 'DD MMMM YYYY';
				break;
		}
		
		return sMomentDateFormat;
	};
	
	/**
	 * @param {string} sDateFormat
	 * 
	 * @return string
	 */
	Utils.getDateFormatForDatePicker = function (sDateFormat)
	{
		var sDatePickerDateFormat = 'mm/dd/yy';
		
		switch (sDateFormat)
		{
			case 'MM/DD/YYYY':
				sDatePickerDateFormat = 'mm/dd/yy';
				break;
			case 'DD/MM/YYYY':
				sDatePickerDateFormat = 'dd/mm/yy';
				break;
			case 'DD Month YYYY':
				sDatePickerDateFormat = 'dd MM yy';
				break;
		}
		
		return sDatePickerDateFormat;
	};
	
	/**
	 * @return Array
	 */
	Utils.getDateFormatsForSelector = function ()
	{
		return _.map(AppData.App.DateFormats, function (sDateFormat) {
			switch (sDateFormat)
			{
				case 'MM/DD/YYYY':
					return {name: Utils.i18n('DATETIME/DATEFORMAT_MMDDYYYY'), value: sDateFormat};
				case 'DD/MM/YYYY':
					return {name: Utils.i18n('DATETIME/DATEFORMAT_DDMMYYYY'), value: sDateFormat};
				case 'DD Month YYYY':
					return {name: Utils.i18n('DATETIME/DATEFORMAT_DDMONTHYYYY'), value: sDateFormat};
				default:
					return {name: sDateFormat, value: sDateFormat};
			}
		});
	};
	
	/**
	 * @param {string} sSubject
	 * 
	 * @return {string}
	 */
	Utils.getTitleForEvent = function (sSubject)
	{
		var
			sTitle = Utils.trim(sSubject.replace(/[\n\r]/, ' ')),
			iFirstSpacePos = sTitle.indexOf(' ', 180)
		;
		
		if (iFirstSpacePos >= 0)
		{
			sTitle = sTitle.substring(0, iFirstSpacePos) + '...';
		}
		
		if (sTitle.length > 200)
		{
			sTitle = sTitle.substring(0, 200) + '...';
		}
		
		return sTitle;
	};
	
	/**
	 * @param {string} sPhone
	 */
	Utils.getFormattedPhone = function (sPhone)
	{
		var
			oPrefixes = {
				'+7': '8'
			},
			sCleanedPhone = (/#/g).test(sPhone) ? sPhone.split('#')[1] : sPhone.replace(/[()\s_\-]/g, ''), //sPhone.replace(/\D \+/g, "")
			sFirstTwoSymbols = (sPhone + '').slice(0,2),
			bIsInList
		;
	
		bIsInList = _.any(oPrefixes, function (val, key) {
			return key === sFirstTwoSymbols;
		}, this);
	
		return bIsInList ? sCleanedPhone.replace(sFirstTwoSymbols, oPrefixes[sFirstTwoSymbols]) : sCleanedPhone.replace(/[+]/g, '');
	};
	
	Utils.desktopNotify = (function () {
	
		var notification,
			timeoutID = 0
		;
	
		/**
		 * @param {string} sAction
		 * @param {string=} sTitle
		 * @param {string=} sBody
		 * @param {string=} sIcon
		 * @param {Function=} fnCallback
		 * @param {number=} iTimeout
		 */
		return function (sAction, sTitle, sBody, sIcon, fnCallback, iTimeout)
		{
			if (sAction === 'show')
			{
				var self = this,
					winNotification = null,
					iPermission,
					oIcon = {
						'phone': 'skins/Default/images/logo.png'
					},
					oOptions = { // https://developer.mozilla.org/en-US/docs/Web/API/Notification
						body: sBody,
						//dir: "auto",// The direction of the notification; it can be auto, ltr, or rtl
						//lang: "",// Specify the lang used within the notification. This string must be a valid BCP 47 language tag
						//tag: 'test',// An ID for a given notification that allows to retrieve, replace or remove it if necessary
						icon: oIcon[sIcon] || false
					}
				;
	
				if (window.Notification && window.Notification.permission)
				{
					winNotification = window.Notification;
	
					switch (window.Notification.permission.toLowerCase())
					{
						case 'granted':
							iPermission = Enums.DesktopNotifications.Allowed;// 0
							break;
						case 'denied':
							iPermission = Enums.DesktopNotifications.Denied;// 2
							break;
						case 'default':
							iPermission = Enums.DesktopNotifications.NotAllowed;// 1
							break;
					}
				}
				else if (window.webkitNotifications && window.webkitNotifications.checkPermission)
				{
					winNotification = window.webkitNotifications;
					iPermission = window.webkitNotifications.checkPermission();
				}
	
				//	if (winNotification && iPermission !== Enums.DesktopNotifications.Allowed) {winNotification.requestPermission()}
	
				if (winNotification && iPermission === Enums.DesktopNotifications.Allowed)
				{
					if (notification) {
						notification.close();
					}
	
					notification = new window.Notification(sTitle, oOptions);
	
					clearTimeout(timeoutID);
					if(iTimeout) {
						timeoutID = setTimeout(function() { notification.close(); }, iTimeout);
					}
	
					// events
					notification.onclick = function ()
					{
						if(fnCallback)
						{
							fnCallback();
						}
						notification.close();
					};
					/*notification.onshow = function () {};
					notification.onclose = function () {};
					notification.onerror = function () {};*/
				}
			}
			else if (sAction === 'hide' && notification)
			{
				notification.close();
			}
		};
	}());
	
	/**
	 * @return {boolean}
	 */
	Utils.isRTL = function ()
	{
		return $html.hasClass('rtl');
	};
	
	(function ($) {
	 
	 /**
	  * @param {{name:string,resizeFunc:Function}} args
	  */
	 $.fn.splitter = function(args){
		args = args || {};
	
		return this.each(function() {
			var
				bIsMouseSplit = false,
				storageKey = args.name,
				startSplitMouse = function (e) {
					bIsMouseSplit = true;
					bar.addClass(opts['activeClass']);
	
					opts['_posSplit'] = -((rtl ? splitter._overallWidth - e[opts['eventPos']] : e[opts['eventPos']]) - panes.get(0)[opts['pxSplit']] );
					
					$('body')
						.attr({'unselectable': "on"})
						.addClass('unselectable');
	
					$(document)
						.bind('mousemove', doSplitMouse)
						.bind('mouseup', endSplitMouse);
				},
				doSplitMouse = function (e) {
					var newPos = (rtl ? splitter._overallWidth - e[opts['eventPos']] : e[opts['eventPos']]) + opts['_posSplit'];
					resplit(newPos);
					
					if (Utils.isFunc(args.resizeFunc))
					{
						args.resizeFunc();
					}
				},
				endSplitMouse = function endSplitMouse(e) {
					bar.removeClass(opts['activeClass']);
	
					$('body')
						.attr({'unselectable': 'off'})
						.removeClass('unselectable');
	
					// Store 'width' data
					if (storageKey)
					{
						App.Storage.setData(storageKey + 'ResizerWidth', panes.get(0)[opts['pxSplit']]);
					}
	
					$(document)
						.unbind('mousemove', doSplitMouse)
						.unbind('mouseup', endSplitMouse);
					
					if (Utils.isFunc(args.resizeFunc))
					{
						args.resizeFunc();
					}
				},
				resplit = function (newPosition) {
	
					// Constrain new splitbar position to fit pane size limits
					newPosition = window.Math.max(
						panes.get(0)._min, splitter._overallWidth - panes.get(1)._max,
						window.Math.min(newPosition, panes.get(0)._max, splitter._overallWidth - panes.get(1)._min)
					);
	
					panes.get(0).$.css(opts['split'], newPosition);
					panes.get(1).$.css(opts['split'], splitter._overallWidth - newPosition);
	
					if (!App.browser.ie8AndBelow)
					{
						panes.trigger('resize');
					}
				},
				dimSum = function (elem, dims) {
					// Opera returns -1 for missing min/max width, turn into 0
					var sum = 0;
					for (var i = 1; i < arguments.length; i++)
					{
						sum += window.Math.max(window.parseInt(elem.css(arguments[i]), 10) || 0, 0);
					}
					
					return sum;
				},
				vh = (args.splitHorizontal ? 'h' : args.splitVertical ? 'v' : args.type) || 'v',
				opts = $.extend({
					'activeClass': 'active',	// class name for active splitter
					'pxPerKey': 8,			// splitter px moved per keypress
					'tabIndex': 0,			// tab order indicator
					'accessKey': ''			// accessKey for splitbar
				},{
					v: {					// Vertical splitters:
						'keyLeft': 39, 'keyRight': 37,
						'type': 'v', 'eventPos': "pageX", 'origin': "left",
						'split': "width",  'pxSplit': "offsetWidth",  'side1': "Left", 'side2': "Right",
						'fixed': "height", 'pxFixed': "offsetHeight", 'side3': "Top",  'side4': "Bottom"
					},
					h: {					// Horizontal splitters:
						'keyTop': 40, 'keyBottom': 38,
						'type': 'h', 'eventPos': "pageY", 'origin': "top",
						'split': "height", 'pxSplit': "offsetHeight", 'side1': "Top",  'side2': "Bottom",
						'fixed': "width",  'pxFixed': "offsetWidth",  'side3': "Left", 'side4': "Right"
					}
				}[vh], args),
				
				splitter = $(this).css({'position': 'relative'}),
				panes = $(">*:not(css3pie)", splitter).each(function(){this.$ = $(this);}),
				bar = $('.resize_handler', panes.get(0))
					.attr({'unselectable': 'on'})
					.bind('mousedown', startSplitMouse),
				rtl = splitter.css('direction') === 'rtl'
			;
	
			panes.get(0)._paneName = opts['side1'];
			panes.get(1)._paneName = opts['side2'];
			
			panes.each(function(){
				this._min = opts['min' + this._paneName] || dimSum(this.$, 'min-' + opts['split']);
				this._max = opts['max' + this._paneName] || dimSum(this.$, 'max-' + opts['split']) || 9999;
				this._init = opts['size' + this._paneName] === true ?
					window.parseInt($.css(this, opts['split']), 10) : opts['size' + this._paneName];
			});
	
			// Determine initial position, get from cookie if specified
			var initPosition = 0;
			if (storageKey)
			{
				initPosition = App.Storage.getData(storageKey + 'ResizerWidth') || panes.get(0)._init;
			}
			else
			{
				initPosition = panes.get(0)._init;
			}
	
			if (!isNaN(panes.get(1)._init))	// recalc initial B size as an offset from the top or left side
			{
				initPosition = splitter[0][opts['pxSplit']] - panes.get(1)._init;
			}
			
			if (isNaN(initPosition))
			{
				initPosition = splitter[0][opts['pxSplit']];
				initPosition = window.Math.round(initPosition / panes.length);
			}
			
			// Resize event propagation and splitter sizing
			if (opts['resizeToWidth'] && !(App.browser.ie8AndBelow))
			{
				$(window).bind('resize', function(e) {
					if (e.target !== this)
					{
						return;
					}
					splitter.trigger('resize'); 
				});
			}
	
			var
				nSize = 0,
				oLastState = {},
				oLastStateReserve = {}
			;
			splitter.bind('resize', function (ev, size, command) {
				var tKey = ev.target.className + '_' + command;
	
				if(bIsMouseSplit)
				{
					oLastState = {};
				}
	
				// Custom events bubble in jQuery 1.3; don't get into a Yo Dawg
				if (ev.target !== this)
				{
					return;
				}
	
				// Determine new width/height of splitter container
				splitter._overallWidth = splitter[0][opts['pxSplit']];
	
				// Return if splitter isn't visible or content isn't there yet
				if (splitter._overallWidth <= 0)
				{
					return;
				}
	
				if (!(opts['sizeRight'] || opts['sizeBottom']))
				{
					nSize = panes.get(0)[opts['pxSplit']];
				}
				else
				{
					nSize = splitter._overallWidth - panes.get(1)[opts['pxSplit']];
				}
	
				if (isNaN(size))
				{
					size = nSize;
				}
				else if (command)
				{
					bIsMouseSplit = false;
	
					if (oLastState[tKey])
					{
						size = oLastState[tKey];
						oLastState[tKey] = null;
					}
					else
					{
						if (size === nSize)
						{
							oLastState[tKey] = null;
							size = oLastStateReserve[tKey];
						}
						else
						{
							oLastState[tKey] = oLastStateReserve[tKey] = nSize;
						}
	
						_.each(oLastState, function(num, key) {
							if (key !== tKey)
							{
								oLastState[key] = null;
							}
						});
					}
				}
	
				resplit(size);
				
			}).trigger('resize', [initPosition]);
		});
	};
	
	})(jQuery);
	
	/**
	 * @constructor
	 */
	function CApi()
	{
		
	}
	
	/**
	 * @param {string} sToAddresses
	 */
	CApi.prototype.openComposeMessage = function (sToAddresses)
	{
		App.Routing.setHash(App.Links.composeWithToField(sToAddresses));
	};
	
	/**
	 * @param {string} sLoading
	 */
	CApi.prototype.showLoading = function (sLoading)
	{
		App.Screens.showLoading(sLoading);
	};
	
	CApi.prototype.hideLoading = function ()
	{
		App.Screens.hideLoading();
	};
	
	/**
	 * @param {string} sReport
	 * @param {number=} iDelay
	 */
	CApi.prototype.showReport = function (sReport, iDelay)
	{
		App.Screens.showReport(sReport, iDelay);
	};
	
	/**
	 * @param {string} sError
	 * @param {boolean=} bHtml = false
	 * @param {boolean=} bNotHide = false
	 */
	CApi.prototype.showError = function (sError, bHtml, bNotHide)
	{
		App.Screens.showError(sError, bHtml, bNotHide);
	};
	CApi.prototype.hideError = function (sError, bHtml, bNotHide)
	{
		App.Screens.hideError();
	};
	
	/**
	 * @param {number} iErrorCode
	 * @param {string=} sDefaultError
	 */
	CApi.prototype.showErrorByCode = function (iErrorCode, sDefaultError)
	{
		switch (iErrorCode)
		{
			case Enums.Errors.AuthError:
				this.showError(Utils.i18n('WARNING/LOGIN_PASS_INCORRECT'));
				break;
			case Enums.Errors.DemoLimitations:
				this.showError(Utils.i18n('DEMO/WARNING_THIS_FEATURE_IS_DISABLED'));
				break;
			case Enums.Errors.Recaptcha:
				this.showError(Utils.i18n('WARNING/CAPTCHA_IS_INCORRECT'));
				break;
			case Enums.Errors.CanNotGetMessage:
				this.showError(Utils.i18n('MESSAGE/ERROR_MESSAGE_DELETED'));
				break;
			case Enums.Errors.CanNotChangePassword:
				this.showError(Utils.i18n('WARNING/UNABLE_CHANGE_PASSWORD'));
				break;
			case Enums.Errors.AccountOldPasswordNotCorrect:
				this.showError(Utils.i18n('WARNING/CURRENT_PASSWORD_NOT_CORRECT'));
				break;
			case Enums.Errors.FetcherIncServerNotAvailable:
				this.showError(Utils.i18n('WARNING/FETCHER_SAVE_ERROR'));
				break;
			case Enums.Errors.FetcherLoginNotCorrect:
				this.showError(Utils.i18n('WARNING/FETCHER_SAVE_ERROR'));
				break;
			case Enums.Errors.HelpdeskUserNotExists:
				this.showError(Utils.i18n('HELPDESK/ERROR_FORGOT_NO_ACCOUNT'));
				break;
			case Enums.Errors.MailServerError:
				this.showError(Utils.i18n('WARNING/CANT_CONNECT_TO_SERVER'));
				break;
			case Enums.Errors.DataTransferFailed:
				this.showError(Utils.i18n('WARNING/DATA_TRANSFER_FAILED'));
				break;
			case Enums.Errors.NotDisplayedError:
				break;
			default:
				if (sDefaultError && sDefaultError.length > 0)
				{
					this.showError(sDefaultError);
				}
				break;
		}
	};
	
	/**
	 * @constructor
	 */
	function CStorage()
	{
		Data.init();
	}
	
	CStorage.prototype.setData = function (key, value)
	{
		Data.setVar(key, value);
	};
	
	CStorage.prototype.removeData = function (key)
	{
		Data.setVar(key, '');
	};
	
	CStorage.prototype.getData = function (key)
	{
		return Data.getVar(key);
	};
	
	CStorage.prototype.hasData = function (key)
	{
		return Data.hasVar(key);
	};
	/**
	 * @constructor
	 */
	function AlertPopup()
	{
		this.alertDesc = ko.observable('');
		this.closeCallback = null;
		this.title = ko.observable('');
		this.okButtonText = ko.observable(Utils.i18n('MAIN/BUTTON_OK'));
	}
	
	/**
	 * @param {string} sDesc
	 * @param {Function=} fCloseCallback = null
	 * @param {string=} sTitle = ''
	 * @param {string=} sOkButtonText = 'Ok'
	 */
	AlertPopup.prototype.onShow = function (sDesc, fCloseCallback, sTitle, sOkButtonText)
	{
		this.alertDesc(sDesc);
		this.closeCallback = fCloseCallback || null;
		this.title(sTitle || '');
		this.okButtonText(sOkButtonText || Utils.i18n('MAIN/BUTTON_OK'));
	};
	
	/**
	 * @return {string}
	 */
	AlertPopup.prototype.popupTemplate = function ()
	{
		return 'Popups_AlertPopupViewModel';
	};
	
	AlertPopup.prototype.onEnterHandler = function ()
	{
		this.close();
	};
	
	AlertPopup.prototype.close = function ()
	{
		if (Utils.isFunc(this.closeCallback))
		{
			this.closeCallback();
		}
		this.closeCommand();
	};
	
	/**
	 * @constructor
	 */
	function ConfirmPopup()
	{
		this.fConfirmCallback = null;
		this.confirmDesc = ko.observable('');
		this.title = ko.observable('');
		this.okButtonText = ko.observable(Utils.i18n('MAIN/BUTTON_OK'));
		this.cancelButtonText = ko.observable(Utils.i18n('MAIN/BUTTON_CANCEL'));
	}
	
	/**
	 * @param {string} sDesc
	 * @param {Function} fConfirmCallback
	 * @param {string=} sTitle = ''
	 * @param {string=} sOkButtonText = ''
	 * @param {string=} sCancelButtonText = ''
	 */
	ConfirmPopup.prototype.onShow = function (sDesc, fConfirmCallback, sTitle, sOkButtonText, sCancelButtonText)
	{
		this.title(sTitle || '');
		this.okButtonText(sOkButtonText || Utils.i18n('MAIN/BUTTON_OK'));
		this.cancelButtonText(sCancelButtonText || Utils.i18n('MAIN/BUTTON_CANCEL'));
		if (Utils.isFunc(fConfirmCallback))
		{
			this.fConfirmCallback = fConfirmCallback;
			this.confirmDesc(sDesc);
		}
	};
	
	/**
	 * @return {string}
	 */
	ConfirmPopup.prototype.popupTemplate = function ()
	{
		return 'Popups_ConfirmPopupViewModel';
	};
	
	ConfirmPopup.prototype.onEnterHandler = function ()
	{
		this.yesClick();
	};
	
	ConfirmPopup.prototype.yesClick = function ()
	{
		if (this.fConfirmCallback)
		{
			this.fConfirmCallback(true);
		}
	
		this.closeCommand();
	};
	
	ConfirmPopup.prototype.noClick = function ()
	{
		if (this.fConfirmCallback)
		{
			this.fConfirmCallback(false);
		}
	
		this.closeCommand();
	};
	
	
	
	
	
	/**
	 * @constructor
	 */
	function CalendarCreatePopup()
	{
		this.fCallback = null;
		
		this.calendarId = ko.observable(null);
		this.calendarName = ko.observable('');
		this.calendarDescription = ko.observable('');
		
		this.calendarNameFocus = ko.observable(false);
		this.calendarDescriptionFocus = ko.observable(false);
		
		this.colors = ko.observableArray([]);
		this.selectedColor = ko.observable(this.colors()[0]);
	}
	
	CalendarCreatePopup.prototype.clearFields = function ()
	{
		this.calendarName('');
		this.calendarDescription('');
		this.selectedColor(this.colors[0]);
		this.calendarId(null);
	};
	
	/**
	 * @param {Function} fCallback
	 * @param {Array} aColors
	 * @param {Object} oCalendar
	 */
	CalendarCreatePopup.prototype.onShow = function (fCallback, aColors, oCalendar)
	{
		this.clearFields();
		if (Utils.isFunc(fCallback))
		{
			this.fCallback = fCallback;
		}
		if (!Utils.isUnd(aColors))
		{
			this.colors(aColors);
			this.selectedColor(aColors[0]);		
		}
		if (!Utils.isUnd(oCalendar))
		{
			this.calendarName(oCalendar.name ? oCalendar.name() : '');
			this.calendarDescription(oCalendar.description ? oCalendar.description() : '');
			this.selectedColor(oCalendar.color ? oCalendar.color() : '');
			this.calendarId(oCalendar.id ? oCalendar.id : null);
		}
	};
	
	/**
	 * @return {string}
	 */
	CalendarCreatePopup.prototype.popupTemplate = function ()
	{
		return 'Popups_CalendarCreatePopupViewModel';
	};
	
	CalendarCreatePopup.prototype.onSaveClick = function ()
	{
		if (this.calendarName() === '')
		{
			App.Screens.showPopup(AlertPopup, [Utils.i18n('CALENDAR/WARNING_BLANK_CALENDAR_NAME')]);
		}
		else
		{
			if (this.fCallback)
			{
				this.fCallback(this.calendarName(), this.calendarDescription(), this.selectedColor(), this.calendarId());
				this.clearFields();
			}
			this.closeCommand();
		}
	};
	
	CalendarCreatePopup.prototype.onCancelClick = function ()
	{
		this.closeCommand();
	};
	/**
	 * @constructor
	 */
	function CalendarSharePopup()
	{
		this.defaultAccount = AppData.Accounts.getDefault();
	
		this.fCallback = null;
	
		this.calendarId = ko.observable(null);
		this.selectedColor = ko.observable('');
		this.calendarUrl = ko.observable('');
		this.exportUrl = ko.observable('');
		this.icsLink = ko.observable('');
		this.isPublic = ko.observable(false);
		this.shares = ko.observableArray([]);
		this.oldShares = ko.observableArray([]);
		this.owner = ko.observable('');
	
		this.recivedAnim = ko.observable(false).extend({'autoResetToFalse': 500});
		this.whomAnimate = ko.observable('');
	
		this.shareAutocompleteItem = ko.observable(null);
		this.shareAutocompleteItem.subscribe(function (oItem) {
			if (oItem) { this.setGlobal(oItem.email, this.newShareList()); }
		}, this);
	
		this.newShareList = ko.observableArray([]);
		this.newShare = ko.observable('');
		this.newShareFocus = ko.observable(false);
		this.newShareAccess = ko.observable(2);
		this.canAdd = ko.observable(false);
		this.aAccess = [
			{'value': Enums.CalendarAccess.Read, 'display': Utils.i18n('CALENDAR/CALENDAR_ACCESS_READ')},
			{'value': Enums.CalendarAccess.Write, 'display': Utils.i18n('CALENDAR/CALENDAR_ACCESS_WRITE')}
		];
	
		this.autocompleteCallbackBinded = _.bind(this.autocompleteCallback, this);
	}
	
	/**
	 * @param {Function} fCallback
	 * @param {Object} oCalendar
	 */
	CalendarSharePopup.prototype.onShow = function (fCallback, oCalendar)
	{
		if (Utils.isFunc(fCallback))
		{
			this.fCallback = fCallback;
		}
		if (!Utils.isUnd(oCalendar))
		{
			this.selectedColor(oCalendar.color());
			this.calendarId(oCalendar.id);
			this.calendarUrl(oCalendar.davUrl() + oCalendar.url());
			this.exportUrl(oCalendar.exportUrl());
			this.icsLink(oCalendar.davUrl() + oCalendar.url() + '?export');
			this.isPublic(oCalendar.isPublic());
			this.shares(oCalendar.shares().slice(0));
			this.oldShares(oCalendar.shares().slice(0));
			this.owner(oCalendar.owner());
		}
	};
	
	/**
	 * @return {string}
	 */
	CalendarSharePopup.prototype.popupTemplate = function ()
	{
		return 'Popups_CalendarSharePopupViewModel';
	};
	
	CalendarSharePopup.prototype.onSaveClick = function ()
	{
		if (this.fCallback)
		{
			this.addShare();
			this.fCallback(this.calendarId(), this.isPublic(), this.shares());
		}
		this.closePopup();
	};
	
	CalendarSharePopup.prototype.onCancelClick = function ()
	{
		this.shares(this.oldShares());
		this.closePopup();
	};
	
	CalendarSharePopup.prototype.closePopup = function ()
	{
		this.cleanAll();
	
		this.closeCommand();
	};
	
	CalendarSharePopup.prototype.cleanAll = function ()
	{
		this.newShare('');
		this.newShareAccess(2);
		this.shareAutocompleteItem(null);
		this.canAdd(false);
	};
	
	CalendarSharePopup.prototype.addShare = function ()
	{
		var sEmail = this.shareAutocompleteItem() ? this.shareAutocompleteItem().email : this.newShare();
	
		if (this.canAdd())
		{
			this.shares.push(
				{
					name: this.shareAutocompleteItem() ? this.shareAutocompleteItem().name : '',
					email: sEmail,
					access: this.newShareAccess()
				});
	
			this.cleanAll();
		}
		else
		{
			this.whomAnimate(sEmail);
			this.recivedAnim(true);
		}
	};
	
	/**
	 * @param {Object} oItem
	 */
	CalendarSharePopup.prototype.removeShare = function (oItem)
	{
		this.shares.remove(oItem);
	};
	
	/**
	 * @param {string} sTerm
	 * @param {Function} fResponse
	 */
	CalendarSharePopup.prototype.autocompleteCallback = function (sTerm, fResponse)
	{
		var oParameters = {
				'Action': 'ContactSuggestions',
				'Search': sTerm,
				'GlobalOnly': '1'
			}
		;
	
		this.shareAutocompleteItem(null);
	
		sTerm = Utils.trim(sTerm);
		if ('' !== sTerm)
		{
			App.Ajax.send(oParameters, function (oData) {
				var aList = [];
				if (oData && oData.Result && oData.Result && oData.Result.List)
				{
					aList = _.map(oData.Result.List, function (oItem) {
						/*return oItem && oItem.Email ? oItem.Email : '';*/
						/*return oItem && oItem.Email ?
							(oItem.Name && 0 < Utils.trim(oItem.Name).length ?
								'"' + oItem.Name + '" <' + oItem.Email + '>' : oItem.Email) : '';*/
						return oItem && oItem.Email && oItem.Email !== this.owner() ?
							(oItem.Name && 0 < Utils.trim(oItem.Name).length ?
							{value:'"' + oItem.Name + '" <' + oItem.Email + '>', name: oItem.Name, email: oItem.Email} : {value: oItem.Email, name: '', email: oItem.Email}) : null;
					}, this);
	
					aList = _.compact(aList);
	
					this.newShareList(aList);
	
					this.setGlobal(this.newShare(), aList);
				}
				fResponse(aList);
			}, this);
		}
		else
		{
			fResponse([]);
		}
	};
	
	/**
	 * @param {string} sText
	 * @param {Array} aList
	 */
	CalendarSharePopup.prototype.setGlobal = function (sText, aList)
	{
		var
			isInGlobal = _.any(aList, function (sItem) {
				return sItem.email === sText;
			}, this),
			isAlreadyAdded = _.any(this.shares(), function (sItem) {
				return sItem.email === sText;
			}, this)
		;
	
		this.canAdd(isInGlobal && !isAlreadyAdded);
	};
	/**
	 * @constructor
	 */
	function CalendarGetLinkPopup()
	{
		this.fCallback = null;
	
		this.calendarId = ko.observable(null);
		this.selectedColor = ko.observable('');
		this.calendarUrl = ko.observable('');
		this.exportUrl = ko.observable('');
		this.icsLink = ko.observable('');
		this.isPublic = ko.observable(false);
		this.pubUrl = ko.observable('');
	}
	
	/**
	 * @param {Function} fCallback
	 * @param {Object} oCalendar
	 */
	CalendarGetLinkPopup.prototype.onShow = function (fCallback, oCalendar)
	{
		if (Utils.isFunc(fCallback))
		{
			this.fCallback = fCallback;
		}
		if (!Utils.isUnd(oCalendar))
		{
			this.selectedColor(oCalendar.color());
			this.calendarId(oCalendar.id);
			this.calendarUrl(oCalendar.davUrl() + oCalendar.url());
			this.exportUrl(oCalendar.exportUrl());
			this.icsLink(oCalendar.davUrl() + oCalendar.url() + '?export');
			this.isPublic(oCalendar.isPublic());
			this.pubUrl(oCalendar.pubUrl());
			this.exportUrl(oCalendar.exportUrl());
		}
	};
	
	/**
	 * @return {string}
	 */
	CalendarGetLinkPopup.prototype.popupTemplate = function ()
	{
		return 'Popups_CalendarGetLinkPopupViewModel';
	};
	
	CalendarGetLinkPopup.prototype.onCancelClick = function ()
	{
		if (this.fCallback)
		{
			this.fCallback(this.calendarId(), this.isPublic());
		}
		this.closeCommand();
	};
	/**
	 * @constructor
	 */
	function CalendarCreateEventPopup()
	{
		this.defaultAccount = (AppData.Accounts) ? AppData.Accounts.getDefault() : null;
	
		this.modified = false;
		this.isPublic = bExtApp;
		this.isEditable = ko.observable(false);
		this.selectedCalendarIsShared = ko.observable(false);
		this.selectedCalendarIsEditable = ko.observable(false);
	
		this.callbackSave = null;
		this.callbackDelete = null;
		this.timeFormatMoment = 'HH:mm';
		this.dateFormatMoment = 'MM/DD/YYYY';
		this.dateFormatDatePicker = 'mm/dd/yy';
	
		this.calendarId = ko.observable(null);
		this.id = ko.observable(null);
		this.uid = ko.observable(null);
		this.recurrenceId = ko.observable(null);
		this.allEvents = ko.observable(Enums.CalendarEditRecurrenceEvent.AllEvents);
	
		this.isMyEvent = ko.observable(false);
	
		this.startDom = ko.observable(null);
		this.endDom = ko.observable(null);
		this.repeatEndDom = ko.observable(null);
	
		this.yearlyDate = ko.observable('');
		this.monthlyDate = ko.observable('');
	
		this.subject = ko.observable('');
		this.description = ko.observable('');
	
		this.startDate = ko.observable('');
		this.startTime = ko.observable('');
		this.startTime.subscribe(function () {
			this.selectStartDate();
		}, this);
		this.allDay = ko.observable(false);
		this.allDay.subscribe(function (arg) {
			if(!arg)
			{
				this.setActualTime();
			}
		}, this);
	
		this.endDate = ko.observable('');
		this.endTime = ko.observable('');
		this.endTime.subscribe(function () {
			this.selectEndDate();
		}, this);
	
		this.repeatEndDate = ko.observable('');
	
		this.isEvOneDay = ko.observable(true);
		this.isEvOneTime = ko.observable(true);
	
		this.isRepeat = ko.observable(false);
	
		this.location = ko.observable('');
	
		this.repeatPeriodOptions = ko.observableArray(this.getDisplayedPeriods());
		this.repeatWeekIntervalOptions = ko.observableArray([1, 2, 3, 4]);
		this.repeatMonthIntervalOptions = ko.observableArray(this.getDisplayedIntervals());
		this.alarmOptions = ko.observableArray(this.getDisplayedAlarms([5, 10, 15, 30, 60, 120, 180, 240, 300, 360, 420, 480, 540, 600, 660, 720, 1080, 1440, 2880, 4320, 5760, 10080, 20160])); // available minutes array
		this.timeOptions = ko.observableArray(this.getDisplayedTimes((AppData.User.defaultTimeFormat() === Enums.TimeFormat.F24) ? 'HH:mm' : 'hh:mm A'));
		/*AppData.User.defaultTimeFormat.subscribe(function (iTimeFormat) {
			this.timeOptions(this.getDisplayedTimes((iTimeFormat === Enums.TimeFormat.F24) ? 'HH:mm' : 'hh:mm A'))
		}, this);*/
	
		this.displayedAlarms = ko.observableArray([]);
		this.displayedAlarms.subscribe(function (arg) {
			this.disableAlarms();
		}, this);
	
		this.excluded = ko.observable(false);
		this.repeatPeriod = ko.observable(0);
		this.repeatPeriod.subscribe(function (arg) {
			this.isRepeat(!!arg);
		}, this);
		this.repeatInterval = ko.observable(1);
		this.repeatEnd = ko.observable(0);
		this.repeatCount = ko.observable(null);
		this.repeatWeekNum = ko.observable(null);
	
		this.weekDays = ko.observable(false);
		this.weekMO = ko.observable(false);
		this.weekTU = ko.observable(false);
		this.weekWE = ko.observable(false);
		this.weekTH = ko.observable(false);
		this.weekFR = ko.observable(false);
		this.weekSA = ko.observable(false);
		this.weekSU = ko.observable(false);
	
		this.appointment = ko.observable(false);
		this.attendees = ko.observableArray([]);
		this.attenderStatus = ko.observable(0);
		this.owner = ko.observable('');
		this.ownerName = ko.observable('');
		this.ownerDisplay = ko.computed(function(){
			return (this.ownerName() !== '') ? this.ownerName() : this.owner();
		}, this);
	
		this.recivedAnim = ko.observable(false).extend({'autoResetToFalse': 500});
		this.whomAnimate = ko.observable('');
		this.guestAutocompleteItem = ko.observable(null);
		this.guestAutocomplete = ko.observable('');
		this.guestEmailFocus = ko.observable(false);
		this.guestAutocomplete.subscribe(function (sItem) {
			if (sItem === '')
			{
				this.guestAutocompleteItem(null);
			}
		}, this);
	
		this.condition = ko.observable('');
	
		this.autosizeTrigger = ko.observable(true);
	
		this.calendars = null;
		this.visibleCalendarList = ko.observable(false);
		this.calendarsList = ko.observableArray([]);
		this.calendarsList.subscribe(function () {
			if (this.calendars && this.calendars.count() > 1)
			{
				this.visibleCalendarList(true);
			}
			else
			{
				this.visibleCalendarList(false);
			}
		}, this);
		this.calendarColor = ko.observable('');
		this.selectedCalendar = ko.observable('');
		this.selectedCalendarName = ko.observable('');
		this.selectedCalendar.subscribe(function (sValue) {
			if (sValue)
			{
				var oCalendar = this.calendars.getCalendarById(sValue);
				
				this.selectedCalendarName(oCalendar.name());
				this.selectedCalendarIsShared(oCalendar.isShared);
				this.visibleCalendarList(oCalendar.isEditable());
				this.selectedCalendarIsEditable(oCalendar.isEditable());
				this.changeCalendarColor(sValue);
			}
		}, this);
		
		this.subjectFocus = ko.observable(false);
		this.subjectFocus.subscribe(function (sValue) {
	
		}, this);
		this.descriptionFocus = ko.observable(false);
		this.locationFocus = ko.observable(false);
	
		this.dateEdit = ko.observable(false);
		this.repeatEdit = ko.observable(false);
		this.guestsEdit = ko.observable(false);
		this.defaultOptionsAfterRender = Utils.defaultOptionsAfterRender;
		this.isEditForm = ko.computed(function(){
			return !!this.id();
		}, this);
	
		this.callbackAttendeeActionDecline = null;
	
		this.calendarAppointments = AppData.User.AllowCalendar && AppData.User.CalendarAppointments;
	
		this.allChanges = ko.computed(function() {
			this.subject();
			this.description();
			this.location();
			this.isRepeat();
			this.allDay();
			this.repeatPeriod();
			this.repeatInterval();
			this.repeatEnd();
			this.repeatCount();
			this.repeatWeekNum();
			this.startDate();
			this.startTime();
			this.endDate();
			this.repeatEndDate();
			this.displayedAlarms();
			this.weekDays();
			this.weekMO();
			this.weekTU();
			this.weekWE();
			this.weekTH();
			this.weekFR();
			this.weekSA();
			this.weekSU();
			this.attendees();
			this.selectedCalendar();
	
			this.modified = true;
		}, this);
	
	}
	
	/**
	 * @param {Object} oElement
	 * @param {Function} fSelect
	 */
	CalendarCreateEventPopup.prototype.createDatePickerObject = function (oElement, fSelect)
	{
		$(oElement).datepicker({
			showOtherMonths: true,
			selectOtherMonths: true,
			monthNames: Utils.getMonthNamesArray(),
			dayNamesMin: Utils.i18n('DATETIME/DAY_NAMES_MIN').split(' '),
			firstDay: AppData.User.CalendarWeekStartsOn,
			showOn: "both",
			buttonText: "",
			buttonImage: "skins/Default/images/calendar-icon.png",
			buttonImageOnly: true,
			dateFormat: this.dateFormatDatePicker,
			onSelect: fSelect
		});
	
		$(oElement).mousedown(function() {
			$('#ui-datepicker-div').toggle();
		});
	};
	
	
	CalendarCreateEventPopup.prototype.initializeDatePickers = function ()
	{
		this.createDatePickerObject(this.startDom(), this.selectStartDate.bind(this));
		this.createDatePickerObject(this.endDom(), this.selectEndDate.bind(this));
		this.createDatePickerObject(this.repeatEndDom(), Utils.emptyFunction());
	};
	
	/**
	 * @param {Object} oParameters
	 */
	CalendarCreateEventPopup.prototype.onShow = function (oParameters)
	{
		var
			oAccount = this.defaultAccount,
			owner = (oAccount) ? oAccount.email() : '',
			ownerName = (oAccount) ? oAccount.friendlyName() : ''
		;
	
		this.isMyEvent(owner === (oParameters.Owner || owner));
	
		this.callbackSave = oParameters.CallbackSave;
		this.callbackDelete = oParameters.CallbackDelete;
		this.callbackAttendeeActionDecline = oParameters.CallbackAttendeeActionDecline;
	
		this.timeFormatMoment = oParameters.TimeFormat;
		this.dateFormatMoment = Utils.getDateFormatForMoment(oParameters.DateFormat);
		this.dateFormatDatePicker = Utils.getDateFormatForDatePicker(oParameters.DateFormat);
		
		this.initializeDatePickers();
	
		this.startDate(oParameters.StartMoment.format(this.dateFormatMoment));
		this.startDom().datepicker("setDate", oParameters.StartMoment.toDate());
		this.startTime(oParameters.StartMoment.format(this.timeFormatMoment));
	
		this.endDate(oParameters.EndMoment ? oParameters.EndMoment.format(this.dateFormatMoment) : this.startDate());
		this.endDom().datepicker("setDate", oParameters.EndMoment ? oParameters.EndMoment.toDate() : oParameters.StartMoment.toDate());
		this.endTime(oParameters.EndMoment ? oParameters.EndMoment.format(this.timeFormatMoment) : this.startTime());
	
		this.allDay(oParameters.AllDay);
	
		this.calendars = oParameters.Calendars;
		if (this.calendars)
		{
			this.calendarsList(this.calendars.collection);
		}
		this.selectedCalendar(oParameters.SelectedCalendar);
		this.calendarId(this.selectedCalendar());
		this.editableSwitch(this.selectedCalendarIsShared(), this.selectedCalendarIsEditable(), this.isMyEvent());
	
		this.changeCalendarColor(this.selectedCalendar());
		
		// parameters for event editing only (not for creating)
		this.id(oParameters.ID || null);
		this.uid(oParameters.Uid || null);
		this.recurrenceId(oParameters.RecurrenceId || null);
		
		this.subject(oParameters.Subject || '');
		this.location(oParameters.Location || '');
		this.description(oParameters.Description || '');
		this.allEvents(oParameters.AllEvents || Enums.CalendarEditRecurrenceEvent.AllEvents);
	
		this.displayedAlarms(this.getDisplayedAlarms(oParameters.Alarms || []));
	
		this.appointment(oParameters.Appointment);
	
		this.attendees(oParameters.Attendees || []);
		this.setCurrentAttenderStatus(owner, oParameters.Attendees || []);
	
		this.owner(oParameters.Owner || owner);
		this.ownerName(oParameters.OwnerName || ownerName);
	/*	
		if (oParameters.OwnerName === '')
		{
			this.ownerName(oParameters.OwnerName);
		}
		else
		{
			this.ownerName(ownerName);
		}
	*/	
		this.guestAutocomplete('');
	
		this.excluded(oParameters.Excluded || false);
		this.repeatRuleParse(oParameters.RRule || null);
	
		if (this.id() === null) {
			this.subjectFocus(true);
		}
	
		this.selectStartDate();
		this.selectEndDate();
	
		this.autosizeTrigger.notifySubscribers(true);
		
		this.modified = false;
	};
	
	/**
	 * @return {string}
	 */
	CalendarCreateEventPopup.prototype.popupTemplate = function ()
	{
		return 'Popups_CalendarCreateEventPopupViewModel';
	};
	
	/**
	 * @param {string} sId
	 */
	CalendarCreateEventPopup.prototype.changeCalendarColor = function (sId)
	{
		if (Utils.isFunc(this.calendars.getCalendarById))
		{
			var oCalendar = this.calendars.getCalendarById(sId);
			if (oCalendar)
			{
				this.calendarColor('');
				this.calendarColor(oCalendar.color());
			}
		}
	};
	
	/**
	 * @param {string} sId
	 */
	CalendarCreateEventPopup.prototype.isVisibleCalendarListItem = function (sId)
	{
		var 
			bResult = false,
			oCalendar = this.calendars.getCalendarById(sId)
		;
		if (oCalendar)
		{
			bResult = (this.selectedCalendar() !== sId && oCalendar.isEditable());
		}
		return bResult;
	};
	
	CalendarCreateEventPopup.prototype.onSaveClick = function ()
	{
		if (this.subject() === '')
		{
			App.Screens.showPopup(AlertPopup, [Utils.i18n('CALENDAR/WARNING_EVENT_BLANK_SUBJECT'),
				_.bind(function () {
					this.subjectFocus(true);
				}, this)]);
		}
		else
		{
			if (this.callbackSave)
			{
				var
					iPeriod = this.repeatPeriod(),
					sDate = '',
					oDate = null,
					iInterval = 0,
					iWeekNum = 0,
					iEnd = 0,
					oEventData = {
						calendarId: this.calendarId(),
						newCalendarId: this.selectedCalendar(),
						id: this.id(),
						uid: this.uid(),
						recurrenceId: this.recurrenceId(),
						allEvents:  this.allEvents(),
						subject: this.subject(),
						title: Utils.getTitleForEvent(this.subject()),
						start: this.getDateFromStr(this.startDate(), this.startTime()),
						end: this.getDateFromStr(this.endDate(), this.endTime()),
						allDay: this.allDay(),
						location: this.location(),
						description: this.description(),
						alarms: this.getAlarmsArray(this.displayedAlarms()),
						attendees: this.attendees(),
						owner: this.owner(),
						modified: this.modified
					}
				;
	
				if (iPeriod)
				{
					sDate = this.repeatEndDom().datepicker('getDate');
					oDate = sDate ? this.getUnixTimestamp(sDate) : null;
	
					iInterval = this.repeatInterval();
					iWeekNum = this.repeatWeekNum();
					iEnd = this.repeatEnd();
	
					if (iPeriod === Enums.CalendarRepeatPeriod.Daily)
					{
						oEventData.rrule = {
							byDays: [],
							count: null,
							end: 2,
							interval: 1,
							period: iPeriod,
							until: oDate,
							weekNum: null
						};
					}
					else if (iPeriod === Enums.CalendarRepeatPeriod.Weekly)
					{
						oEventData.rrule = {
							byDays: this.getDays(),
							count: null,
							end: 0,
							interval: iInterval,
							period: iPeriod,
							until: null,
							weekNum: null
						};
					}
					else if (iPeriod === Enums.CalendarRepeatPeriod.Monthly)
					{
						oEventData.rrule = {
							byDays: [],
							count: null,
							end: 0,
							interval: 1,
							period: iPeriod,
							until: null,
							weekNum: null
						};
					}
					else if (iPeriod === Enums.CalendarRepeatPeriod.Yearly)
					{
						oEventData.rrule = {
							byDays: [],
							count: null,
							end: 0,
							interval: 1,
							period: iPeriod,
							until: null,
							weekNum: null
						};
					}
				}
	
				this.callbackSave(oEventData);
			}
	
			this.closePopup();
		}
	};
	
	CalendarCreateEventPopup.prototype.closePopup = function ()
	{
		this.hideAll();
		this.cleanAll();
	
		this.closeCommand();
	};
	
	CalendarCreateEventPopup.prototype.hideAll = function ()
	{
		this.dateEdit(false);
		this.repeatEdit(false);
		this.guestsEdit(false);
	};
	
	CalendarCreateEventPopup.prototype.cleanAll = function ()
	{
		this.repeatPeriod(0);
		this.repeatInterval(1);
	//	this.repeatUntil(null);
		this.repeatEnd(0);
		this.repeatCount(null);
		this.repeatWeekNum(null);
		this.weekDays(false);
		this.weekMO(false);
		this.weekTU(false);
		this.weekWE(false);
		this.weekTH(false);
		this.weekFR(false);
		this.weekSA(false);
		this.weekSU(false);
	
		this.attendees([]);
	};
	
	CalendarCreateEventPopup.prototype.onDeleteClick = function ()
	{
		if (this.callbackDelete)
		{
			var
				oEventData = {
					calendarId: this.selectedCalendar(),
					id: this.id(),
					uid: this.uid(),
					recurrenceId: this.recurrenceId(),
					allEvents:  this.allEvents(),
					subject: this.subject(),
					title: Utils.getTitleForEvent(this.subject()),
					start: this.getDateFromStr(this.startDate(), this.startTime()),
					end: this.getDateFromStr(this.endDate(), this.endTime()),
					allDay: this.allDay(),
					location: this.location(),
					description: this.description()
				}
			;
	
			this.callbackDelete(oEventData);
		}
		this.closePopup();
	};
	
	/**
	 * @param {Object} oModel
	 * @param {Object} oEv
	 */
	CalendarCreateEventPopup.prototype.showDates = function (oModel, oEv)
	{
		oEv.stopPropagation();
		this.dateEdit(!this.dateEdit());
	};
	
	CalendarCreateEventPopup.prototype.showGuests = function ()
	{
		if (this.attendees().length > 0)
		{
			var
				sConfirm = Utils.i18n('CALENDAR/CONFIRM_CLOSE_ATTENDEERS'),
				fAction = _.bind(function (bResult) {
					if (bResult)
					{
						this.guestsEdit(false);
						this.guestEmailFocus(false);
						this.attendees([]);
					}
				}, this)
			;
	
			App.Screens.showPopup(ConfirmPopup, [sConfirm, fAction]);
	
		}
		else
		{
			this.guestsEdit(!this.guestsEdit());
			this.guestEmailFocus(!this.guestEmailFocus());
		}
	};
	
	CalendarCreateEventPopup.prototype.onAddGuestClick = function ()
	{
		var
			oGuestAutocompleteItem = this.guestAutocompleteItem(),
			sGuestAutocomplete = this.guestAutocomplete(),
			oItem = oGuestAutocompleteItem || {name: '', email: sGuestAutocomplete},
			bIsInvited = _.any(this.attendees(), function (oEl) {
				return oEl.email === oItem.email;
			})
		;
	
		if (oItem.email === '')
		{
			App.Api.showErrorByCode(0, Utils.i18n('CALENDAR/EVENT_ERROR_ENTER_EMAIL'));
		}
		else if (oItem.email === this.owner())
		{
			this.recivedAnim(true);
		}
		else if (bIsInvited)
		{
			this.recivedAnim(true);
		}
	//	else if (!(/^([a-z0-9_\-]+\.)*[a-z0-9_\-]+@[a-z0-9_\-]+(\.[a-z0-9_\-]+)*\.[a-z]{2,6}$/).test(oItem.email))
	//	else if (!(/^([a-z0-9_\-]+\.)*[a-z0-9_\-]+@[a-z0-9_\-]+(\.[a-z0-9_\-]+)*/).test(oItem.email))
	//	{
	//		App.Api.showErrorByCode(0, Utils.i18n('CALENDAR/EVENT_ERROR_CORRECT_EMAIL'));
	//	}
		else
		{
			this.attendees.push(
				{
					status: 0,
					name: oItem.name,
					email: oItem.email
				}
			);
		}
	
		this.whomAnimate(oItem.email);
		this.guestAutocomplete('');
	};
	
	/**
	 * @param {Array} aMinutes
	 */
	CalendarCreateEventPopup.prototype.getDisplayedAlarms = function (aMinutes)
	{
		var
			alarm,
			sText,
			aDisplayedAlarms = []
		;
	
		_.each(aMinutes, function(iMinutes, iIdx)
		{
			alarm = this['alarm' + iMinutes] = ko.observable(iMinutes);
			alarm.subscribe(function() {
				//alarm observable value not actual
				this.disableAlarms();
			}, this);
	
			if (iMinutes > 0 && iMinutes < 60)
			{
				sText = (Utils.i18n('CALENDAR/ALARM_MINUTES_PLURAL', {
					'COUNT' :iMinutes
				}, null, iMinutes));
			}
			else if (iMinutes >= 60 && iMinutes < 1440)
			{
				sText = (Utils.i18n('CALENDAR/ALARM_HOURS_PLURAL', {
					'COUNT' :iMinutes/60
				}, null, iMinutes/60));
			}
			else if (iMinutes >= 1440 && iMinutes < 10080)
			{
				sText = (Utils.i18n('CALENDAR/ALARM_DAYS_PLURAL', {
					'COUNT' :iMinutes/1440
				}, null, iMinutes/1440));
			}
			else
			{
				sText = (Utils.i18n('CALENDAR/ALARM_WEEKS_PLURAL', {
					'COUNT' :iMinutes/10080
				}, null, iMinutes/10080));
			}
	
			aDisplayedAlarms.push({
				'value':iMinutes,
				'alarm': alarm,
				'text': sText,
				'isDisabled': false
			});
	
		}, this);
	
		return aDisplayedAlarms;
	};
	
	CalendarCreateEventPopup.prototype.getDisplayedPeriods = function ()
	{
		return [
			{
				label: Utils.i18n('CALENDAR/EVENT_REPEAT_NEVER'),
				value: 0
			},
			{
				label: Utils.i18n('CALENDAR/EVENT_REPEAT_DAILY'),
				value: 1
			},
			{
				label: Utils.i18n('CALENDAR/EVENT_REPEAT_WEEKLY'),
				value: 2
			},
			{
				label: Utils.i18n('CALENDAR/EVENT_REPEAT_MONTHLY'),
				value: 3
			},
			{
				label: Utils.i18n('CALENDAR/EVENT_REPEAT_YEARLY'),
				value: 4
			}
		];
	};
	
	CalendarCreateEventPopup.prototype.getDisplayedIntervals = function ()
	{
		var
			i = 1,
			aDisplayedIntervals = []
		;
	
		for (; i <= 30; i++ )
		{
			aDisplayedIntervals.push(
				{
					label: i + Utils.i18n('th'),
					value: i
				}
			);
		}
	
		return aDisplayedIntervals;
	};
	
	CalendarCreateEventPopup.prototype.getDisplayedTimes = function (sTimeFormatMoment)
	{
		var aDisplayedTimes = [];
	
		switch (sTimeFormatMoment)
		{
			case 'HH:mm':
				aDisplayedTimes = [{'value':'12:00','text':'12:00'},{'value':'12:30','text':'12:30'},{'value':'13:00','text':'13:00'},{'value':'13:30','text':'13:30'},{'value':'14:00','text':'14:00'},{'value':'14:30','text':'14:30'},{'value':'15:00','text':'15:00'},{'value':'15:30','text':'15:30'},{'value':'16:00','text':'16:00'},{'value':'16:30','text':'16:30'},{'value':'17:00','text':'17:00'},{'value':'17:30','text':'17:30'},{'value':'18:00','text':'18:00'},{'value':'18:30','text':'18:30'},{'value':'19:00','text':'19:00'},{'value':'19:30','text':'19:30'},{'value':'20:00','text':'20:00'},{'value':'20:30','text':'20:30'},{'value':'21:00','text':'21:00'},{'value':'21:30','text':'21:30'},{'value':'22:00','text':'22:00'},{'value':'22:30','text':'22:30'},{'value':'23:00','text':'23:00'},{'value':'23:30','text':'23:30'},{'value':'00:00','text':'00:00'},{'value':'00:30','text':'00:30'},{'value':'01:00','text':'01:00'},{'value':'01:30','text':'01:30'},{'value':'02:00','text':'02:00'},{'value':'02:30','text':'02:30'},{'value':'03:00','text':'03:00'},{'value':'03:30','text':'03:30'},{'value':'04:00','text':'04:00'},{'value':'04:30','text':'04:30'},{'value':'05:00','text':'05:00'},{'value':'05:30','text':'05:30'},{'value':'06:00','text':'06:00'},{'value':'06:30','text':'06:30'},{'value':'07:00','text':'07:00'},{'value':'07:30','text':'07:30'},{'value':'08:00','text':'08:00'},{'value':'08:30','text':'08:30'},{'value':'09:00','text':'09:00'},{'value':'09:30','text':'09:30'},{'value':'10:00','text':'10:00'},{'value':'10:30','text':'10:30'},{'value':'11:00','text':'11:00'},{'value':'11:30','text':'11:30'}];
	//			aDisplayedTimes = function() {
	//				var sStartTime = '12:00'
	//			};
				break;
			case 'hh:mm A':
	//			aDisplayedTimes = ['12:00 AM', '12:30 AM', '01:00 AM', '01:30 AM', '02:00 AM', '02:30 AM', '03:00 AM', '03:30 AM', '04:00 AM', '04:30 AM', '05:00 AM', '05:30 AM', '06:00 AM', '06:30 AM', '07:00 AM', '07:30 AM', '08:00 AM', '08:30 AM', '09:00 AM', '09:30 AM', '10:00 AM', '10:30 AM', '11:00 AM', '11:30 AM', '12:00', '12:30', '01:00 PM', '01:30 PM', '02:00 PM', '02:30 PM', '03:00 PM', '03:30 PM', '04:00 PM', '04:30 PM', '05:00 PM', '05:30 PM', '06:00 PM', '06:30 PM', '07:00 PM', '07:30 PM', '08:00 PM', '08:30 PM', '09:00 PM', '09:30 PM', '10:00 PM', '10:30 PM', '11:00 PM', '11:30 PM'];
				aDisplayedTimes = [{'value':'12:00 AM','text':'12:00 AM'},{'value':'12:30 AM','text':'12:30 AM'},{'value':'01:00 AM','text':'01:00 AM'},{'value':'01:30 AM','text':'01:30 AM'},{'value':'02:00 AM','text':'02:00 AM'},{'value':'02:30 AM','text':'02:30 AM'},{'value':'03:00 AM','text':'03:00 AM'},{'value':'03:30 AM','text':'03:30 AM'},{'value':'04:00 AM','text':'04:00 AM'},{'value':'04:30 AM','text':'04:30 AM'},{'value':'05:00 AM','text':'05:00 AM'},{'value':'05:30 AM','text':'05:30 AM'},{'value':'06:00 AM','text':'06:00 AM'},{'value':'06:30 AM','text':'06:30 AM'},{'value':'07:00 AM','text':'07:00 AM'},{'value':'07:30 AM','text':'07:30 AM'},{'value':'08:00 AM','text':'08:00 AM'},{'value':'08:30 AM','text':'08:30 AM'},{'value':'09:00 AM','text':'09:00 AM'},{'value':'09:30 AM','text':'09:30 AM'},{'value':'10:00 AM','text':'10:00 AM'},{'value':'10:30 AM','text':'10:30 AM'},{'value':'11:00 AM','text':'11:00 AM'},{'value':'11:30 AM','text':'11:30 AM'},{'value':'12:00 PM','text':'12:00 PM'},{'value':'12:30 PM','text':'12:30 PM'},{'value':'01:00 PM','text':'01:00 PM'},{'value':'01:30 PM','text':'01:30 PM'},{'value':'02:00 PM','text':'02:00 PM'},{'value':'02:30 PM','text':'02:30 PM'},{'value':'03:00 PM','text':'03:00 PM'},{'value':'03:30 PM','text':'03:30 PM'},{'value':'04:00 PM','text':'04:00 PM'},{'value':'04:30 PM','text':'04:30 PM'},{'value':'05:00 PM','text':'05:00 PM'},{'value':'05:30 PM','text':'05:30 PM'},{'value':'06:00 PM','text':'06:00 PM'},{'value':'06:30 PM','text':'06:30 PM'},{'value':'07:00 PM','text':'07:00 PM'},{'value':'07:30 PM','text':'07:30 PM'},{'value':'08:00 PM','text':'08:00 PM'},{'value':'08:30 PM','text':'08:30 PM'},{'value':'09:00 PM','text':'09:00 PM'},{'value':'09:30 PM','text':'09:30 PM'},{'value':'10:00 PM','text':'10:00 PM'},{'value':'10:30 PM','text':'10:30 PM'},{'value':'11:00 PM','text':'11:00 PM'},{'value':'11:30 PM','text':'11:30 PM'}];
				break;
		}
	
		return aDisplayedTimes;
	};
	
	/**
	 * @param {Array} aDisplayedAlarms
	 */
	CalendarCreateEventPopup.prototype.getAlarmsArray = function (aDisplayedAlarms)
	{
		var aAlarms = [];
	
		_.each(aDisplayedAlarms, function(oAlarm, iIdx)
		{
			aAlarms.push(oAlarm.alarm());
		}, this);
	
		return _.sortBy(aAlarms, function(num){return -num;});
	};
	
	CalendarCreateEventPopup.prototype.addFirstAlarm = function ()
	{
		if(!this.displayedAlarms().length)
		{
			this.displayedAlarms(this.getDisplayedAlarms([this.alarmOptions()[0].value]));
		}
		else
		{
			var
				sConfirm = Utils.i18n('CALENDAR/CONFIRM_CLOSE_ALARMS'),
				fAction = _.bind(function (bResult) {
					if (bResult)
					{
						this.displayedAlarms.removeAll();
					}
				}, this)
			;
	
			App.Screens.showPopup(ConfirmPopup, [sConfirm, fAction]);
		}
	};
	
	CalendarCreateEventPopup.prototype.addAlarm = function ()
	{
		var
			oDisplayedAlarm,
			aSortedAlarms,
			iMinutes = 0
		;
	
		aSortedAlarms = _.sortBy(this.displayedAlarms(), function(oAlarm){return oAlarm.alarm();});
	
		_.each(aSortedAlarms, function(oAlarm) {
	
			var nAlarmMinutes = oAlarm.alarm();
	
			if (nAlarmMinutes !== 5 && iMinutes <= 5)
			{
				iMinutes = 5;
			}
			else if (nAlarmMinutes !== 10 && iMinutes <= 10)
			{
				iMinutes = 10;
			}
			else if (nAlarmMinutes !== 15 && iMinutes <= 15)
			{
				iMinutes = 15;
			}
			else if (nAlarmMinutes !== 30 && iMinutes <= 30)
			{
				iMinutes = 30;
			}
			else if (nAlarmMinutes !== 1440 && iMinutes <= 1440)
			{
				iMinutes = 1440;
			}
		});
	
		oDisplayedAlarm = this.getDisplayedAlarms([iMinutes])[0];
	
		this['alarm' + iMinutes] = ko.observable(iMinutes);
		this.displayedAlarms.push(oDisplayedAlarm);
	};
	
	/**
	 * @param {Object} oItem
	 */
	CalendarCreateEventPopup.prototype.removeAlarm = function (oItem)
	{
		this.displayedAlarms.remove(oItem);
	};
	
	/**
	 * @param {Object} oItem
	 */
	CalendarCreateEventPopup.prototype.removeGuest = function (oItem)
	{
		this.attendees.remove(oItem);
	};
	
	CalendarCreateEventPopup.prototype.disableAlarms = function ()
	{
		_.each(this.alarmOptions(), function(oAlarm, iIdx) {
	
			oAlarm.isDisabled = _.any(this.displayedAlarms(), function(oItem){
				return oItem.alarm() === oAlarm.value;
			});
	
		}, this);
	
		this.alarmOptions.valueHasMutated();
	};
	
	/**
	 * @param {string} sTerm
	 * @param {Function} fResponse
	 */
	CalendarCreateEventPopup.prototype.autocompleteCallback = function (sTerm, fResponse)
	{
		var oParameters = {
				'Action': 'ContactSuggestions',
				'Search': sTerm,
				'GlobalOnly': '0'
			}
		;
	
		this.guestAutocompleteItem(null);
	
		sTerm = Utils.trim(sTerm);
		if ('' !== sTerm)
		{
			App.Ajax.send(oParameters, function (oData) {
				var aList = [];
				if (oData && oData.Result && oData.Result && oData.Result.List)
				{
					aList = _.map(oData.Result.List, function (oItem) {
						/*return oItem && oItem.Email ? oItem.Email : '';*/
						return oItem && oItem.Email && oItem.Email !== this.owner() ?
							(oItem.Name && 0 < Utils.trim(oItem.Name).length ?
							{value:'"' + oItem.Name + '" <' + oItem.Email + '>', name: oItem.Name, email: oItem.Email} : {value: oItem.Email, name: '', email: oItem.Email}) : null;
					}, this);
	
					aList = _.compact(aList);
				}
				fResponse(aList);
	
			}, this);
		}
		else
		{
			fResponse([]);
		}
	};
	
	CalendarCreateEventPopup.prototype.repeatRuleParse = function (oRepeatRule)
	{
		var allEvents = this.allEvents();
	
		this.repeatEndDom().datepicker("option", "minDate", this.datepickerGetDate(this.endDom()));
	
		//new event
		/*if(!oRepeatRule && allEvents === Enums.CalendarEditRecurrenceEvent.AllEvents)
		{
			//event start date + 7days
			//sDate = moment(this.startDom().datepicker( "getDate" )).add('days', 7).format(this.dateFormatMoment);
	
			//this.repeatUntil(sDate);
		}
		//excluded event
		else if(!oRepeatRule && allEvents === Enums.CalendarEditRecurrenceEvent.OnlyThisInstance)
		{
	
		}
		//only this instance
		else if(oRepeatRule && allEvents === Enums.CalendarEditRecurrenceEvent.OnlyThisInstance)
		{
	
		}
		//all events in the series
		else*/ if(oRepeatRule && allEvents === Enums.CalendarEditRecurrenceEvent.AllEvents)
		{
			this.repeatPeriod(oRepeatRule.period);
			this.repeatInterval(oRepeatRule.interval);
			this.repeatEnd(oRepeatRule.end);
			this.repeatCount(oRepeatRule.count);
			this.repeatWeekNum(oRepeatRule.weekNum);
	
			if (oRepeatRule.until)
			{
				this.datepickerSetDate(this.repeatEndDom(), new Date(oRepeatRule.until * 1000));
			}
	
			if (oRepeatRule.byDays.length)
			{
				this.weekDays(true);
	
				_.each(oRepeatRule.byDays, function (sItem)
				{
					this['week' + sItem](true);
				}, this);
			}
		}
	};
	
	CalendarCreateEventPopup.prototype.getDays = function ()
	{
		var aDays = [];
	
		if (this.weekMO()) {aDays.push('MO');}
		if (this.weekTU()) {aDays.push('TU');}
		if (this.weekWE()) {aDays.push('WE');}
		if (this.weekTH()) {aDays.push('TH');}
		if (this.weekFR()) {aDays.push('FR');}
		if (this.weekSA()) {aDays.push('SA');}
		if (this.weekSU()) {aDays.push('SU');}
	
		return aDays;
	};
	
	CalendarCreateEventPopup.prototype.onMainPanelClick = function ()
	{
		if (this.dateEdit())
		{
			this.dateEdit(false);
		}
	};
	
	/**
	 * @param {Object} oEl
	 * @param {Object} oEv
	 */
	CalendarCreateEventPopup.prototype.onKeydown = function (oEl, oEv)
	{
		if(oEv.keyCode === Enums.Key.Enter)
		{
			oEv.preventDefault();
		}
	};
	
	/**
	 * @param {Object} oEl
	 * @param {Object} oEv
	 * @param {Function} oValueObserver
	 */
	CalendarCreateEventPopup.prototype.onKeyup = function (oEl, oEv, oValueObserver)
	{
		if((oEv.target.id === "event_subject" || oEv.target.id === "event_location") && oEv.keyCode === Enums.Key.Enter)
		{
			this.onSaveClick();
		}
	//	this.autosizeTrigger.notifySubscribers();
	};
	
	CalendarCreateEventPopup.prototype.onPaste = function (oEl, oEv, oValueObserver)
	{
		var sWithoutLineBreaks = oValueObserver().replace(/[\r\n\t]+/gm, ' ');
	
		oValueObserver(sWithoutLineBreaks);
	
	};
	
	CalendarCreateEventPopup.prototype.selectStartDate = function ()
	{
		if (this.startDate() && this.endDate())
		{
			var
				sDateFormat = this.dateFormatMoment,
				sTimeFormat = this.timeFormatMoment,
				oDate = this.datepickerGetDate(this.startDom()),
				oTimeDate = (moment(this.getDateFromStr(moment(oDate).format(sDateFormat), this.startTime()))).toDate(),
				oCompareDate = this.datepickerGetDate(this.endDom()),
				oCompareTimeDate = (moment(this.getDateFromStr(moment(oCompareDate).format(sDateFormat), this.endTime()))).toDate(),
				nTimeDate = this.getUnixTimestamp(oTimeDate),
				nCompareTimeDate = this.getUnixTimestamp(oCompareTimeDate),
				oMomentTimeDate = moment(oTimeDate),
				oMomentCompareTimeDate = moment(oCompareTimeDate),
				sDateLoc = oMomentTimeDate.format(sDateFormat),
				sTimeLoc = oMomentTimeDate.format(sTimeFormat),
				sCompareDateLoc = oMomentCompareTimeDate.format(sDateFormat),
				sCompareTimeLoc = oMomentCompareTimeDate.format(sTimeFormat)
			;
	
			this.isEvOneDay(sDateLoc === sCompareDateLoc);
			this.isEvOneTime(sTimeLoc === sCompareTimeLoc);
	
			if (nTimeDate > nCompareTimeDate)
			{
				this.datepickerSetDate(this.endDom(), oTimeDate);
				this.endDate(sDateLoc);
				this.endTime(sTimeLoc);
				this.isEvOneDay(true);
				this.isEvOneTime(true);
			}
	
			this.startDate(sDateLoc);
			this.startTime(sTimeLoc);
	
			this.yearlyDate(oMomentTimeDate.format(this.getPartOfDate(this.dateFormatMoment.slice(0,-5))));
			this.monthlyDate(oMomentTimeDate.format(this.getPartOfDate(this.dateFormatMoment.slice(0,2))));
		}
	};
	
	CalendarCreateEventPopup.prototype.selectEndDate = function ()
	{
		if (this.endDate() && this.startDate())
		{
			var
				sDateFormat = this.dateFormatMoment,
				sTimeFormat = this.timeFormatMoment,
				oDate = this.datepickerGetDate(this.endDom()),
				oTimeDate = (moment(this.getDateFromStr(moment(oDate).format(sDateFormat), this.endTime()))).toDate(),
				oCompareDate = this.datepickerGetDate(this.startDom()),
				oCompareTimeDate = (moment(this.getDateFromStr(moment(oCompareDate).format(sDateFormat), this.startTime()))).toDate(),
				nTimeDate = this.getUnixTimestamp(oTimeDate),
				nCompareTimeDate = this.getUnixTimestamp(oCompareTimeDate),
				oMomentTimeDate = moment(oTimeDate),
				oMomentCompareTimeDate = moment(oCompareTimeDate),
				sDateLoc = oMomentTimeDate.format(sDateFormat),
				sTimeLoc = oMomentTimeDate.format(sTimeFormat),
				sCompareDateLoc = oMomentCompareTimeDate.format(sDateFormat),
				sCompareTimeLoc = oMomentCompareTimeDate.format(sTimeFormat)
			;
	
			this.isEvOneDay(sDateLoc === sCompareDateLoc);
			this.isEvOneTime(sTimeLoc === sCompareTimeLoc);
	
			if(nTimeDate < nCompareTimeDate)
			{
				this.datepickerSetDate(this.startDom(), oTimeDate);
				this.startDate(sDateLoc);
				this.startTime(sTimeLoc);
				this.isEvOneDay(true);
				this.isEvOneTime(true);
			}
	
			this.endDate(sDateLoc);
			this.endTime(sTimeLoc);
	
			this.repeatEndDom().datepicker('option', 'minDate', oTimeDate);
	
			if (!this.isRepeat())
			{
				this.datepickerSetDate(this.repeatEndDom(), oMomentTimeDate.add('days', 7).toDate());
			}
		}
	};
	
	/**
	 * @param {Object} oDate
	 * @return {number}
	 */
	CalendarCreateEventPopup.prototype.getUnixTimestamp = function (oDate)
	{
		return Number(moment(oDate).format('X'));
	};
	
	/**
	 * @param {Object} oInput
	 * @return {Date}
	 */
	CalendarCreateEventPopup.prototype.datepickerGetDate = function (oInput)
	{
		return oInput.datepicker('getDate');
	};
	
	/**
	 * @param {Object} oInput
	 * @param {Object} oDate
	 */
	CalendarCreateEventPopup.prototype.datepickerSetDate = function (oInput, oDate)
	{
		oInput.datepicker('setDate', oDate);
	};
	
	/**
	 * @param {string} sDate
	 * @param {string} sTime
	 * @return {Date}
	 */
	CalendarCreateEventPopup.prototype.getDateFromStr = function (sDate, sTime)
	{
		return (moment(sDate + ' ' + sTime, this.dateFormatMoment + ' ' + this.timeFormatMoment)).toDate();
	};
	
	/**
	 * @param {string} sDateFormat
	 * @return {string}
	 */
	CalendarCreateEventPopup.prototype.getPartOfDate = function (sDateFormat)
	{
		var sMomentDateFormat = 'MM/DD';
	
			switch (sDateFormat)
			{
				case 'MM/DD':
					sMomentDateFormat = 'MM/DD';
					break;
				case 'DD/MM':
					sMomentDateFormat = 'DD/MM';
					break;
				case 'DD MMMM':
					sMomentDateFormat = 'DD MMMM';
					break;
				case 'DD':
					sMomentDateFormat = 'DD';
					break;
			}
	
		return sMomentDateFormat;
	};
	
	CalendarCreateEventPopup.prototype.setActualTime = function ()
	{
		var oMoment = moment();
	
		if (oMoment.minutes() > 30)
		{
			oMoment.minutes(60);
		}
		else
		{
			oMoment.minutes(30);
		}
		oMoment.seconds(0);
		oMoment.milliseconds(0);
	
		this.startTime(oMoment.format(this.timeFormatMoment));
		this.endTime(oMoment.add('minutes', 30).format(this.timeFormatMoment));
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CalendarCreateEventPopup.prototype.onProcessAppointmentResponse = function (oData, oParameters)
	{
		if (!oData.Result)
		{
			App.Api.showErrorByCode(0, Utils.i18n('WARNING/UNKNOWN_ERROR'));
		}
	};
	
	/**
	 * @param {number} iDecision
	 */
	CalendarCreateEventPopup.prototype.doProcessAppointment = function (iDecision)
	{
		var
			oParameters = {
				'Action': 'EventUpdateAppointment',
				'actionAppointment': iDecision,
				'calendarId': this.selectedCalendar(),
				'uid' : this.uid
			},
			aAttendees = this.attendees(),
			sEmail = this.defaultAccount ? this.defaultAccount.email() : '',
			oAttendee = _.find(this.attendees(), function(oAttendee){
				return oAttendee.email === sEmail; 
			}, this),
			oCalendar = this.calendars.getCalendarById(this.selectedCalendar())
		;
	
		if (oAttendee)
		{
			App.Ajax.send(oParameters, this.onProcessAppointmentResponse, this);
	
			oAttendee.status = iDecision;
			this.attendees([]);
			this.attendees(aAttendees);
			this.setCurrentAttenderStatus(oAttendee.email, this.attendees());
			if (iDecision === Enums.IcalConfigInt.Declined && oCalendar && 
					this.callbackAttendeeActionDecline && Utils.isFunc(this.callbackAttendeeActionDecline))
			{
				this.callbackAttendeeActionDecline(oCalendar,  this.id());
				this.closePopup();
			}
		}
	};
	
	/**
	 * @param {boolean} bShared
	 * @param {boolean} bEditable
	 * @param {boolean} bMyEvent
	 */
	CalendarCreateEventPopup.prototype.editableSwitch = function (bShared, bEditable, bMyEvent)
	{
		this.isEditable((!bShared && bEditable && bMyEvent) || (bShared && bEditable && !bMyEvent));
	};
	
	/**
	 * @param {string} sCurrentEmail
	 * @param {Array} aAttendees
	 */
	CalendarCreateEventPopup.prototype.setCurrentAttenderStatus = function (sCurrentEmail, aAttendees)
	{
		var oCurrentAttender = _.find(aAttendees, function(oAttender){ return oAttender.email === sCurrentEmail; });
	
		this.attenderStatus(oCurrentAttender ? oCurrentAttender.status : 0);
	};
	
	CalendarCreateEventPopup.prototype.getAttenderTextStatus = function (sStatus)
	{
		switch (sStatus)
		{
			case 0:
				sStatus="pending";
				break;
			case 1:
				sStatus="accepted";
				break;
			case 2:
				sStatus="declined";
				break;
			case 3:
				sStatus="tentative";
				break;
		}
		return sStatus;
	};
	/**
	 * @constructor
	 */
	function CalendarEditRecurrenceEventPopup()
	{
		this.fCallback = null;
		this.confirmDesc = Utils.i18n('CALENDAR/EDIT_RECURRENCE_CONFIRM_DESCRIPTION');
		this.onlyThisInstanceButtonText = ko.observable(Utils.i18n('CALENDAR/ONLY_THIS_INSTANCE'));
		this.allEventsButtonText = ko.observable(Utils.i18n('CALENDAR/ALL_EVENTS_IN_THE_SERIES'));
		this.cancelButtonText = ko.observable(Utils.i18n('MAIN/BUTTON_CANCEL'));
	}
	
	/**
	 * @param {Function} fCallback
	 */
	CalendarEditRecurrenceEventPopup.prototype.onShow = function (fCallback)
	{
		if (Utils.isFunc(fCallback))
		{
			this.fCallback = fCallback;
		}
	};
	
	/**
	 * @return {string}
	 */
	CalendarEditRecurrenceEventPopup.prototype.popupTemplate = function ()
	{
		return 'Popups_CalendarEditRecurrenceEventPopupViewModel';
	};
	
	CalendarEditRecurrenceEventPopup.prototype.onlyThisInstanceButtonClick = function ()
	{
		if (this.fCallback)
		{
			this.fCallback(Enums.CalendarEditRecurrenceEvent.OnlyThisInstance);
		}
	
		this.closeCommand();
	};
	
	CalendarEditRecurrenceEventPopup.prototype.allEventsButtonClick = function ()
	{
		if (this.fCallback)
		{
			this.fCallback(Enums.CalendarEditRecurrenceEvent.AllEvents);
		}
	
		this.closeCommand();
	};
	
	CalendarEditRecurrenceEventPopup.prototype.cancelButtonClick = function ()
	{
		if (this.fCallback)
		{
			this.fCallback(Enums.CalendarEditRecurrenceEvent.None);
		}
	
		this.closeCommand();
	};
	
	
	
	
	
	/**
	 * @constructor
	 */
	function CUserSettingsModel()
	{
		this.IdUser = 1;
	
		// general settings that can be changed in the settings screen
		this.MailsPerPage = 20;
		this.ContactsPerPage = 20;
		this.iInterval = -1;
		this.autoCheckMailInterval = ko.observable(0);
		this.autoCheckMailInterval.subscribe(this.changeAutoCheckMailInterval, this);
		this.DefaultEditor = 1;
		this.Layout = Enums.MailboxLayout.Side;
		this.DefaultTheme = 'Default';
		this.DefaultLanguage = 'English';
		this.DefaultDateFormat = 'MM/DD/YYYY';
		this.defaultTimeFormat = ko.observable(0);
		this.ThreadsEnabled = true;
		this.useThreads = ko.observable(true);
	
		// allows the creation of messages
		this.AllowCompose = true;
	
		this.AllowReply = true;
		this.AllowForward = true;
		this.SaveMail = Enums.SaveMail.Checked;
	
		this.OutlookSyncEnable = true;
		this.MobileSyncEnable = true;
	
		this.ShowPersonalContacts = true;
		this.ShowGlobalContacts = false;
		
		this.IsFilesSupported = false;
		this.IsHelpdeskSupported = false;
		this.IsHelpdeskAgent = false;
		this.HelpdeskIframeUrl = '';
	
		// allows to go to contacts screen and edit their settings
		this.ShowContacts = this.ShowPersonalContacts || this.ShowGlobalContacts;
	
		this.LastLogin = 0;
		this.IsDemo = false;
	
		this.AllowVoice = false;
		this.VoiceRealm = '';
		this.VoiceWebsocketProxyUrl = '';
		this.VoiceOutboundProxyUrl = '';
		this.VoiceCallerID = '';
		this.VoiceImpi = '';
		this.VoiceImpu = '';
		this.VoicePassword = '';
		
		this.VoiceProvider = '';
	//	this.VoiceAccountSID = '';
	//	this.VoiceAuthToken = '';
	//	this.VoiceAppSID = '';
	
		// allows to go to calendar screen and edit its settings
		this.AllowCalendar = true;
	
		this.CalendarSharing = false;
		this.CalendarAppointments = false;
	
		// calendar settings that can be changed in the settings screen
		this.CalendarShowWeekEnds = false;
		this.CalendarShowWorkDay = false;
		this.CalendarWorkDayStarts = 0;
		this.CalendarWorkDayEnds = 0;
		this.CalendarWeekStartsOn = 0;
		this.CalendarDefaultTab = Enums.CalendarDefaultTab.Month;
		
		this.needToReloadCalendar = ko.observable(false);
	
		this.mobileSync = ko.observable(null);
		this.MobileSyncDemoPass = 'demo';
		this.outlookSync = ko.observable(null);
		this.OutlookSyncDemoPass = 'demo';
		
		this.AllowHelpdeskNotifications = false;
	}
	
	CUserSettingsModel.prototype.changeAutoCheckMailInterval = function ()
	{
		clearInterval(this.iInterval);
		if (!AppData.SingleMode && this.autoCheckMailInterval() > 0)
		{
			this.iInterval = setInterval(function () {
				App.MailCache.executeCheckMail();
			}, this.autoCheckMailInterval() * 60 * 1000);
		}
	};
	
	/**
	 * @return {boolean}
	 */
	CUserSettingsModel.prototype.getSaveMailInSentItems = function ()
	{
		var bSaveMailInSentItems = true;
		
		switch (this.SaveMail)
		{
			case Enums.SaveMail.Unchecked:
				bSaveMailInSentItems = false;
				break;
			case Enums.SaveMail.Checked:
			case Enums.SaveMail.Hidden:
				bSaveMailInSentItems = true;
				break;
		}
		
		return bSaveMailInSentItems;
	};
	
	/**
	 * @return {boolean}
	 */
	CUserSettingsModel.prototype.getUseSaveMailInSentItems = function ()
	{
		var bUseSaveMailInSentItems = false;
		
		switch (this.SaveMail)
		{
			case Enums.SaveMail.Unchecked:
			case Enums.SaveMail.Checked:
				bUseSaveMailInSentItems = true;
				break;
			case Enums.SaveMail.Hidden:
				bUseSaveMailInSentItems = false;
				break;
		}
		
		return bUseSaveMailInSentItems;
	};
	
	/**
	 * @param {AjaxUserSettingsResponse} oData
	 */
	CUserSettingsModel.prototype.parse = function (oData)
	{
		var oCalendar = null;
	
		if (oData !== null)
		{
			this.IdUser = Utils.pInt(oData.IdUser);
			this.MailsPerPage = Utils.pInt(oData.MailsPerPage);
			this.ContactsPerPage = Utils.pInt(oData.ContactsPerPage);
			this.autoCheckMailInterval(Utils.pInt(oData.AutoCheckMailInterval));
			this.DefaultEditor = Utils.pInt(oData.DefaultEditor);
			this.Layout = Utils.pInt(oData.Layout);
			this.DefaultTheme = oData.DefaultTheme.toString();
			this.DefaultLanguage = oData.DefaultLanguage.toString();
			this.DefaultDateFormat = oData.DefaultDateFormat.toString();
			this.defaultTimeFormat(Utils.pInt(oData.DefaultTimeFormat));
			this.ThreadsEnabled = !!oData.ThreadsEnabled;
			this.useThreads(!!oData.UseThreads);
			this.AllowCompose = !!oData.AllowCompose;
			this.AllowReply = !!oData.AllowReply;
			this.AllowForward = !!oData.AllowForward;
			this.SaveMail = Utils.pInt(oData.SaveMail);
			this.OutlookSyncEnable = !!oData.OutlookSyncEnable;
			this.MobileSyncEnable = !!oData.MobileSyncEnable;
			this.ShowPersonalContacts = !!oData.ShowPersonalContacts;
			this.ShowGlobalContacts = !!oData.ShowGlobalContacts;
			this.ShowContacts = this.ShowPersonalContacts || this.ShowGlobalContacts;
			
			this.IsFilesSupported = !!oData.IsFilesSupported;
			this.IsHelpdeskSupported = !!oData.IsHelpdeskSupported;
			this.IsHelpdeskAgent = !!oData.IsHelpdeskAgent;
			
			this.LastLogin = Utils.pInt(oData.LastLogin);
			this.AllowCalendar = !!oData.AllowCalendar;
	
			this.CalendarSharing = !!oData.CalendarSharing;
			this.CalendarAppointments = !!oData.CalendarAppointments;
			
			this.IsDemo = !!oData.IsDemo;
	
			this.AllowVoice = !!oData.AllowVoice;
			this.VoiceRealm = oData.VoiceRealm;
			this.VoiceWebsocketProxyUrl = oData.VoiceWebsocketProxyUrl;
			this.VoiceOutboundProxyUrl = oData.VoiceOutboundProxyUrl;
			this.VoiceCallerID = oData.VoiceCallerID;
			this.VoiceImpi = oData.VoiceImpi;
			this.VoiceImpu = oData.VoiceImpu;
			this.VoicePassword = oData.VoicePassword;
			
			this.VoiceProvider = oData.VoiceRealm;
	//		this.VoiceAccountSID = oData.VoiceRealm;
	//		this.VoiceAuthToken = oData.VoiceAuthToken;
	//		this.VoiceAppSID = oData.VoiceAppSID;
			
			
			
			this.AllowHelpdeskNotifications = oData.AllowHelpdeskNotifications;
	
			oCalendar = oData.Calendar;
			if (oCalendar)
			{
				this.CalendarShowWeekEnds = !!oCalendar.ShowWeekEnds;
				this.CalendarShowWorkDay = !!oCalendar.ShowWorkDay;
				this.CalendarWorkDayStarts = Utils.pInt(oCalendar.WorkDayStarts);
				this.CalendarWorkDayEnds = Utils.pInt(oCalendar.WorkDayEnds);
				this.CalendarWeekStartsOn = Utils.pInt(oCalendar.WeekStartsOn);
				this.CalendarDefaultTab = Utils.pInt(oCalendar.DefaultTab);
			}
		}
	};
	
	/**
	 * @param {number} iMailsPerPage
	 * @param {number} iContactsPerPage
	 * @param {number} iAutoCheckMailInterval
	 * @param {number} iDefaultEditor
	 * @param {number} iLayout
	 * @param {string} sDefaultTheme
	 * @param {string} sDefaultLanguage
	 * @param {string} sDefaultDateFormat
	 * @param {number} iDefaultTimeFormat
	 * @param {string} sUseThreads
	 */
	CUserSettingsModel.prototype.updateCommonSettings = function (iMailsPerPage, iContactsPerPage,
			iAutoCheckMailInterval, iDefaultEditor, iLayout, sDefaultTheme, sDefaultLanguage,
			sDefaultDateFormat, iDefaultTimeFormat, sUseThreads)
	{
		if (this.DefaultTheme !== sDefaultTheme || this.DefaultLanguage !== sDefaultLanguage || 
			this.DefaultDateFormat !== sDefaultDateFormat ||this.defaultTimeFormat() !== iDefaultTimeFormat)
		{
			this.needToReloadCalendar(true);
		}
		
		this.MailsPerPage = iMailsPerPage;
		this.ContactsPerPage = iContactsPerPage;
		this.autoCheckMailInterval(iAutoCheckMailInterval);
		this.DefaultEditor = iDefaultEditor;
		this.Layout = iLayout;
		this.DefaultTheme = sDefaultTheme;
		this.DefaultLanguage = sDefaultLanguage;
		this.DefaultDateFormat = sDefaultDateFormat;
		this.defaultTimeFormat(iDefaultTimeFormat);
		this.useThreads(sUseThreads === '1');
	};
	
	/**
	 * @param {boolean} bShowWeekEnds
	 * @param {boolean} bShowWorkDay
	 * @param {number} iWorkDayStarts
	 * @param {number} iWorkDayEnds
	 * @param {number} iWeekStartsOn
	 * @param {number} iDefaultTab
	 */
	CUserSettingsModel.prototype.updateCalendarSettings = function (bShowWeekEnds, bShowWorkDay,
			iWorkDayStarts, iWorkDayEnds, iWeekStartsOn, iDefaultTab)
	{
		if (this.CalendarShowWeekEnds !== bShowWeekEnds || this.CalendarShowWorkDay !== bShowWorkDay || 
			this.CalendarWorkDayStarts !== iWorkDayStarts ||this.CalendarWorkDayEnds !== iWorkDayEnds ||
			this.CalendarWeekStartsOn !== iWeekStartsOn ||this.CalendarDefaultTab !== iDefaultTab)
		{
			this.needToReloadCalendar(true);
		}
		
		this.CalendarShowWeekEnds = bShowWeekEnds;
		this.CalendarShowWorkDay = bShowWorkDay;
		this.CalendarWorkDayStarts = iWorkDayStarts;
		this.CalendarWorkDayEnds = iWorkDayEnds;
		this.CalendarWeekStartsOn = iWeekStartsOn;
		this.CalendarDefaultTab = iDefaultTab;
	};
	
	/**
	 * @param {boolean} bAllowHelpdeskNotifications
	 */
	CUserSettingsModel.prototype.updateHelpdeskSettings = function (bAllowHelpdeskNotifications)
	{
		this.AllowHelpdeskNotifications = bAllowHelpdeskNotifications;
	};
	
	/**
	 * @return {boolean}
	 */
	CUserSettingsModel.prototype.isNeedToReloadCalendar = function ()
	{
		var bNeedToReloadCalendar = this.needToReloadCalendar();
		
		this.needToReloadCalendar(false);
		
		return bNeedToReloadCalendar;
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CUserSettingsModel.prototype.onSyncSettingsResponse = function (oData, oParameters)
	{
		if (oData.Result)
		{
			this.mobileSync(oData.Result.Mobile);
			this.outlookSync(oData.Result.Outlook);
		}
		else
		{
			App.Api.showErrorByCode(oData.ErrorCode);
		}
	};
	
	CUserSettingsModel.prototype.requestSyncSettings = function ()
	{
		if (this.mobileSync() === null || this.outlookSync() === null)
		{
			App.Ajax.send({'Action': 'SyncSettings'}, this.onSyncSettingsResponse, this);
		}
	};
	
	/**
	 * @constructor
	 */
	function CCalendarModel()
	{
		this.id = 0;
		this.name = ko.observable('');
		this.description = ko.observable('');
		this.owner = ko.observable('');
		this.isDefault = false;
		this.isShared = false;
		this.isPublic = ko.observable(false);
		this.access = Enums.CalendarAccess.Write;
		this.url = ko.observable('');
		this.davUrl = ko.observable('');
		this.exportUrl = ko.observable('');
		this.pubUrl = ko.observable('');
		this.shares = ko.observableArray([]);
		this.events = [];
		this.hiddenEvents = [];
		
		this.color = ko.observable('');
		this.color.subscribe(function(){
			var 
				events = [],
				self = this
			;
			
			_.each(this.events, function (oEvent) {
				oEvent.backgroundColor = self.color();
				oEvent.borderColor = self.color();
				events.push(oEvent);
			});
			
			this.events = events;
			this.name.valueHasMutated();
	
		}, this);
		
		this.active = ko.observable(true);
		
		this.startDateTime = 0;
		this.endDateTime = 0;
	}
	
	/**
	 * @param {AjaxCalendarResponse} oData
	 */
	CCalendarModel.prototype.parse = function (oData)
	{
		this.id = oData.Id;
		this.name(oData.Name);
		this.description(oData.Description || '');
		this.owner(oData.Owner || '');
		var bActive = App.Storage.hasData(oData.Id) ? App.Storage.getData(oData.Id) : true;
		this.active(bActive);
		this.isDefault = oData.IsDefault;
		this.access = oData.Access;
		this.isShared = oData.Shared;
		this.isPublic(oData.IsPublic);
		this.color(oData.Color);
		this.url(oData.Url);
		this.davUrl(oData.ServerUrl);
		this.exportUrl(Utils.getAppPath() + '?/Raw/Calendars/0/' + oData.ExportHash);
		this.pubUrl(Utils.getAppPath() + '?calendar-pub=' + oData.PubHash);
		this.shares(oData.Shares || []);
		
		_.each(oData.Events, function (oEvent) {
			this.addEvent(oEvent);
		}, this);
	};
	
	/**
	 * @param {string} sId
	 * 
	 * @return {?}
	 */
	CCalendarModel.prototype.eventExists = function (sId)
	{
		return _.find(this.events, function(oEvent){ 
			return (sId === oEvent.id); 
		});
	};
	
	/**
	 * @param {Object} oEvent
	 */
	CCalendarModel.prototype.addEvent = function (oEvent)
	{
		if (oEvent && !this.eventExists(oEvent.id))
		{
			oEvent.backgroundColor = this.color();
			oEvent.borderColor = this.color();
			oEvent.title = Utils.getTitleForEvent(oEvent.subject);
	
			this.events.push(oEvent);
		}
	};
	
	/**
	 * @param {string} sId
	 */
	CCalendarModel.prototype.removeEvent = function (sId)
	{
		this.events = _.filter(this.events, function(oEvent){ 
			return oEvent.id !== sId;
		}, this);
	};
	
	/**
	 * @param {string} sUid
	 * @param {boolean} bSkipExcluded
	 */
	CCalendarModel.prototype.removeEventByUid = function (sUid, bSkipExcluded)
	{
		if (Utils.isUnd(bSkipExcluded))
		{
			bSkipExcluded = false;
		}
		
		this.events = _.filter(this.events, function(oEvent){ 
			return (oEvent.uid !== sUid && (!bSkipExcluded || !oEvent.excluded));
		}, this);
	};
	
	CCalendarModel.prototype.removeEvents = function ()
	{
		this.events = [];
	};
	
	/**
	 * @return {boolean}
	 */
	CCalendarModel.prototype.isEditable = function ()
	{
		return this.access !== Enums.CalendarAccess.Read;
	};
	/**
	 * @param {Object} oParameters
	 * @constructor
	 */
	function CCalendarListModel(oParameters)
	{
		this.parentOnCalendarActiveChange = oParameters.onCalendarActiveChange;
		
		this.defaultCal = ko.observable(null);
		this.currentCal = ko.observable(null);
		
		this.collection = ko.observableArray([]);
		this.collection.subscribe(function () {
			this.pickCurrentCalendar(this.defaultCal());
		}, this);
		this.count = ko.computed(function () {
			return this.collection().length;
		}, this);
		this.sharedCount = ko.computed(function () {
			var sharedCalendars = _.filter(this.collection(), 
				function(oItem){ return oItem.isShared; 
			});
			return sharedCalendars.length;
		}, this);
	}
	
	/**
	 * @param {Object} oPickCalendar
	 */
	CCalendarListModel.prototype.pickCurrentCalendar = function (oPickCalendar)
	{
		var
			oFirstActiveCal = _.find(this.collection(), function (oCalendar) {
				return oCalendar.active();
			}, this)
		;
		
		if (!this.currentCal() || !this.currentCal().active())
		{
			if (oPickCalendar && oPickCalendar.active())
			{
				this.currentCal(oPickCalendar);
			}
			else if (this.defaultCal() && (this.defaultCal().active() || !oFirstActiveCal))
			{
				this.currentCal(this.defaultCal());
			}
			else if (oFirstActiveCal)
			{
				this.currentCal(oFirstActiveCal);
			}
		}
	};
	
	/**
	 * @param {number} iCalendarId
	 */
	CCalendarListModel.prototype.hideOtherCalendars = function (iCalendarId)
	{
		_.each(this.collection(), function (oCalendar) {
			oCalendar.active(oCalendar.id === iCalendarId);
		}, this);
	};
	
	/**
	 * @param {string} sCalendarId
	 */
	CCalendarListModel.prototype.getCalendarById = function (sCalendarId)
	{
		var oCalendar = _.find(this.collection(), function(oCalendar) {
			return oCalendar.id === sCalendarId;
		}, this);
		
		return oCalendar;
	};
	
	/**
	 * @return {Array}
	 */
	CCalendarListModel.prototype.getEvents = function ()
	{
		var
			aEvents = []
		;
		
		_.each(this.collection(), function (oCalendar) {
			if (oCalendar.active())
			{
				aEvents = _.union(aEvents, oCalendar.events);
			}
		}, this);
	
		return aEvents;
	};
	
	/**
	 * @param {AjaxCalendarResponse} oCalendarData
	 * 
	 * @return {Object}
	 */
	CCalendarListModel.prototype.parseAndAddCalendar = function (oCalendarData)
	{
		var oCalendar = new CCalendarModel();
		
		oCalendar.parse(oCalendarData);
		oCalendar.active.subscribe(function (value) {
			this.parentOnCalendarActiveChange(oCalendar);
			var oPickCalendar = oCalendar.active() ? oCalendar : this.defaultCal();
			this.pickCurrentCalendar(oPickCalendar);
			App.Storage.setData(oCalendar.id, value);
		}, this);
		
		if (oCalendar.isDefault)
		{
			this.defaultCal(oCalendar);
		}
		
		var mIndex = this.calendarExists(oCalendar.id);
		if (mIndex || mIndex === 0)
		{
			this.collection.splice(mIndex, 1, oCalendar);
			
		}
		else
		{
			this.collection.push(oCalendar);
		}
		
		return oCalendar;
	};
	
	/**
	 * @param {string} sId
	 * 
	 * @return {?}
	 */
	CCalendarListModel.prototype.calendarExists = function (sId)
	{
		var iIndex = _.indexOf(_.map(this.collection(), function(oItem){return oItem.id;}), sId);
		
		return (iIndex < 0) ? false : iIndex;
	};
	
	/**
	 * @param {string} sId
	 */
	CCalendarListModel.prototype.removeCalendar = function (sId)
	{
		this.collection(_.filter(this.collection(), function(oCalendar) {
			return oCalendar.id !== sId;
		}, this));
	};
	
	
	CCalendarListModel.prototype.clearCollection = function ()
	{
		this.collection.removeAll();
	};
	
	CCalendarListModel.prototype.getColors = function ()
	{
		return _.map(this.collection(), function (oCalendar) {
			return oCalendar.color();
		}, this);
	};
	
	
	/**
	 * @constructor
	 */
	function CCalendarViewModel()
	{
		this.aDayNames = Utils.i18n('DATETIME/DAY_NAMES').split(' ');
		this.correctedDayNames = ko.observable(this.aDayNames);
		
		var self = this;
		
		this.isPublic = bExtApp;
		
		this.publicCalendarId = (this.isPublic) ? AppData.CalendarPubParams.Id : '';
		this.publicCalendarName = ko.observable('');
	
		this.timeFormatGrid = (AppData.User.defaultTimeFormat() === Enums.TimeFormat.F24) ? 'HH:mm' : 'hh:mm TT';
		this.timeFormatMoment = (AppData.User.defaultTimeFormat() === Enums.TimeFormat.F24) ? 'HH:mm' : 'hh:mm A';
		this.dateFormat = AppData.User.DefaultDateFormat;
		
		this.dateTitle = ko.observable('');
		this.dateTitleHelper = ko.observableArray(Utils.i18n('DATETIME/MONTH_NAMES').split(' '));
		this.selectedView = ko.observable('');
		this.visibleWeekdayHeader = ko.computed(function () {
			return this.selectedView() === 'month';
		}, this);
		this.selectedView.subscribe(function () {
			this.resize();
		}, this);
		
		this.$calendarGrid = null;
		this.calendarGridDom = ko.observable(null);
		
		this.$datePicker = null;
		this.datePickerDom = ko.observable(null);
	
		this.calendars = new CCalendarListModel({
			onCalendarActiveChange: _.bind(function (oCalendar) {
				var sAction = oCalendar.active() ? 'addEventSource' : 'removeEventSource';
				
				this.$calendarGrid.fullCalendar(sAction, {'events': oCalendar.events});
				
				this.refreshDatePicker();
			}, this)
		});
	
		this.colors = ['#f09650', '#f68987', '#6fd0ce', '#8fbce2', '#b9a4f5', '#f68dcf', '#d88adc', '#4afdb4', '#9da1ff', '#5cc9c9', '#77ca71', '#aec9c9'];
		
		this.busyDays = ko.observableArray([]);
		
		this.$inlineEditedEvent = null;
		this.inlineEditedEventText = null;
		this.checkStarted = ko.observable(false);
		
		this.loaded = false;
		
		this.startDateTime = 0;
		this.endDateTime = 0;
		
		this.needsToReload = false;
		
		this.calendarListClick = function (oItem) {
			oItem.active(!oItem.active());
		};
		this.currentCalendarDropdown = ko.observable(false);
		this.currentCalendarDropdownOffset = ko.observable(0);
		this.calendarDropdownToggle = function (bValue, oElement) {
			if (oElement && bValue)
			{
				var
					position = oElement.position(),
					height = oElement.outerHeight()
				;
	
				self.currentCalendarDropdownOffset(parseInt(position.top, 10) + height);
			}
	
			self.currentCalendarDropdown(bValue);
		};
		
		this.dayNamesResizeBinding = _.throttle(_.bind(this.resize, this), 50);
	
		this.customscrollTop = ko.observable(0);
		this.fullcalendarOptions = {
	//		ignoreTimezone: false,
			header: false,
			editable: !this.isPublic,
			selectable: !this.isPublic,
			allDayText: Utils.i18n('CALENDAR/TITLE_ALLDAY'),
			dayNames: this.aDayNames,
			isRTL: Utils.isRTL(),
			firstHour : 8,
			columnFormat: {
				month: 'dddd', // Monday
				week: 'dddd d', // Monday 7
				day: 'dddd d' // Monday 7
			},
			
			select: _.bind(this.createEventFromGrid, this),
			eventClick: _.bind(this.openEditEventForm, this),
			eventDragStop: _.bind(this.refreshDatePicker, this),
			eventResizeStop: _.bind(this.refreshDatePicker, this),
			eventDrop: _.bind(this.moveEvent, this),
			eventResize: _.bind(this.resizeEvent, this),
			viewRender: _.bind(this.displayViewCallback, this)
		};
		this.revertFunction = null;
		
		this.calendarSharing = AppData.User.AllowCalendar && AppData.User.CalendarSharing;
	}
	
	CCalendarViewModel.prototype.initFullCalendar = function ()
	{
		this.$calendarGrid.fullCalendar(this.fullcalendarOptions);
	};
	
	CCalendarViewModel.prototype.applyCalendarSettings = function ()
	{
		this.timeFormatGrid = (AppData.User.defaultTimeFormat() === Enums.TimeFormat.F24) ? 'HH:mm' : 'hh:mm TT';
		this.timeFormatMoment = (AppData.User.defaultTimeFormat() === Enums.TimeFormat.F24) ? 'HH:mm' : 'hh:mm A';
		this.dateFormat = AppData.User.DefaultDateFormat;
	
		this.calendarGridDom().removeClass("fc-show-weekends");
		if (AppData.User.CalendarShowWeekEnds)
		{
			this.calendarGridDom().addClass("fc-show-weekends");
		}
	
		this.fullcalendarOptions.timeFormat = this.timeFormatGrid + '{ - ' + this.timeFormatGrid + '}';
		this.fullcalendarOptions.axisFormat = this.timeFormatGrid;
		this.fullcalendarOptions.firstDay = AppData.User.CalendarWeekStartsOn;
		
		switch (AppData.User.CalendarDefaultTab)
		{
			case Enums.CalendarDefaultTab.Day:
				this.displayDayView();
				break;
			case Enums.CalendarDefaultTab.Week:
				this.displayWeekView();
				break;
			case Enums.CalendarDefaultTab.Month:
				this.displayMonthView();
				break;
		}
		
		this.applyFirstDay();
		this.fullcalendarOptions.defaultView = this.selectedView();
	
		this.$calendarGrid.fullCalendar('destroy');
		this.$calendarGrid.fullCalendar(this.fullcalendarOptions);
	};
	
	CCalendarViewModel.prototype.applyFirstDay = function ()
	{
		var
			aDayNames = [],
			sFirstDay = '',
			sLastDay = ''
		;
		
		_.each(this.aDayNames, function (sDayName) {
			aDayNames.push(sDayName);
		});
		
		switch (AppData.User.CalendarWeekStartsOn)
		{
			case 1:
				sLastDay = aDayNames.shift();
				aDayNames.push(sLastDay);
				break;
			case 6:
				sFirstDay = aDayNames.pop();
				aDayNames.unshift(sFirstDay);
				break;
		}
		
		this.correctedDayNames(aDayNames);
		
		this.$datePicker.datepicker('option', 'firstDay', AppData.User.CalendarWeekStartsOn);
	};
	
	CCalendarViewModel.prototype.initDatePicker = function ()
	{
		this.$datePicker.datepicker({
			showOtherMonths: true,
			selectOtherMonths: true,
			monthNames: Utils.getMonthNamesArray(),
			dayNamesMin: Utils.i18n('DATETIME/DAY_NAMES_MIN').split(' '),
			onChangeMonthYear: _.bind(this.changeMonthYearFromDatePicker, this),
			onSelect: _.bind(this.selectDateFromDatePicker, this),
			beforeShowDay: _.bind(this.getDayDescription, this)
		});
	};
	
	CCalendarViewModel.prototype.onApplyBindings = function ()
	{
		this.$calendarGrid = $(this.calendarGridDom());
		this.$datePicker = $(this.datePickerDom());
		
		this.initFullCalendar();
		this.initDatePicker();
		
		this.applyCalendarSettings();
		
		this.highlightWeekInDayPicker();
		
		_.delay(_.bind(this.initResizing, this), 300);
	};
	
	CCalendarViewModel.prototype.onShow = function ()
	{
		if (App.CalendarCache && App.CalendarCache.calendarSettingsChanged())
		{
			this.applyCalendarSettings();
			this.reloadAll();
		}
		else if (App.CalendarCache && App.CalendarCache.calendarChanged())
		{
			App.CalendarCache.calendarChanged(false);
			this.reloadAll();
		}
		else if (!this.loaded)
		{
			this.getCalendars();	
		}
	};
	
	/**
	 * @param {Object} oView
	 * @param {Object} oElement
	 */
	CCalendarViewModel.prototype.displayViewCallback = function (oView, oElement)
	{
		var 
			count = 0,
			prevDate = null,
			constDate = "01/01/2001 "
		;
		
		if (oView.name !== 'month' && AppData.User.CalendarShowWorkDay)
		{
			$('.fc-agenda-slots tr').each(function() {
				var $tds = $(this).find('td');
				if($tds.length !== 0) {
					var newClass = '.fc-slot' + count;
					$(newClass + ' th').each(function() {
						var 
							theValue = $(this).eq(0).text(),
							theDate = (theValue !== '') ? Date.parse(constDate + theValue) : null,
							rangeTimeFrom = Date.parse(constDate + AppData.User.CalendarWorkDayStarts + ":00"),
							rangeTimeTo = Date.parse(constDate + AppData.User.CalendarWorkDayEnds + ":00")
						;
						
						if (theDate)
						{
							prevDate = theDate;
						}
						else
						{
							theDate = prevDate;
						}
	
						if(theDate < rangeTimeFrom || theDate >= rangeTimeTo){
							$tds.addClass("fc-non-working-time");
						}
					});
					count++;
				}
			});	
		}
		
		this.activateCustomScrollInDayAndWeekView();
	};
	
	CCalendarViewModel.prototype.collectBusyDays = function ()
	{
		var 
			aBusyDays = [],
			aEvents = this.calendars.getEvents()
		;
	
		_.each(aEvents, function (oEvent) {
			var
				oStart = moment(oEvent.start),
				oEnd = oEvent.end ? moment(oEvent.end) : null,
				iDaysDiff = oEnd ? oEnd.diff(oStart, 'days') : 0,
				iIndex = 0
			;
	
			for (; iIndex <= iDaysDiff; iIndex++)
			{
				aBusyDays.push(oStart.clone().add('days', iIndex).toDate());
			}
		}, this);
	
		this.busyDays(aBusyDays);
	};
	
	CCalendarViewModel.prototype.refreshDatePicker = function ()
	{
		_.defer(_.bind(function () {
			this.collectBusyDays();
			this.$datePicker.datepicker('refresh');
			this.highlightWeekInDayPicker();
		}, this));
	};
	
	/**
	 * @param {Object} oDate
	 */
	CCalendarViewModel.prototype.getDayDescription = function (oDate)
	{
		var
			bSelectable = true,
			oFindedBusyDay = _.find(this.busyDays(), function (oBusyDay) {
				return oBusyDay.getDate() === oDate.getDate() && oBusyDay.getMonth() === oDate.getMonth() &&
					oBusyDay.getYear() === oDate.getYear();
			}, this),
			sDayClass = oFindedBusyDay ? 'day_with_events' : '',
			sDayTitle = ''
		;
		
		return [bSelectable, sDayClass, sDayTitle];
	};
	
	CCalendarViewModel.prototype.initResizing = function ()
	{
		var fResize = _.throttle(_.bind(this.resize, this), 50);
	
		$(window).bind('resize', function (e) {
			if (e.target !== this)
			{
				return;
			}
	
			fResize();
		});
	
		fResize();
	};
	
	CCalendarViewModel.prototype.resize = function ()
	{
		var oParent = this.$calendarGrid.parent();
		if (oParent)
		{
			this.$calendarGrid.fullCalendar('option', 'height', oParent.height());
		}
		this.dayNamesResize();
	};
	
	CCalendarViewModel.prototype.dayNamesResize = function ()
	{
		if (this.selectedView() === 'month')
		{
			var
				oDayNamesHeaderItem = $('div.weekday-header-item'),
				oFirstWeek = $('tr.fc-first td.fc-day'),
				oFirstWeekWidth = $(oFirstWeek[0]).width(),
				iIndex = 0
			;
			
			if (oDayNamesHeaderItem.length === 7 && oFirstWeek.length === 7 && oFirstWeekWidth !== 0)
			{
				for(; iIndex < 7; iIndex++)
				{
					$(oDayNamesHeaderItem[iIndex]).width(oFirstWeekWidth);
				}
			}
		}
	};
	
	/**
	 * @param {number} iYear
	 * @param {number} iMonth
	 * @param {Object} oInst
	 */
	CCalendarViewModel.prototype.changeMonthYearFromDatePicker = function (iYear, iMonth, oInst)
	{
		var oDate = this.$calendarGrid.fullCalendar('getDate');
		// Date object in javascript and fullcalendar use numbers 0,1,2...11 for monthes
		// datepiker uses numbers 1,2,3...12 for monthes
		this.$calendarGrid.fullCalendar('gotoDate', iYear, iMonth - 1, oDate.getDate());
		this.changeDate();
	};
	
	/**
	 * @param {string} sDate
	 * @param {Object} oInst
	 */
	CCalendarViewModel.prototype.selectDateFromDatePicker = function (sDate, oInst)
	{
		var oDate = new Date(sDate);
		this.$calendarGrid.fullCalendar('gotoDate', oDate.getFullYear(), oDate.getMonth(), oDate.getDate());
		this.changeDate();
		
		_.defer(_.bind(this.highlightWeekInDayPicker, this));
	};
	
	CCalendarViewModel.prototype.highlightWeekInDayPicker = function ()
	{
		var
			$currentDay = this.$datePicker.find('td.ui-datepicker-current-day'),
			$currentWeek = $currentDay.parent(),
			$currentMonth = this.$datePicker.find('table.ui-datepicker-calendar'),
			oView = this.$calendarGrid.fullCalendar('getView')
		;
		
		switch (oView.name)
		{
			case 'agendaDay':
				$currentMonth.addClass('highlight_day').removeClass('highlight_week');
				break;
			case 'agendaWeek':
				$currentMonth.removeClass('highlight_day').addClass('highlight_week');
				break;
			default:
				$currentMonth.removeClass('highlight_day').removeClass('highlight_week');
				break;
		}
		
		$currentWeek.addClass('current_week');
	};
	
	CCalendarViewModel.prototype.changeDate = function ()
	{
		var
			oMoment = moment(this.$calendarGrid.fullCalendar('getDate')),
			oView = this.$calendarGrid.fullCalendar('getView'),
			sTitle = oMoment.format('MMMM YYYY'),
			oStartMoment = oView.start ? moment(oView.start) : null,
			oEndMoment = oView.end ? moment(oView.end).add('days', -1) : null,
			aCalendarIds = _.map(this.calendars.collection(), function (oCalendar){
				return oCalendar.id;
			}, this)
		;
	
		if (aCalendarIds.length > 0)
		{
			this.getEvents(aCalendarIds);
		}
		
		switch (oView.name)
		{
			case 'agendaDay':
				sTitle = oMoment.format('MMMM D, YYYY');
				break;
			case 'agendaWeek':
				if (oStartMoment && oEndMoment)
				{
					sTitle = oStartMoment.format('MMMM D, YYYY') + ' - ' + oEndMoment.format('MMMM D, YYYY');
				}
				break;
		}
		
		this.dateTitle(sTitle);
		this.selectedView(oView.name);
	};
	
	CCalendarViewModel.prototype.changeDateInDatePicker = function ()
	{
		this.$datePicker.datepicker('setDate', this.$calendarGrid.fullCalendar('getDate'));
		this.highlightWeekInDayPicker();
	};
	
	CCalendarViewModel.prototype.activateCustomScrollInDayAndWeekView = function ()
	{
		var aWeekGridInner = $('table.fc-agenda-slots').parent().parent();
	
		aWeekGridInner.each(function() {
	
			var
				oWeekGridInner = $(this),
				oWeekGrid = $(this).parent()
			;
	
			if (!oWeekGrid.hasClass('scroll-wrap'))
			{
				oWeekGrid.attr('data-bind', 'customScrollbar: {x: false, relativeToInner: true}');
				oWeekGridInner.css({'overflow': 'hidden'}).addClass('scroll-inner');
	
				ko.applyBindings({}, oWeekGrid[0]);
			}
		});
	};
	
	/**
	 * @param {string} sCmd
	 * @param {string=} sParam = ''
	 */
	CCalendarViewModel.prototype.execCommand = function (sCmd, sParam)
	{
		if (sParam)
		{
			this.$calendarGrid.fullCalendar(sCmd, sParam);
		}
		else
		{
			this.$calendarGrid.fullCalendar(sCmd);
		}
		
		this.changeDate();
		this.changeDateInDatePicker();
	};
	
	CCalendarViewModel.prototype.displayToday = function ()
	{
		this.execCommand('today');
	};
	
	CCalendarViewModel.prototype.displayPrev = function ()
	{
		this.execCommand('prev');
	};
	
	CCalendarViewModel.prototype.displayNext = function ()
	{
		this.execCommand('next');
	};
	
	CCalendarViewModel.prototype.displayDayView = function ()
	{
		this.execCommand('changeView', 'agendaDay');
	};
	
	CCalendarViewModel.prototype.displayWeekView = function ()
	{
		this.execCommand('changeView', 'agendaWeek');
	};
	
	CCalendarViewModel.prototype.displayMonthView = function ()
	{
		this.execCommand('changeView', 'month');
	};
	
	CCalendarViewModel.prototype.reloadAll = function ()
	{
		this.startDateTime = 0;
		this.endDateTime = 0;
		this.needsToReload = true;
		
		this.getCalendars();
	};
	
	CCalendarViewModel.prototype.getCalendars = function ()
	{
		this.checkStarted(true);
		App.Ajax.abortRequestByActionName('CalendarList');
		App.Ajax.sendExt({
				'Action': 'CalendarList',
				'IsPublic': this.isPublic ? 1 : 0,
				'PublicCalendarId': this.publicCalendarId
			}, this.onCalendarsResponse, this
		);
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onCalendarsResponse = function (oData, oParameters)
	{
		var
			aCalendarIds = [],
			oCalendar = null
		;
		if (oData.Result)
		{
			this.loaded = true;
	
			//sets default calendar aways fist in list
			oData.Result = _.sortBy(oData.Result, function(oItem){ return oItem.Id !== "Default"; });
	
			_.each(oData.Result, function (oCalendarData) {
				oCalendar = this.calendars.parseAndAddCalendar(oCalendarData);
				if (oCalendar)
				{
					if (this.isPublic)
					{
						App.setTitle(oCalendar.name());
						this.publicCalendarName(oCalendar.name());
					}
					aCalendarIds.push(oCalendar.id);
				}
			}, this);
			
			if (aCalendarIds.length === 0 && this.isPublic)
			{
				App.setTitle(Utils.i18n('CALENDAR/NO_CALENDAR_FOUND'));
				App.Api.showErrorByCode(0, Utils.i18n('CALENDAR/NO_CALENDAR_FOUND'));
			}
	
			this.getEvents(aCalendarIds);
		}
		this.checkStarted(false);
	};
	
	/**
	 * @param {Array} aCalendarIds
	 */
	CCalendarViewModel.prototype.getEvents = function (aCalendarIds)
	{
		var 
			bNeedsToLoad = false,
			oView = this.$calendarGrid.fullCalendar('getView'),
			iStart = oView && oView['visStart'] && oView['visStart']['getTime'] ? oView['visStart']['getTime']() / 1000 : 0,
			iEnd = oView && oView['visEnd'] && oView['visEnd']['getTime'] ? oView['visEnd']['getTime']() / 1000 : 0
		;
		if (this.startDateTime === 0 && this.endDateTime === 0)
		{
			this.startDateTime = iStart;
			this.endDateTime = iEnd;
			bNeedsToLoad = true;
		}
		else if (iStart < this.startDateTime && iEnd > this.endDateTime)
		{
			this.startDateTime = iStart;
			this.endDateTime = iEnd;
			bNeedsToLoad = true;
		}
		else if (iStart < this.startDateTime)
		{
			iEnd= this.startDateTime;
			this.startDateTime = iStart;
			bNeedsToLoad = true;
		}
		else if (iEnd > this.endDateTime)
		{
			iStart = this.endDateTime;
			this.endDateTime = iEnd;
			bNeedsToLoad = true;
		}
	
		if (aCalendarIds.length > 0 && bNeedsToLoad)
		{
			this.checkStarted(true);
			if (aCalendarIds.length > 1)
			{
				this.$calendarGrid.find('.fc-view div').first().css('visibility', 'hidden');
			}
			App.Ajax.abortRequestByActionName('EventList');
			App.Ajax.sendExt({
				'CalendarIds': JSON.stringify(aCalendarIds),
				'Start': iStart,
				'End': iEnd,
				'IsPublic': this.isPublic ? 1 : 0,
				'Action': 'EventList'
			}, this.onEventsResponse, this);
		}
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onEventsResponse = function (oData, oParameters)
	{
		if (oData.Result)
		{
			if (this.needsToReload)
			{
				this.$calendarGrid.fullCalendar('removeEvents');
			}
			_.each(this.calendars.collection(), function (oCalendar){
				if (oCalendar)
				{
					this.$calendarGrid.fullCalendar('removeEventSource', {'events': oCalendar.events});
					if (this.needsToReload)
					{
						oCalendar.removeEvents();
					}
				}
			}, this);
			this.needsToReload = false;
			
			_.each(oData.Result, function (oEventData) {
				var oCalendar = this.calendars.getCalendarById(oEventData.calendarId);			
				if (oCalendar)
				{
					oCalendar.addEvent(oEventData);
				}
			}, this);
			
			_.each(this.calendars.collection(), function (oCalendar, iIndex, aCollection) {
				if (oCalendar && oCalendar.active())
				{
					this.$calendarGrid.fullCalendar('addEventSource',  {'events': oCalendar.events});
				}
			}, this);	
			
			this.refreshDatePicker();
		}
		
		this.$calendarGrid.css('visibility', '');
		this.$calendarGrid.find('.fc-view div').first().css('visibility', '');
		this.checkStarted(false);
	};
	
	
	CCalendarViewModel.prototype.getUnusedColor = function ()
	{
		var colors = _.difference(this.colors, this.calendars.getColors());
		
		if (colors.length > 0)
		{
			return colors[0];
		}
		return this.colors[0];
	};
	
	CCalendarViewModel.prototype.openCreateCalendarForm = function ()
	{
		if (!this.isPublic)
		{
			var oCalendar = new CCalendarModel();
			oCalendar.color(this.getUnusedColor());
			App.Screens.showPopup(CalendarCreatePopup, [_.bind(this.createCalendar, this), this.colors, oCalendar]);
		}
	};
	
	/**
	 * @param {string} sName
	 * @param {string} sDescription
	 * @param {string} sColor
	 */
	CCalendarViewModel.prototype.createCalendar = function (sName, sDescription, sColor)
	{
		if (!this.isPublic)
		{
			App.Ajax.send({
					'Name': sName,
					'Description': sDescription,
					'Color': sColor,
					'Action': 'CalendarCreate'
				}, this.onCalendarCreateResponse, this
			);
		}
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onCalendarCreateResponse = function (oData, oParameters)
	{
		if (oData.Result)
		{
			this.calendars.parseAndAddCalendar(/** @type {AjaxCalendarResponse} */{
				Id: oData.Result,
				Name: oParameters.Name,
				Description: oParameters.Description,
				Active: true,
				isDefault: false,
				Color: oParameters.Color,
				Events: []
			});
		}
	};
	
	/**
	 * @param {Object} oCalendar
	 */
	CCalendarViewModel.prototype.openUpdateCalendarForm = function (oCalendar)
	{
		if (!this.isPublic)
		{
			App.Screens.showPopup(CalendarCreatePopup, [_.bind(this.updateCalendar, this), this.colors, oCalendar]);
		}
	};
	
	/**
	 * @param {string} sName
	 * @param {string} sDescription
	 * @param {string} sColor
	 * @param {string} sId
	 */
	CCalendarViewModel.prototype.updateCalendar = function (sName, sDescription, sColor, sId)
	{
		if (!this.isPublic)
		{
			App.Ajax.send({
					'Name': sName,
					'Description': sDescription,
					'Color': sColor,
					'Id': sId,
					'Action': 'CalendarUpdate'
				}, this.onCalendarUpdateResponse, this
			);
		}
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onCalendarUpdateResponse = function (oData, oParameters)
	{
		var
			oCalendar = null,
			aEvents = []
		;
		if (oData.Result)
		{
			oCalendar = this.calendars.getCalendarById(oParameters.Id);
			if (oCalendar)
			{
				aEvents = oCalendar.events;
				
				oCalendar.name(oParameters.Name);
				oCalendar.description(oParameters.Description);
				oCalendar.color(oParameters.Color);
				
				this.$calendarGrid.fullCalendar('removeEventSource', {'events': aEvents});
				this.$calendarGrid.fullCalendar('addEventSource',  {'events': oCalendar.events});
			}
		}
	};
	
	/**
	 * @param {string} sColor
	 * @param {string} sId
	 */
	CCalendarViewModel.prototype.updateCalendarColor = function (sColor, sId)
	{
		if (!this.isPublic)
		{
			App.Ajax.send({
					'Color': sColor,
					'Id': sId,
					'Action': 'CalendarUpdateColor'
				}, this.onCalendarUpdateColorResponse, this
			);
		}
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onCalendarUpdateColorResponse = function (oData, oParameters)
	{
		var
			oCalendar = null,
			aEvents = []
		;
		if (oData.Result)
		{
			oCalendar = this.calendars.getCalendarById(oParameters.Id);
			if (oCalendar)
			{
				aEvents = oCalendar.events; 
		
				oCalendar.color(oParameters.Color);
				
				this.$calendarGrid.fullCalendar('removeEventSource', {'events': aEvents});
				this.$calendarGrid.fullCalendar('addEventSource',  {'events': oCalendar.events});
			}
		}
	};
	
	/**
	 * @param {Object} oCalendar
	 */
	CCalendarViewModel.prototype.openGetLinkCalendarForm = function (oCalendar)
	{
		if (!this.isPublic)
		{
			App.Screens.showPopup(CalendarGetLinkPopup, [_.bind(this.publicCalendar, this), oCalendar]);
		}
	};
	
	/**
	 * @param {Object} oCalendar
	 */
	CCalendarViewModel.prototype.openShareCalendarForm = function (oCalendar)
	{
		if (!this.isPublic)
		{
			App.Screens.showPopup(CalendarSharePopup, [_.bind(this.shareCalendar, this), oCalendar]);
		}
	};
	
	/**
	 * @param {string} sId
	 * @param {boolean} bIsPublic
	 * @param {Array} aShares
	 */
	CCalendarViewModel.prototype.shareCalendar = function (sId, bIsPublic, aShares)
	{
		if (!this.isPublic)
		{
			App.Ajax.send({
					'Action': 'CalendarShareUpdate',
					'Id': sId,
					'IsPublic': bIsPublic ? 1 : 0,
					'Shares': JSON.stringify(aShares)			
				}, this.onCalendarShareUpdateResponse, this
			);
		}
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onCalendarShareUpdateResponse = function (oData, oParameters)
	{
		if (oData.Result)
		{
			var	oCalendar = this.calendars.getCalendarById(oParameters.Id);
			if (oCalendar)
			{
				oCalendar.shares(JSON.parse(oParameters.Shares));
			}
		}
	};
	
	/**
	 * @param {string} sId
	 * @param {boolean} bIsPublic
	 */
	CCalendarViewModel.prototype.publicCalendar = function (sId, bIsPublic)
	{
		if (!this.isPublic)
		{
			App.Ajax.send({
					'Action': 'CalendarPublicUpdate',
					'Id': sId,
					'IsPublic': bIsPublic ? 1 : 0
				}, this.onCalendarPublicUpdateResponse, this
			);
		}
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onCalendarPublicUpdateResponse = function (oData, oParameters)
	{
		if (oData.Result)
		{
			var	oCalendar = this.calendars.getCalendarById(oParameters.Id);
			if (oCalendar)
			{
				oCalendar.isPublic(oParameters.IsPublic);
			}
		}
	};
	
	/**
	 * @param {string} sId
	 */
	CCalendarViewModel.prototype.deleteCalendar = function (sId)
	{
		var
			oCalendar = this.calendars.getCalendarById(sId),
			sConfirm = oCalendar ? Utils.i18n('CALENDAR/CONFIRM_REMOVE_CALENDAR', {'CALENDARNAME': oCalendar.name()}) : '',
			fRemove = _.bind(function (bRemove) {
				if (bRemove)
				{
					App.Ajax.send({
							'Id': sId,
							'Action': 'CalendarDelete'
						}, this.onCalendarDeleteResponse, this
					);
				}
			}, this)
		;
		
		if (!this.isPublic && oCalendar)
		{
			App.Screens.showPopup(ConfirmPopup, [sConfirm, fRemove]);
		}	
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onCalendarDeleteResponse = function (oData, oParameters)
	{
		if (oData.Result)
		{
			var oCalendar = this.calendars.getCalendarById(oParameters.Id);
			if (oCalendar && !oCalendar.isDefault)
			{
				if (this.calendars.currentCal().id === oCalendar.id)
				{
					this.calendars.currentCal(null);
				}
				this.calendars.removeCalendar(oCalendar.id);
				this.$calendarGrid.fullCalendar('removeEventSource', {'events': oCalendar.events});
			}
		}
	};
	
	CCalendarViewModel.prototype.createNewEventInCurrentCalendar = function ()
	{
		this.createNewEventToday(this.calendars.currentCal());
	};
	
	/**
	 * @param {string} sCalendarId
	 */
	CCalendarViewModel.prototype.createNewEventInCalendar = function (sCalendarId)
	{
		this.createNewEventToday(this.calendars.getCalendarById(sCalendarId));
	};
	
	/**
	 * @param {Object} oSelectedCalendar
	 */
	CCalendarViewModel.prototype.createNewEventToday = function (oSelectedCalendar)
	{
		var oMoment = moment();
		
		if (oMoment.minutes() > 30)
		{
			oMoment.add('minutes', 60 - oMoment.minutes());
		}
		else
		{
			oMoment.minutes(30);
		}
		oMoment.seconds(0);
		oMoment.milliseconds(0);
		
		this.openCreateEventForm(oSelectedCalendar, oMoment, oMoment.clone().add('minutes', 30), false);
	};
	
	/**
	 * @param {Object} oEventData
	 */
	CCalendarViewModel.prototype.getParametersFromEventData = function (oEventData)
	{
		var	
			aParameters = {
				id: oEventData.id,
				uid: oEventData.uid,
				calendarId: oEventData.calendarId,
				newCalendarId: !Utils.isUnd(oEventData.newCalendarId) ? oEventData.newCalendarId : oEventData.calendarId,
				subject: oEventData.subject,
				allDay: oEventData.allDay ? 1 : 0,
				location: oEventData.location,
				description: oEventData.description,
				alarms: oEventData.alarms ? JSON.stringify(oEventData.alarms) : '[]',
				attendees: oEventData.attendees ? JSON.stringify(oEventData.attendees) : '[]',
				owner: oEventData.owner,
				recurrenceId: oEventData.recurrenceId,
				excluded: oEventData.excluded,
				allEvents: oEventData.allEvents,
				modified: oEventData.modified ? 1 : 0
			}
		;
	
		
		if (oEventData.rrule)
		{
			aParameters.rrule = JSON.stringify(oEventData.rrule);
		}
	
		aParameters.start = oEventData.start;
		aParameters.end = oEventData.end;
	
		aParameters.startTimestamp = aParameters.start.getTime() / 1000;
		aParameters.endTimestamp = aParameters.end ? aParameters.end.getTime() / 1000 : aParameters.startTimestamp;
		
		return aParameters;
	};
	
	/**
	 * @param {Array} aParameters
	 */
	CCalendarViewModel.prototype.getEventDataFromParameters = function (aParameters)
	{
		var	oEventData = aParameters;
		
		oEventData.alarms = aParameters.alarms ? JSON.parse(aParameters.alarms) : [];
		oEventData.attendees = aParameters.attendees ? JSON.parse(aParameters.attendees) : [];
	
		if(aParameters.rrule)
		{
			oEventData.rrule = JSON.parse(aParameters.rrule);
		}
	
		return oEventData;
	};
	
	/**
	 * @param {Object} oStart
	 * @param {Object} oEnd
	 * @param {boolean} bAllDay
	 */
	CCalendarViewModel.prototype.createEventFromGrid = function (oStart, oEnd, bAllDay)
	{
		this.openCreateEventForm(this.calendars.currentCal(), moment(oStart), moment(oEnd), bAllDay);
	};
	
	/**
	 * @param {Object} oSelectedCalendar
	 * @param {Object} oStartMoment
	 * @param {Object} oEndMoment
	 * @param {boolean} bAllDay
	 */
	CCalendarViewModel.prototype.openCreateEventForm = function (oSelectedCalendar, oStartMoment, oEndMoment, bAllDay)
	{
		if (!this.isPublic)
		{
			App.Screens.showPopup(CalendarCreateEventPopup, [{
				CallbackSave: _.bind(this.createEvent, this),
				CallbackDelete: _.bind(this.deleteEvent, this),
				Calendars: this.calendars,
				SelectedCalendar: oSelectedCalendar.id,
				StartMoment: oStartMoment,
				EndMoment: oEndMoment,
				AllDay: bAllDay,
				TimeFormat: this.timeFormatMoment,
				DateFormat: this.dateFormat,
				CallbackAttendeeActionDecline: _.bind(this.attendeeActionDecline, this),
				Owner: oSelectedCalendar.owner()
			}]);
		}
	};
	
	/**
	 * @param {Object} oEventData
	 */
	CCalendarViewModel.prototype.createEvent = function (oEventData)
	{
		var 
			aParameters = this.getParametersFromEventData(oEventData),
			oView = this.$calendarGrid.fullCalendar('getView'),
			iStart = oView && oView['visStart'] && oView['visStart']['getTime'] ? oView['visStart']['getTime']() / 1000 : 0,
			iEnd = oView && oView['visEnd'] && oView['visEnd']['getTime'] ? oView['visEnd']['getTime']() / 1000 : 0
		;
	
		if (!this.isPublic)
		{
			aParameters.calendarId = oEventData.newCalendarId;
			aParameters.selectStart = iStart;
			aParameters.selectEnd = iEnd;
			aParameters.Action = 'EventCreate';
			App.Ajax.send(aParameters, this.onEventActionResponse, this);
		}
	};
	
	/**
	 * @param {Object} oEventData
	 */
	CCalendarViewModel.prototype.openEditEventForm = function (oEventData)
	{
		var
			/**
			 * @param {number} iResult
			 */
			fCallback = _.bind(function (iResult) {
				var oEventParams = {
						CallbackSave: _.bind(this.updateEvent, this),
						CallbackDelete: _.bind(this.deleteEvent, this),
						ID: oEventData.id,
						Uid: oEventData.uid,
						RecurrenceId: oEventData.recurrenceId,
						Calendars: this.calendars,
						SelectedCalendar: oEventData.calendarId,
						AllDay: oEventData.allDay,
						Location: oEventData.location,
						Description: oEventData.description,
						Subject: oEventData.subject,
						Alarms: oEventData.alarms,
						Attendees: oEventData.attendees,
						RRule: oEventData.rrule ? oEventData.rrule : null,
						Excluded: oEventData.excluded ? oEventData.excluded : false,
						Owner: oEventData.owner,
						Appointment: oEventData.appointment,
						OwnerName: oEventData.ownerName,
						TimeFormat: this.timeFormatMoment,
						DateFormat: this.dateFormat,
						AllEvents: iResult,
						CallbackAttendeeActionDecline: _.bind(this.attendeeActionDecline, this)
					}
				;
				if (iResult !== Enums.CalendarEditRecurrenceEvent.None)
				{
					if (iResult === Enums.CalendarEditRecurrenceEvent.AllEvents && oEventData.rrule)
					{
						oEventParams.StartMoment = moment(oEventData.rrule.startBase * 1000);
						oEventParams.EndMoment = moment(oEventData.rrule.endBase * 1000);
	
					}
					else
					{
						oEventParams.StartMoment = moment(oEventData.start);
						oEventParams.EndMoment = moment(oEventData.end);
					}
					App.Screens.showPopup(CalendarCreateEventPopup, [oEventParams]);
				}
			}, this)
		;
		
		if (oEventData.rrule)
		{
			if (oEventData.excluded)
			{
				fCallback(Enums.CalendarEditRecurrenceEvent.OnlyThisInstance);
			}
			else
			{
				App.Screens.showPopup(CalendarEditRecurrenceEventPopup, [fCallback]);
			}
		}
		else
		{
			fCallback(Enums.CalendarEditRecurrenceEvent.AllEvents);
		}
	};
	
	/**
	 * @param {string} sAction
	 * @param {Object} oParameters
	 * @param {Function=} fRevertFunc = undefined
	 */
	CCalendarViewModel.prototype.eventAction = function (sAction, oParameters, fRevertFunc)
	{
		var oCalendar = this.calendars.getCalendarById(oParameters.calendarId);
		
		if (oCalendar.access === Enums.CalendarAccess.Read)
		{
			if (fRevertFunc)
			{
				fRevertFunc();		
			}
		}
		else
		{
			if (!this.isPublic)
			{
				if (fRevertFunc)
				{
					this.revertFunction = fRevertFunc;
				}
				
				oParameters.Action = sAction;
				App.Ajax.send(
					oParameters,
					this.onEventActionResponse, this
				);
			}
		}
	};
	
	/**
	 * @param {Object} oEventData
	 */
	CCalendarViewModel.prototype.updateEvent = function (oEventData)
	{
		var 
			oParameters = this.getParametersFromEventData(oEventData),
			oView = this.$calendarGrid.fullCalendar('getView'),
			iStart = oView && oView['visStart'] && oView['visStart']['getTime'] ? oView['visStart']['getTime']() / 1000 : 0,
			iEnd = oView && oView['visEnd'] && oView['visEnd']['getTime'] ? oView['visEnd']['getTime']() / 1000 : 0
		;
		
		oParameters.selectStart = iStart;
		oParameters.selectEnd = iEnd;
		if (oEventData.modified)
		{
			this.eventAction('EventUpdate', oParameters);
		}
	};
	
	/**
	 * @param {Object} oEventData
	 * @param {number} dayDelta
	 * @param {number} minuteDelta
	 * @param {boolean} allDay
	 * @param {Function} revertFunc
	 */
	CCalendarViewModel.prototype.moveEvent = function (oEventData, dayDelta, minuteDelta, allDay, revertFunc)
	{
		oEventData.dayDelta = dayDelta ? dayDelta : 0;
		oEventData.minuteDelta = minuteDelta ? minuteDelta : 0;
		var 
			oParameters = this.getParametersFromEventData(oEventData),
			oView = this.$calendarGrid.fullCalendar('getView'),
			iStart = oView && oView['visStart'] && oView['visStart']['getTime'] ? oView['visStart']['getTime']() / 1000 : 0,
			iEnd = oView && oView['visEnd'] && oView['visEnd']['getTime'] ? oView['visEnd']['getTime']() / 1000 : 0,
			iNewStart = oParameters.startTimestamp,
			iAllEvStart,
			iAllEvEnd,
	
	//		sConfirm = Utils.i18n('With drag-n-drop you can change the date of this single instance only. To alter the entire series, open the event and change its date.'),
			fConfirm = _.bind(function (bConfirm) {
				if (bConfirm)
				{
					oParameters.allEvents = Enums.CalendarEditRecurrenceEvent.OnlyThisInstance;
					this.eventAction('EventUpdate', oParameters, revertFunc);
				}
				else if (revertFunc)
				{
					revertFunc();
				}
			}, this)
		;
	
		oParameters.selectStart = iStart;
		oParameters.selectEnd = iEnd;
		if (!this.isPublic)
		{
			if (oParameters.rrule)
			{
				revertFunc(false);
	/*			
				iAllEvStart = JSON.parse(oParameters.rrule).startBase;
				iAllEvEnd = JSON.parse(oParameters.rrule).until;
	
				if (iAllEvStart <= iNewStart && iNewStart <= iAllEvEnd)
				{
					if (oParameters.excluded)
					{
						oParameters.allEvents = Enums.CalendarEditRecurrenceEvent.OnlyThisInstance;
						this.eventAction('EventUpdate', oParameters, revertFunc);
					}
					else
					{
						App.Screens.showPopup(ConfirmPopup, [sConfirm, fConfirm, '', 'Update this instance']);
					}
				}
				else 
				{
					revertFunc(false);
				}
	*/				
			}
			else
			{
				oParameters.allEvents = Enums.CalendarEditRecurrenceEvent.AllEvents;
				this.eventAction('EventUpdate', oParameters, revertFunc);
			}
		}	
		
	};
	
	/**
	 * @param {Object} oEventData
	 * @param {number} dayDelta, 
	 * @param {number} minuteDelta, 
	 * @param {Function} revertFunc
	 */
	CCalendarViewModel.prototype.resizeEvent = function (oEventData, dayDelta, minuteDelta, revertFunc)
	{
		var 
			oParameters = this.getParametersFromEventData(oEventData),
			oView = this.$calendarGrid.fullCalendar('getView'),
			iStart = oView && oView['visStart'] && oView['visStart']['getTime'] ? oView['visStart']['getTime']() / 1000 : 0,
			iEnd = oView && oView['visEnd'] && oView['visEnd']['getTime'] ? oView['visEnd']['getTime']() / 1000 : 0,
			/**
			 * @param {number} iResult
			 */
			fCallback = _.bind(function (iResult) {
				if (iResult !== Enums.CalendarEditRecurrenceEvent.None)
				{
	//				if (iResult === Enums.CalendarEditRecurrenceEvent.AllEvents)
	//				{
	//
	//				}
					oParameters.allEvents = iResult;
					this.eventAction('EventUpdate', oParameters, revertFunc);
				}
				else
				{
					revertFunc();
				}
			}, this)
		;
		
		oParameters.selectStart = iStart;
		oParameters.selectEnd = iEnd;
		if (oEventData.rrule)
		{
			if (oParameters.excluded)
			{
				fCallback(Enums.CalendarEditRecurrenceEvent.OnlyThisInstance);
			}
			else
			{
				App.Screens.showPopup(CalendarEditRecurrenceEventPopup, [fCallback]);
			}
		}
		else
		{
			fCallback(Enums.CalendarEditRecurrenceEvent.AllEvents);
		}
	};
	
	/**
	 * @param {Object} oEventData
	 */
	CCalendarViewModel.prototype.deleteEvent = function (oEventData)
	{
		var oParameters = this.getParametersFromEventData(oEventData);
	
		this.eventAction('EventDelete', oParameters);
	};
	
	/**
	 * @param {Object} oData
	 * @param {Object} oParameters
	 */
	CCalendarViewModel.prototype.onEventActionResponse = function (oData, oParameters)
	{
		var oCalendar = this.calendars.getCalendarById(oParameters.calendarId);
		
		if(oData.Result && !Utils.isUnd(oCalendar))
		{
			if (oParameters.Action === 'EventUpdate' || oParameters.Action === 'EventCreate')
			{
				this.customscrollTop(parseInt($('.calendar .scroll-inner').scrollTop(), 10));
				this.$calendarGrid.fullCalendar('removeEventSource', {'events': oCalendar.events});
				if(oParameters.rrule && oParameters.allEvents === Enums.CalendarEditRecurrenceEvent.AllEvents)
				{
					oCalendar.removeEventByUid(oParameters.uid, true);
				}
				else
				{
					oCalendar.removeEvent(oParameters.id);
				}
				
				if (oParameters.newCalendarId && oParameters.newCalendarId !== oParameters.calendarId)
				{
					this.$calendarGrid.fullCalendar('addEventSource', {'events': oCalendar.events});
					oCalendar = this.calendars.getCalendarById(oParameters.newCalendarId);			
					this.$calendarGrid.fullCalendar('removeEventSource', {'events': oCalendar.events});
				}
				_.each(oData.Result, function (oEventData) {
						oCalendar.addEvent(oEventData);
				}, this);
				
				if (!oCalendar.active())
				{
					oCalendar.active(true);
				}
				else
				{
					this.$calendarGrid.fullCalendar('addEventSource', {'events': oCalendar.events});
				}
				this.refreshDatePicker();
				
				this.customscrollTop.valueHasMutated();
			}
			else if (oParameters.Action === 'EventDelete')
			{
				this.$calendarGrid.fullCalendar('removeEventSource', {'events': oCalendar.events});
				if(oParameters.allEvents === Enums.CalendarEditRecurrenceEvent.OnlyThisInstance)
				{
					oCalendar.removeEvent(oParameters.id);
				}
				else
				{
					oCalendar.removeEventByUid(oParameters.uid);
				}
				this.$calendarGrid.fullCalendar('addEventSource', {'events': oCalendar.events});
				this.refreshDatePicker();
			}
			else if (oParameters.Action === 'EventBase')
			{
				var oEventData = oData.Result;
				App.Screens.showPopup(CalendarCreateEventPopup, [{
					CallbackSave: _.bind(this.updateEvent, this),
					CallbackDelete: _.bind(this.deleteEvent, this),
					ID: oEventData.id,
					Uid: oEventData.uid,
					RecurrenceId: oEventData.recurrenceId,
					Calendars: this.calendars,
					SelectedCalendar: oEventData.calendarId,
					StartMoment: moment(oEventData.start * 1000),
					EndMoment: moment(oEventData.end * 1000),
					AllDay: oEventData.allDay,
					Location: oEventData.location,
					Description: oEventData.description,
					Subject: oEventData.subject,
					Alarms: oEventData.alarms,
					Attendees: oEventData.attendees,
					RRule: oEventData.rrule ? oEventData.rrule : null,
					Excluded: oEventData.excluded ? oEventData.excluded : false,
					Owner: oEventData.owner,
					Appointment: oEventData.appointment,
					TimeFormat: this.timeFormatMoment,
					DateFormat: this.dateFormat,
					AllEvents: Enums.CalendarEditRecurrenceEvent.AllEvents
				}]);
			}
		}
		else
		{
			if (this.revertFunction)
			{
				this.revertFunction();
			}
		}
		this.revertFunction = null;
	};
	
	/**
	 * @param {Object} oCalendar
	 * @param {string} sId
	 */
	CCalendarViewModel.prototype.attendeeActionDecline = function (oCalendar, sId)
	{
		this.$calendarGrid.fullCalendar('removeEventSource', {'events': oCalendar.events});
		oCalendar.removeEvent(sId);
		this.$calendarGrid.fullCalendar('addEventSource', {'events': oCalendar.events});
		this.refreshDatePicker();
	};
	
	/**
	 * @constructor
	 */
	function CInformationViewModel()
	{
		this.loadingMessage = ko.observable('');
		this.loadingVisible = ko.observable(false);
		this.reportMessage = ko.observable('');
		this.reportVisible = ko.observable(false);
		this.iReportTimeout = NaN;
		this.errorMessage = ko.observable('');
		this.errorNotHide = ko.observable(false);
		this.errorVisible = ko.observable(false);
		this.iErrorTimeout = -1;
		this.isHtmlError = ko.observable(false);
	}
	
	/**
	 * @param {string} sMessage
	 */
	CInformationViewModel.prototype.showLoading = function (sMessage)
	{
		if (sMessage && sMessage !== '')
		{
			this.loadingMessage(sMessage);
		}
		else
		{
			this.loadingMessage(Utils.i18n('MAIN/LOADING'));
		}
		this.loadingVisible(true);
	}
	;
	
	CInformationViewModel.prototype.hideLoading = function ()
	{
		this.loadingVisible(false);
	};
	
	/**
	 * Displays a message. Starts a timer for hiding.
	 * 
	 * @param {string} sMessage
	 * @param {number} iDelay
	 */
	CInformationViewModel.prototype.showReport = function (sMessage, iDelay)
	{
		var self = this;
		
		iDelay = iDelay || 5000;
		
		if (sMessage && sMessage !== '')
		{
			this.reportMessage(sMessage);
			this.reportVisible(true);
			if (!isNaN(this.iReportTimeout))
			{
				clearTimeout(this.iReportTimeout);
			}
			this.iReportTimeout = setTimeout(function () {
				self.reportVisible(false);
			}, iDelay);
		}
		else
		{
			this.reportVisible(false);
		}
	};
	
	/**
	 * Displays an error message. Starts a timer for hiding.
	 * 
	 * @param {string} sMessage
	 * @param {boolean=} bHtml = false
	 * @param {boolean=} bNotHide = false
	 */
	CInformationViewModel.prototype.showError = function (sMessage, bHtml, bNotHide)
	{
		var self = this;
		
		if (sMessage && sMessage !== '')
		{
			this.errorNotHide(!!bNotHide);
			this.errorMessage(sMessage);
			this.errorVisible(true);
			this.isHtmlError(bHtml);
			clearTimeout(this.iErrorTimeout);
			if (!bNotHide)
			{
				this.iErrorTimeout = setTimeout(function () {
					self.errorVisible(false);
				}, 5000);
			}
		}
		else
		{
			this.errorVisible(false);
		}
	};
	
	CInformationViewModel.prototype.hideError = function (sMessage, bHtml, bNotHide)
	{
		this.errorVisible(false);
	};
	
	
	/**
	 * @constructor
	 */
	function CScreens()
	{
		this.oScreens = {};
	
		this.currentScreen = ko.observable('');
	
		this.popupVisibility = ko.observable(false);
		
		this.informationScreen = ko.observable(null);
		
		this.popups = [];
	}
	
	CScreens.prototype.initScreens = function () {};
	CScreens.prototype.initLayout = function () {};
	
	CScreens.prototype.init = function ()
	{
		this.initScreens();
		
		this.initLayout();
		
		$('#pSevenContent').addClass('single_mode');
		
		_.defer(function () {
			if (!AppData.SingleMode)
			{
				$('#pSevenContent').removeClass('single_mode');
			}
		});
		
		this.informationScreen(this.showNormalScreen(Enums.Screens.Information));
	};
	
	/**
	 * @param {string} sScreen
	 * @param {?=} mParams
	 */
	CScreens.prototype.showCurrentScreen = function (sScreen, mParams)
	{
		if (this.currentScreen() !== sScreen)
		{
			this.hideCurrentScreen();
			this.currentScreen(sScreen);
		}
	
		this.showNormalScreen(sScreen, mParams);
	};
	
	CScreens.prototype.hideCurrentScreen = function ()
	{
		if (this.currentScreen().length > 0)
		{
			this.hideScreen(this.currentScreen());
		}
	};
	
	/**
	 * @param {string} sScreen
	 */
	CScreens.prototype.hideScreen = function (sScreen)
	{
		var
			sScreenId = sScreen,
			oScreen = this.oScreens[sScreenId]
		;
	
		if (typeof oScreen !== 'undefined' && oScreen.bInitialized)
		{
			oScreen.Model.hideViewModel();
		}
	};
	
	/**
	 * @param {string} sScreen
	 * @param {?=} mParams
	 * 
	 * @return Object
	 */
	CScreens.prototype.showNormalScreen = function (sScreen, mParams)
	{
		var
			sScreenId = sScreen,
			oScreen = this.oScreens[sScreenId]
		;
	
		if (oScreen)
		{
			oScreen.bInitialized = (typeof oScreen.bInitialized !== 'boolean') ? false : oScreen.bInitialized;
			if (!oScreen.bInitialized)
			{
				oScreen.Model = this.initViewModel(oScreen.Model, oScreen.TemplateName);
				oScreen.bInitialized = true;
			}
	
			oScreen.Model.showViewModel(mParams);
		}
		
		return oScreen ? oScreen.Model : null;
	};
	
	/**
	 * @param {?} CViewModel
	 * @param {string} sTemplateId
	 * 
	 * @return {Object}
	 */
	CScreens.prototype.initViewModel = function (CViewModel, sTemplateId)
	{
		var
			oViewModel = null,
			$viewModel = null
		;
	
		oViewModel = new CViewModel();
	
		$viewModel = $('div[data-view-model="' + sTemplateId + '"]')
			.attr('data-bind', 'template: {name: \'' + sTemplateId + '\'}')
			.hide();
	
		oViewModel.$viewModel = $viewModel;
		oViewModel.bShown = false;
		oViewModel.showViewModel = function (mParams)
		{
			this.$viewModel.show();
			if (typeof this.onRoute === 'function')
			{
				this.onRoute(mParams);
			}
			if (!this.bShown)
			{
				if (typeof this.onShow === 'function')
				{
					this.onShow(mParams);
				}
				if (AfterLogicApi.runPluginHook)
				{
					if (this.__name)
					{
						AfterLogicApi.runPluginHook('view-model-on-show', [this.__name, this]);
					}
				}
				
				this.bShown = true;
			}
		};
		oViewModel.hideViewModel = function ()
		{
			this.$viewModel.hide();
			if (typeof this.onHide === 'function')
			{
				this.onHide();
			}
			this.bShown = false;
		};
		ko.applyBindings(oViewModel, $viewModel[0]);
	
		if (typeof oViewModel.onApplyBindings === 'function')
		{
			oViewModel.onApplyBindings($viewModel);
		}
	
		return oViewModel;
	};
	
	/**
	 * @param {?} CPopupViewModel
	 * @param {Array=} aParameters
	 */
	CScreens.prototype.showPopup = function (CPopupViewModel, aParameters)
	{
		if (CPopupViewModel)
		{
			if (!CPopupViewModel.__builded)
			{
				var
					oViewModelDom = null,
					oViewModel = new CPopupViewModel(),
					sTemplate = oViewModel.popupTemplate ? oViewModel.popupTemplate() : ''
				;
	
				if ('' !== sTemplate)
				{
					oViewModelDom = $('div[data-view-model="' + sTemplate + '"]')
						.attr('data-bind', 'template: {name: \'' + sTemplate + '\'}')
						.removeClass('visible').hide();
	
					if (oViewModelDom && 1 === oViewModelDom.length)
					{
						oViewModel.visibility = ko.observable(false);
	
						CPopupViewModel.__builded = true;
						CPopupViewModel.__vm = oViewModel;
	
						oViewModel.$viewModel = oViewModelDom;
						CPopupViewModel.__dom = oViewModelDom;
	
						oViewModel.showViewModel = Utils.createCommand(oViewModel, function () {
							if (App && App.Screens)
							{
								App.Screens.showPopup(CPopupViewModel);
							}
						});
	
						oViewModel.closeCommand = Utils.createCommand(oViewModel, function () {
							if (App && App.Screens)
							{
								App.Screens.hidePopup(CPopupViewModel);
							}
						});
						
						ko.applyBindings(oViewModel, oViewModelDom[0]);
	
						Utils.delegateRun(oViewModel, 'onApplyBindings', [oViewModelDom]);
					}
				}
			}
	
			if (CPopupViewModel.__vm && CPopupViewModel.__dom)
			{
				CPopupViewModel.__dom.show();
				_.delay(function() {
					CPopupViewModel.__dom.addClass('visible');
				}, 50);
				CPopupViewModel.__vm.visibility(true);
	
				Utils.delegateRun(CPopupViewModel.__vm, 'onShow', aParameters);
				this.popupVisibility(true);
				
				this.popups.push(CPopupViewModel);
				
				this.keyupPopupBinded = _.bind(this.keyupPopup, this, CPopupViewModel.__vm);
				$(document).on('keyup', this.keyupPopupBinded);
			}
		}
	};
	
	/**
	 * @param {Object} oViewModel
	 * @param {Object} oEvent
	 */
	CScreens.prototype.keyupPopup = function (oViewModel, oEvent)
	{
		if (oEvent)
		{
			var iKeyCode = window.parseInt(oEvent.keyCode, 10);
			
			// Esc
			if (27 === iKeyCode)
			{
				oViewModel.closeCommand();
			}
	
			// Enter or Space
			if ((13 === iKeyCode || 32 === iKeyCode) && oViewModel.onEnterHandler)
			{
				oViewModel.onEnterHandler();
			}
		}
	};
	
	/**
	 * @param {?} CPopupViewModel
	 */
	CScreens.prototype.hidePopup = function (CPopupViewModel)
	{
		if (CPopupViewModel && CPopupViewModel.__vm && CPopupViewModel.__dom)
		{
			if (this.keyupPopupBinded)
			{
				$(document).off('keyup', this.keyupPopupBinded);
				this.keyupPopupBinded = undefined;
			}
			CPopupViewModel.__dom.removeClass('visible').hide();
	
			CPopupViewModel.__vm.visibility(false);
	
			Utils.delegateRun(CPopupViewModel.__vm, 'onHide');
			this.popupVisibility(false);
			
			this.popups = _.without(this.popups, CPopupViewModel);
		}
	};
	
	CScreens.prototype.hideAllPopup = function ()
	{
		_.each(this.popups, function (oPopup) {
			this.hidePopup(oPopup);
		}, this);
	};
	
	/**
	 * @param {string} sMessage
	 */
	CScreens.prototype.showLoading = function (sMessage)
	{
		if (this.informationScreen())
		{
			this.informationScreen().showLoading(sMessage);
		}
	};
	
	CScreens.prototype.hideLoading = function ()
	{
		if (this.informationScreen())
		{
			this.informationScreen().hideLoading();
		}
	};
	
	/**
	 * @param {string} sMessage
	 * @param {number} iDelay
	 */
	CScreens.prototype.showReport = function (sMessage, iDelay)
	{
		if (this.informationScreen())
		{
			this.informationScreen().showReport(sMessage, iDelay);
		}
	};
	
	/**
	 * @param {string} sMessage
	 * @param {boolean=} bHtml = false
	 * @param {boolean=} bNotHide = false
	 */
	CScreens.prototype.showError = function (sMessage, bHtml, bNotHide)
	{
		if (this.informationScreen())
		{
			this.informationScreen().showError(sMessage, bHtml, bNotHide);
		}
	};
	CScreens.prototype.hideError = function (sMessage, bHtml, bNotHide)
	{
		if (this.informationScreen())
		{
			this.informationScreen().hideError();
		}
	};
	CScreens.prototype.initScreens = function ()
	{
		this.oScreens[Enums.Screens.Information] = {
			'Model': CInformationViewModel,
			'TemplateName': 'Common_InformationViewModel'
		};
		this.oScreens[Enums.Screens.Calendar] = {
			'Model': CCalendarViewModel,
			'TemplateName': 'Calendar_CalendarViewModel'
		};
	};
	
	CScreens.prototype.initLayout = function ()
	{
		$('#pSevenContent').append($('#CalendarPubLayout').html());
	};
	
	
	/**
	 * @constructor
	 */
	function CApp()
	{
		this.browser = new CBrowser();
		
		this.Ajax = new CAjax();
		this.Screens = new CScreens();
		this.Api = new CApi();
		this.Storage = new CStorage();
		
		this.init();
	}
	
	/**
	 * @param {string=} sTitle
	 */
	CApp.prototype.setTitle = function (sTitle)
	{
		document.title = '.';
		document.title = sTitle || '';
	};
	
	CApp.prototype.init = function ()
	{
		AppData.User = new CUserSettingsModel();
		
		// other lines for Google Compiler
		AppData.Accounts = {
			currentId: function () {return 0;},
			defaultId: function () {return 0;},
			hasAccountWithId: function (iId) {return true;},
			getDefault: function () {}
		};
		
		AppData.App = {
			DateFormats: ''
		};
		
		App.MailCache = {
			executeCheckMail: function () {}
		};
		
		App.Routing = {
			buildHashFromArray: function () {},
			setHash: function () {}
		};
		
		App.Links = {
			composeWithToField: function () {}
		};
	};
	
	// proto
	CApp.prototype.tokenProblem = function ()
	{
		var
			sReloadFunc= 'window.location.reload(); return false;',
			sHtmlError = Utils.i18n('WARNING/TOKEN_PROBLEM_HTML', {'RELOAD_FUNC': sReloadFunc})
		;
		
		AppData.Auth = false;
		App.Api.showError(sHtmlError, true, true);
	};
	
	CApp.prototype.authProblem = function ()
	{
	};
	
	CApp.prototype.run = function ()
	{
		Utils.log('run', AppData);
		
		this.Screens.init();
		
		this.Screens.showCurrentScreen(Enums.Screens.Calendar);
	};
	
	App = new CApp();
	window.App = App;
	$(function () {
		App.run();
	}, 0);
	
	window.AfterLogicApi = AfterLogicApi;
	
	// export
	window.Enums = Enums;
	
	$html.removeClass('no-js').addClass('js');
	
	if ($html.hasClass('pdf'))
	{
		aViewMimeTypes.push('application/pdf');
		aViewMimeTypes.push('application/x-pdf');
	}
	
}(jQuery, window, ko, crossroads, hasher));
